package com.oas.modbusutilities.configuration;

import java.text.MessageFormat;
import java.util.logging.Logger;

import javax.baja.nre.annotations.NiagaraType;
import javax.baja.status.BStatus;
import javax.baja.sys.BComplex;
import javax.baja.sys.BComponent;
import javax.baja.sys.BDouble;
import javax.baja.sys.BLink;
import javax.baja.sys.Context;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.oas.modbusutilities.configuration.point.BAiPoint;
import com.oas.modbusutilities.enums.BOasAutobauding;
import com.oas.modbusutilities.enums.BOasBautrate;
import com.oas.modbusutilities.enums.BOasModuleCmd;
import com.oas.modbusutilities.enums.BOasPointCharacteristic;
import com.oas.modbusutilities.utils.AsyncLinkUtils;
import com.oas.modbusutilities.utils.LinkUtils;
import com.tridium.modbusAsync.BModbusAsyncDevice;
import com.tridium.modbusAsync.BModbusAsyncDeviceFolder;
import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegisters;


/**
 * BAiConfiguration
 */
@SuppressWarnings({
  "squid:S2629", // Invoke LOGs conditionally will bloat up the code
  "squid:S110" // Niagara Framework has more inherit lvl than 6
})
@NiagaraType
public class BAsyncAiConfiguration
  extends BAbstractAiConfiguration
  implements AsyncConfiguration
{
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.oas.modbusutilities.configuration.BAsyncAiConfiguration(2979906276)1.0$ @*/
/* Generated Wed Mar 25 14:10:17 CET 2020 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////

  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BAsyncAiConfiguration.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/


  private static final Logger LOG = Logger.getLogger("oasModbusUtilities");
  private static final String LOG_PREFIX = BAsyncAiConfiguration.class.getSimpleName();

  @Override
  public boolean isParentLegal(BComponent parent) {

    return (parent instanceof BModbusAsyncDeviceFolder) ||
           (parent instanceof BModbusAsyncDevice);
  }

  @Override
  public void doWriteConfiguration() {
    super.doWriteConfiguration();

    if (!isRunning()) {
      return;
    }
    if (isModuleTypeValid()) {
      writeCharacteristics();

    }
  }


  @Override
  public void doWriteModuleCmd(BOasModuleCmd cmd) {
    LOG.info(MessageFormat.format("{0}.doWriteModuleCmd({1}) write {2}", LOG_PREFIX, getDeviceName(), cmd));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getModuleCmdPreset(), this);
    AsyncBasicConfiguration.doWriteModuleCmd(getModuleCmdRegister(), getModuleCmdPreset(), cmd);
  }

  @Override
  protected void writeAutobinding(BOasAutobauding value) {
    LOG.info(MessageFormat.format("{0}.writeAutobinding({1}) write {2}", LOG_PREFIX, getDeviceName(), value));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getAutobaudingPreset(), this);
    AsyncBasicConfiguration.writeAutobinding(getAutobaudingPreset(), value);
  }

  @Override
  protected void writeBaudreate(BOasBautrate value) {
    LOG.info(MessageFormat.format("{0}.writeBaudreate({1}) write {2}", LOG_PREFIX, getDeviceName(), value));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getBaudratePreset(), this);
    AsyncBasicConfiguration.writeBaudreate(getBaudratePreset(), value);
  }

  @Override
  protected void writeTimeout(BDouble value) {
    LOG.info(MessageFormat.format("{0}.writeTimeout({1}) write {2}", LOG_PREFIX, getDeviceName(), value));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getTimeoutPreset(), this);
    AsyncBasicConfiguration.writeTimeout(getTimeoutPreset());
  }

  @Override
  protected void writeLedStatus() {
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getStatusLedPreset(), this);
    AsyncBasicConfiguration.writeLedStatus(getStatusLedRegister(), getStatusLedPreset(), statusLedOneValue, statusLedTwoValue);
  }

  @Override
  public BConfigurationStatus checkConfiguration() {

    BConfigurationStatus newStatus = super.checkConfiguration();

    if (getDeviceStatus().isDisabled() || getDeviceStatus().isDown()) {
      return BConfigurationStatus.offline;
    }

    if (!isRunning()) {
      LOG.finest(MessageFormat.format("{0}.checkConfiguration({1}) station is not running",
                                        LOG_PREFIX,  getDeviceName()));
      return BConfigurationStatus.offline;
    }



    if (isConfigurationValid()) {


      if (isCharacteristicModifiedOf(getAi01())) {
        LOG.warning(MessageFormat.format(
                                "{0}.checkConfiguration({1}) configuration missmatch: Characteristic 01 is {2} instead of {3}",
                                LOG_PREFIX, getDeviceDisplayName(null),
                                getAi01().getActiveCharacteristicValue(), getAi01().getSetPointCharacteristicValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeCharacteristicOf(getCharacteristic01Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) Characteristic 01 is up to date", LOG_PREFIX, getDeviceName()));
        setConfigGoodStatus();

      }

      if (isCharacteristicModifiedOf(getAi02())) {
        LOG.warning(MessageFormat.format(
                                  "{0}.checkConfiguration({1}) configuration missmatch: Characteristic 02 is {2} instead of {3}",
                                  LOG_PREFIX, getDeviceDisplayName(null),
                                  getAi02().getActiveCharacteristicValue(), getAi02().getSetPointCharacteristicValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeCharacteristicOf(getCharacteristic02Preset());

      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) Characteristic 02 is up to date", LOG_PREFIX, getDeviceName()));
        setConfigGoodStatus();

      }

      if (isCharacteristicModifiedOf(getAi03())) {
        LOG.warning(MessageFormat.format(
                                  "{0}.checkConfiguration({1}) configuration missmatch: Characteristic 03 is {2} instead of {3}",
                                  LOG_PREFIX, getDeviceDisplayName(null),
                                  getAi03().getActiveCharacteristicValue(), getAi03().getSetPointCharacteristicValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeCharacteristicOf(getCharacteristic03Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) Characteristic 03 is up to date", LOG_PREFIX, getDeviceName()));

      }

      if (isCharacteristicModifiedOf(getAi04())) {
        LOG.warning(MessageFormat.format(
                                  "{0}.checkConfiguration({1}) configuration missmatch: Characteristic 04 is {2} instead of {3}",
                                  LOG_PREFIX, getDeviceDisplayName(null),
                                  getAi04().getActiveCharacteristicValue(), getAi04().getSetPointCharacteristicValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeCharacteristicOf(getCharacteristic04Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) Characteristic 04 is up to date", LOG_PREFIX, getDeviceName()));

      }

      if (isCharacteristicModifiedOf(getAi05())) {
        LOG.warning(MessageFormat.format(
                                  "{0}.checkConfiguration({1}) configuration missmatch: Characteristic 05 is {2} instead of {3}",
                                  LOG_PREFIX, getDeviceDisplayName(null),
                                  getAi05().getActiveCharacteristicValue(), getAi05().getSetPointCharacteristicValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeCharacteristicOf(getCharacteristic05Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) Characteristic 05 is up to date", LOG_PREFIX, getDeviceName()));

      }

      if (isCharacteristicModifiedOf(getAi06())) {
        LOG.warning(MessageFormat.format(
                                  "{0}.checkConfiguration({1}) configuration missmatch: Characteristic 06 is {2} instead of {3}",
                                  LOG_PREFIX, getDeviceDisplayName(null),
                                  getAi06().getActiveCharacteristicValue(), getAi06().getSetPointCharacteristicValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeCharacteristicOf(getCharacteristic06Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) Characteristic 06 is up to date", LOG_PREFIX, getDeviceName()));

      }

      if (isCharacteristicModifiedOf(getAi07())) {
        LOG.warning(MessageFormat.format(
                                  "{0}.checkConfiguration({1}) configuration missmatch: Characteristic 07 is {2} instead of {3}",
                                  LOG_PREFIX, getDeviceDisplayName(null),
                                  getAi07().getActiveCharacteristicValue(), getAi07().getSetPointCharacteristicValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeCharacteristicOf(getCharacteristic07Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) Characteristic 07 is up to date", LOG_PREFIX, getDeviceName()));

      }

      if (isCharacteristicModifiedOf(getAi08())) {
        LOG.warning(MessageFormat.format(
                                  "{0}.checkConfiguration({1}) configuration missmatch: Characteristic 08 is {2} instead of {3}",
                                  LOG_PREFIX, getDeviceDisplayName(null),
                                  getAi08().getActiveCharacteristicValue(), getAi08().getSetPointCharacteristicValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeCharacteristicOf(getCharacteristic08Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) Characteristic 08 is up to date", LOG_PREFIX, getDeviceName()));

      }

    }
    else {
      LOG.warning(MessageFormat.format(
                  "{0}.checkConfiguration({1}) either ModuleType[{2} != {3}] or Baudrate[{4} != {5}] is modified",
                  LOG_PREFIX, getDeviceName(),
                  getActiveModuleType(), getSetPointModuleType(),
                  getActiveBaudrate(), getSetPointBaudrate()));
    }



    if (!newStatus.isOk()) {
      LOG.info(MessageFormat.format("{0}.checkConfiguration({1}) config status is {2}", LOG_PREFIX, getDeviceName(), newStatus));
    }

    return newStatus;

  }



  public boolean isCharacteristicModifiedOf(BAiPoint ai) {
    if (null != ai) {
      BOasPointCharacteristic active = ai.getActiveCharacteristicValue();
      BOasPointCharacteristic setPoint = ai.getSetPointCharacteristicValue();

      return active.getOrdinal() != setPoint.getOrdinal();
    }
    return false;
  }

  public void writeCharacteristicOf(BComponent preset) {
    if (null != preset) {
      LOG.finer(MessageFormat.format("{0}.updateActiveCharacteristic({1}) update {2}",
                                        LOG_PREFIX, getDeviceDisplayName(null), preset.getName()));

      if (preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters characteristicPreset = (BModbusClientPresetRegisters)preset;
        AsyncBasicConfiguration.checkPresetWriteStatus(characteristicPreset, this);

        characteristicPreset.doWrite();
      }
      else {
        LOG.warning(MessageFormat.format("{0}.writeCharacteristicOf({1}) parameter is no BModbusClientPresetRegisters", LOG_PREFIX, getDeviceName()));
      }
    }
  }

  public void writeCharacteristics() {
    if (isCharacteristicModifiedOf(getAi01())) {
      LOG.fine(MessageFormat.format("{0}.writeCharacteristics({1}) write AO01 Characteristic", LOG_PREFIX, getDeviceName()));
      writeCharacteristicOf(getCharacteristic01Preset());
    }
    else {
      LOG.finer(MessageFormat.format("{0}.writeCharacteristics({1}) AO01 Characteristic is up to date", LOG_PREFIX, getDeviceName()));
    }

    if (isCharacteristicModifiedOf(getAi02())) {
      LOG.fine(MessageFormat.format("{0}.writeCharacteristics({1}) write AO02 Characteristic", LOG_PREFIX, getDeviceName()));
      writeCharacteristicOf(getCharacteristic02Preset());
    }
    else {
      LOG.finer(MessageFormat.format("{0}.writeCharacteristics({1}) AO02 Characteristic is up to date", LOG_PREFIX, getDeviceName()));
    }

    if (isCharacteristicModifiedOf(getAi03())) {
      LOG.fine(MessageFormat.format("{0}.writeCharacteristics({1}) write AO03 Characteristic", LOG_PREFIX, getDeviceName()));
      writeCharacteristicOf(getCharacteristic03Preset());
    }
    else {
      LOG.finer(MessageFormat.format("{0}.writeCharacteristics({1}) AO03 Characteristic is up to date", LOG_PREFIX, getDeviceName()));
    }

    if (isCharacteristicModifiedOf(getAi04())) {
      LOG.fine(MessageFormat.format("{0}.writeCharacteristics({1}) write AO04 Characteristic", LOG_PREFIX, getDeviceName()));
      writeCharacteristicOf(getCharacteristic04Preset());
    }
    else {
      LOG.finer(MessageFormat.format("{0}.writeCharacteristics({1}) AO04 Characteristic is up to date", LOG_PREFIX, getDeviceName()));
    }

    if (isCharacteristicModifiedOf(getAi05())) {
      LOG.fine(MessageFormat.format("{0}.writeCharacteristics({1}) write AO05 Characteristic", LOG_PREFIX, getDeviceName()));
      writeCharacteristicOf(getCharacteristic05Preset());
    }
    else {
      LOG.finer(MessageFormat.format("{0}.writeCharacteristics({1}) AO05 Characteristic is up to date", LOG_PREFIX, getDeviceName()));
    }

    if (isCharacteristicModifiedOf(getAi06())) {
      LOG.fine(MessageFormat.format("{0}.writeCharacteristics({1}) write AO06 Characteristic", LOG_PREFIX, getDeviceName()));
      writeCharacteristicOf(getCharacteristic06Preset());
    }
    else {
      LOG.finer(MessageFormat.format("{0}.writeCharacteristics({1}) AO06 Characteristic is up to date", LOG_PREFIX, getDeviceName()));
    }

    if (isCharacteristicModifiedOf(getAi07())) {
      LOG.fine(MessageFormat.format("{0}.writeCharacteristics({1}) write AO07 Characteristic", LOG_PREFIX, getDeviceName()));
      writeCharacteristicOf(getCharacteristic07Preset());
    }
    else {
      LOG.finer(MessageFormat.format("{0}.writeCharacteristics({1}) AO07 Characteristic is up to date", LOG_PREFIX, getDeviceName()));
    }

    if (isCharacteristicModifiedOf(getAi08())) {
      LOG.fine(MessageFormat.format("{0}.writeCharacteristics({1}) write AO08 Characteristic", LOG_PREFIX, getDeviceName()));
      writeCharacteristicOf(getCharacteristic08Preset());
    }
    else {
      LOG.finer(MessageFormat.format("{0}.writeCharacteristics({1}) AO02 Characteristic is up to date", LOG_PREFIX, getDeviceName()));
    }
  }


  @Override
  public void createLinksToRegister() {

    AsyncBasicConfiguration.createLinksToRegister(getDevice(), this);


    LOG.finer(MessageFormat.format("{0}.createLinksToRegister({1}) {2}.{3}:create Links to Config Component",
                                    LOG_PREFIX, getDeviceDisplayName(null), getDeviceName(), this.getName()));


    BComponent characteristic01 = getActiveCharacteristic01Register();
    if (null != characteristic01) {
      linkRegisterToPoint(characteristic01, getAi01());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic01",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent characteristic01Preset = getCharacteristic01Preset();
    if (null != characteristic01Preset) {
      if (characteristic01Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)characteristic01Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAi01(), "setPointCharacteristicValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic01Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent input01 = getInput01Register();
    if (null != input01) {
      linkInputToPoint(input01, getAi01());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Input01",
              LOG_PREFIX, getDeviceDisplayName(null)));
    }


    BComponent characteristic02 = getActiveCharacteristic02Register();
    if (null != characteristic02) {
      linkRegisterToPoint(characteristic02, getAi02());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic02",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent characteristic02Preset = getCharacteristic02Preset();
    if (null != characteristic02Preset) {
      if (characteristic02Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)characteristic02Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAi02(), "setPointCharacteristicValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic02Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }
    BComponent input02 = getInput02Register();
    if (null != input02) {
      linkInputToPoint(input02, getAi02());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Input02",
              LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent characteristic03 = getActiveCharacteristic03Register();
    if (null != characteristic03) {
      linkRegisterToPoint(characteristic03, getAi03());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic03",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent characteristic03Preset = getCharacteristic03Preset();
    if (null != characteristic03Preset) {
      if (characteristic03Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)characteristic03Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAi03(), "setPointCharacteristicValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic03Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent input03 = getInput03Register();
    if (null != input03) {
      linkInputToPoint(input03, getAi03());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Input03",
              LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent characteristic04 = getActiveCharacteristic04Register();
    if (null != characteristic04) {
      linkRegisterToPoint(characteristic04, getAi04());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic04",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }


    BComponent characteristic04Preset = getCharacteristic04Preset();
    if (null != characteristic04Preset) {
      if (characteristic04Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)characteristic04Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAi04(), "setPointCharacteristicValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic04Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }
    BComponent input04 = getInput04Register();
    if (null != input04) {
      linkInputToPoint(input04, getAi04());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Input04",
              LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent characteristic05 = getActiveCharacteristic05Register();
    if (null != characteristic05) {
      linkRegisterToPoint(characteristic05, getAi05());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic05",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }


    BComponent characteristic05Preset = getCharacteristic05Preset();
    if (null != characteristic05Preset) {
      if (characteristic05Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)characteristic05Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAi05(), "setPointCharacteristicValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic05Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }
    BComponent input05 = getInput05Register();
    if (null != input05) {
      linkInputToPoint(input05, getAi05());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Input05",
              LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent characteristic06 = getActiveCharacteristic06Register();
    if (null != characteristic06) {
      linkRegisterToPoint(characteristic06, getAi06());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic06",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent characteristic06Preset = getCharacteristic06Preset();
    if (null != characteristic06Preset) {
      if (characteristic06Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)characteristic06Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAi06(), "setPointCharacteristicValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic06Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }
    BComponent input06 = getInput06Register();
    if (null != input06) {
      linkInputToPoint(input06, getAi06());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Input06",
              LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent characteristic07 = getActiveCharacteristic07Register();
    if (null != characteristic07) {
      linkRegisterToPoint(characteristic07, getAi07());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic07",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent characteristic07Preset = getCharacteristic07Preset();
    if (null != characteristic07Preset) {
      if (characteristic07Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)characteristic07Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAi07(), "setPointCharacteristicValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic07Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }
    BComponent input07 = getInput07Register();
    if (null != input07) {
      linkInputToPoint(input07, getAi07());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Input07",
              LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent characteristic08 = getActiveCharacteristic08Register();
    if (null != characteristic08) {
      linkRegisterToPoint(characteristic08, getAi08());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic08",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent characteristic08Preset = getCharacteristic08Preset();
    if (null != characteristic08Preset) {
      if (characteristic08Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)characteristic08Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAi08(), "setPointCharacteristicValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Characteristic08Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }
    BComponent input08 = getInput08Register();
    if (null != input08) {
      linkInputToPoint(input08, getAi08());
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Input08",
              LOG_PREFIX, getDeviceDisplayName(null)));
    }


  }

  private void linkRegisterToPoint(BComponent register, BAiPoint ai) {

    Property srcProp = register.getProperty("out");
    Property destProp = ai.getProperty("activeCharacteristicValue");
    BLink activeCharacteristicLink = LinkUtils.createLink(register, srcProp, ai, destProp);
    ai.add("activeCharacteristicLink", activeCharacteristicLink);
  }

  private void linkInputToPoint(BComponent register, BAiPoint ai) {

    Property srcProp = register.getProperty("out");
    Property destProp = ai.getProperty("activeRawInput");
    BLink activeRawInputLink = LinkUtils.createLink(register, srcProp, ai, destProp);
    ai.add("activeRawInputLink", activeRawInputLink);
  }



  public BComponent getAutobaudingPreset() {
    return getRegisterByName("autobaudingPreset");
  }

  public BComponent getBaudratePreset() {
    return getRegisterByName("baudratePreset");
  }

  public BComponent getTimeoutPreset() {
    return getRegisterByName("timeoutSecPreset");
  }

  public BComponent getStatusLedPreset() {
    return getRegisterByName("statusLedPreset");
  }

  public BComponent getModuleCmdPreset() {
    return getRegisterByName("moduleCmdPreset");
  }


  private BComponent getInput01Register() {
    return getRegisterByName("Input01");
  }
  private BComponent getInput02Register() {
    return getRegisterByName("Input02");
  }
  private BComponent getInput03Register() {
    return getRegisterByName("Input03");
  }
  private BComponent getInput04Register() {
    return getRegisterByName("Input04");
  }
  private BComponent getInput05Register() {
    return getRegisterByName("Input05");
  }
  private BComponent getInput06Register() {
    return getRegisterByName("Input06");
  }
  private BComponent getInput07Register() {
    return getRegisterByName("Input07");
  }
  private BComponent getInput08Register() {
    return getRegisterByName("Input08");
  }

  private BComponent getActiveCharacteristic01Register() {
    return getRegisterByName("Characteristic01");
  }

  private BComponent getActiveCharacteristic02Register() {
    return getRegisterByName("Characteristic02");
  }

  private BComponent getActiveCharacteristic03Register() {
    return getRegisterByName("Characteristic03");
  }

  private BComponent getActiveCharacteristic04Register() {
    return getRegisterByName("Characteristic04");
  }

  private BComponent getActiveCharacteristic05Register() {
    return getRegisterByName("Characteristic05");
  }

  private BComponent getActiveCharacteristic06Register() {
    return getRegisterByName("Characteristic06");
  }

  private BComponent getActiveCharacteristic07Register() {
    return getRegisterByName("Characteristic07");
  }

  private BComponent getActiveCharacteristic08Register() {
    return getRegisterByName("Characteristic08");
  }

  private BComponent getCharacteristic01Preset() {
    return getRegisterByName("Characteristic01Preset");
  }

  private BComponent getCharacteristic02Preset() {
    return getRegisterByName("Characteristic02Preset");
  }

  private BComponent getCharacteristic03Preset() {
    return getRegisterByName("Characteristic03Preset");
  }

  private BComponent getCharacteristic04Preset() {
    return getRegisterByName("Characteristic04Preset");
  }

  private BComponent getCharacteristic05Preset() {
    return getRegisterByName("Characteristic05Preset");
  }

  private BComponent getCharacteristic06Preset() {
    return getRegisterByName("Characteristic06Preset");
  }

  private BComponent getCharacteristic07Preset() {
    return getRegisterByName("Characteristic07Preset");
  }

  private BComponent getCharacteristic08Preset() {
    return getRegisterByName("Characteristic08Preset");
  }

  public BModbusAsyncDevice getDevice() {
    BComplex parent = getParent();
    if (parent instanceof BComponent) {
      return (BModbusAsyncDevice)parent;
    }
    return null;
  }

  protected BStatus getDeviceStatus() {

    return AsyncBasicConfiguration.getDeviceStatus(getDevice());
  }

  protected boolean isDeviceEnabeld() {

    return AsyncBasicConfiguration.isDeviceEnabeld(getDevice());
  }

  protected boolean isDeviceOk() {

    return AsyncBasicConfiguration.isDeviceOk(getDevice());
  }

  public String getDeviceName() {

    return AsyncBasicConfiguration.getDeviceName(getDevice());
  }

  @Override
  public String getDeviceDisplayName(Context ctx) {

    return AsyncBasicConfiguration.getDeviceDisplayName(getDevice(), ctx);
  }

  @Override
  protected BComponent[] getRegister() {

    return AsyncBasicConfiguration.getRegister(getDevice());

  }

}
