package com.oas.modbusutilities.configuration;

import java.text.MessageFormat;
import java.util.logging.Logger;

import javax.baja.nre.annotations.NiagaraType;
import javax.baja.status.BStatus;
import javax.baja.status.BStatusBoolean;
import javax.baja.status.BStatusNumeric;
import javax.baja.sys.BComplex;
import javax.baja.sys.BComponent;
import javax.baja.sys.BDouble;
import javax.baja.sys.BLink;
import javax.baja.sys.Context;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.oas.modbusutilities.configuration.point.BAo3Point;
import com.oas.modbusutilities.enums.BOasAutobauding;
import com.oas.modbusutilities.enums.BOasBautrate;
import com.oas.modbusutilities.enums.BOasLedColor;
import com.oas.modbusutilities.enums.BOasModuleCmd;
import com.oas.modbusutilities.enums.BOasShutterStatus;
import com.oas.modbusutilities.utils.AsyncLinkUtils;
import com.oas.modbusutilities.utils.LinkUtils;
import com.tridium.modbusAsync.BModbusAsyncDevice;
import com.tridium.modbusAsync.BModbusAsyncDeviceFolder;
import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegister;
import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegisters;

@SuppressWarnings({
    "squid:S2629", // Invoke LOGs conditionally will bloat up the code
    "squid:S110" // Niagara Framework has more inherit lvl than 6
  })
@NiagaraType
public class BAsyncAo3Configuration
        extends BAbstractAo3Configuration
        implements AsyncConfiguration
{
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.oas.modbusutilities.configuration.BAsyncAo3Configuration(2979906276)1.0$ @*/
/* Generated Mon Jun 08 16:15:23 CEST 2020 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////

  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BAsyncAo3Configuration.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

    private static final Logger LOG = Logger.getLogger("oasModbusUtilities");
    private static final String LOG_PREFIX = BAsyncAo3Configuration.class.getSimpleName();

    @Override
    public boolean isParentLegal(BComponent parent) {

        return (parent instanceof BModbusAsyncDeviceFolder) ||
                (parent instanceof BModbusAsyncDevice);
    }

    @Override
    public void changed(Property property, Context context) {
        super.changed(property, context);

        if (!isRunning()) {
            return;
        }

        if (property.equals(activeOnSideSwitchValue)
          || property.equals(activeOnSideSwitchStatus)) {
            updateActiveOnSideSwitchValue();
            int value = (int)getActiveOnSideSwitchValue().getValue();
            int status = (int)getActiveOnSideSwitchStatus().getValue();
            if (isOnSideSwitchConfigured(value, status)) {

              setOnSideSwitchStatusOn();
            }
            else {
              setOnSideSwitchStatusOff();
            }
          }
          else if (property.equals(Input)) {
            updateInputValues();
          }
          else if (property.equals(activeLedGreenValue) || property.equals(activeLedRedValue)) {
            check();
          }
          else if (property.equals(activeBounceProtectionValue)) {
            check();
          }
          else if (property.equals(activeSwitchLogicValue)) {
            check();
          }
          else if (property.equals(maxWriteTime)) {
            BModbusAsyncDevice device = getDevice();
            if (device.getEnabled()) {
              restartMaxSendTicket();
            }
        }
    }

    @Override
    public void descendantsStarted() throws Exception {
        super.descendantsStarted();
        updateSetPointLedColors();
        updateBounceProtection();
        updateSwitchLogic();
    }

    @Override
    public void doWriteConfiguration() {
        super.doWriteConfiguration();
        if (!isRunning()) {
            return;
        }

        if (isModuleTypeValid()) {

            writeRedLedValue();
            writeGreenLedValue();
            writeSwitchLogicValue();
            writeBounceProtectionValue();
        }
    }

    @Override
    public void doWriteModuleCmd(BOasModuleCmd cmd) {
        LOG.info(MessageFormat.format("{0}.doWriteModuleCmd(): upate to {1}", LOG_PREFIX, cmd));
        AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getModuleCmdPreset(), this);
        AsyncBasicConfiguration.doWriteModuleCmd(getModuleCmdRegister(), getModuleCmdPreset(), cmd);
    }

    @Override
    protected void writeAutobinding(BOasAutobauding value) {
        LOG.info(MessageFormat.format("{0}.writeAutobinding(): upate to {1}", LOG_PREFIX, value));
        AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getAutobaudingPreset(), this);
        AsyncBasicConfiguration.writeAutobinding(getAutobaudingPreset(), value);
    }

    @Override
    protected void writeBaudreate(BOasBautrate value) {
        LOG.info(MessageFormat.format("{0}.writeBaudreate(): upate to {1}", LOG_PREFIX, value));
        AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getBaudratePreset(), this);
        AsyncBasicConfiguration.writeBaudreate(getBaudratePreset(), value);
    }

    @Override
    protected void writeTimeout(BDouble value) {
        LOG.info(MessageFormat.format("{0}.writeTimeout(): upate to {1}", LOG_PREFIX, value.getDouble()));
        AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getTimeoutPreset(), this);
        AsyncBasicConfiguration.writeTimeout(getTimeoutPreset());
    }

    protected void writeLedStatus() {
        AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getStatusLedPreset(), this);
        AsyncBasicConfiguration.writeLedStatus(getStatusLedRegister(), getStatusLedPreset(), statusLedOneValue, statusLedTwoValue);

    }


    @Override
    public BConfigurationStatus checkConfiguration() {
      BConfigurationStatus newStatus = super.checkConfiguration();

      if (getDeviceStatus().isDisabled() || getDeviceStatus().isDown()) {
        return BConfigurationStatus.offline;
      }


      if (!isRunning()) {
        LOG.finest(MessageFormat.format("{0}.checkConfiguration({1}) station is not running",
                                          LOG_PREFIX,  getDeviceName()));
        return BConfigurationStatus.offline;
      }

      if (isConfigurationValid()) {

        if (isRedLedValueModified()) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: RedLed is {2} instead of {3}",
                                          LOG_PREFIX, getDeviceName(),
                                          getActiveLedRedValue(), getSetPointLedRedValue()));
          setConfigModifiedStatus();
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeRedLedValue();
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) RedLed is up to date", LOG_PREFIX, getDeviceName()));
        }

        if (isGreenLedValueModified()) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: GreenLed is {2} instead of {3}",
                                            LOG_PREFIX, getDeviceName(),
                                            getActiveLedGreenValue(), getSetPointLedGreenValue()));
          setConfigModifiedStatus();
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeGreenLedValue();
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) GreenLed is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isSwitchLogicValueModified()) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: Switch Logic is {2} instead of {3}",
                                            LOG_PREFIX, getDeviceName(),
                                            getActiveSwitchLogicValue(), getSetPointSwitchLogicValue()));
          setConfigModifiedStatus();
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeSwitchLogicValue();
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) SwitchLogic is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isBounceProtValueModified()) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: Bounce Prot is {2} instead of {3}",
                                            LOG_PREFIX, getDeviceName(),
                                            getActiveBounceProtectionValue(), getSetPointBounceProtectionValue()));
          setConfigModifiedStatus();
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeBounceProtectionValue();
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) BounceProt is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isRuntimeUpModifiedOf(getAo3p01())) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: runtimeUp01 is {2} instead of {3}",
                                              LOG_PREFIX, getDeviceName(),
                                              getAo3p01().getActiveRuntimeUp(), getAo3p01().getSetPointRuntimeUp()));
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeRuntimeUp(getUpTime01Presets());
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) runtimeUp01 is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isRuntimeUpModifiedOf(getAo3p02())) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: runtimeUp02 is {2} instead of {3}",
                                              LOG_PREFIX, getDeviceName(),
                                              getAo3p02().getActiveRuntimeUp(), getAo3p02().getSetPointRuntimeUp()));
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeRuntimeUp(getUpTime02Presets());
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) runtimeUp02 is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isRuntimeDownModifiedOf(getAo3p01())) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: runtimeDown01 is {2} instead of {3}",
                                            LOG_PREFIX, getDeviceName(),
                                            getAo3p01().getActiveRuntimeDown(), getAo3p01().getSetPointRuntimeDown()));
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeRuntimeDown(getDownTime01Presets());
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) runtimeDown01 is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isRuntimeDownModifiedOf(getAo3p02())) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: runtimeDown02 is {2} instead of {3}",
                                            LOG_PREFIX, getDeviceName(),
                                            getAo3p02().getActiveRuntimeDown(), getAo3p02().getSetPointRuntimeDown()));
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeRuntimeDown(getDownTime02Presets());
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) runtimeDown02 is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isOnOffDelayModifiedOf(getAo3p01())) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: onOffDelay01 is {2} instead of {3}",
                                            LOG_PREFIX, getDeviceName(),
                                            getAo3p01().getActiveOnOffDelay(), getAo3p01().getSetPointOnOffDelay()));
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeOnOffDelay(getOnOffDelay01Presets());
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) onOffDelay01 is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isOnOffDelayModifiedOf(getAo3p02())) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: onOffDelay02 is {2} instead of {3}",
                                            LOG_PREFIX, getDeviceName(),
                                            getAo3p02().getActiveOnOffDelay(), getAo3p02().getSetPointOnOffDelay()));
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeOnOffDelay(getOnOffDelay02Presets());
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) onOffDelay02 is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isChangeOverDelayModifiedOf(getAo3p01())) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: changeOverDelay01 is {2} instead of {3}",
                                        LOG_PREFIX, getDeviceName(),
                                        getAo3p01().getActiveChangeOverDelay(), getAo3p01().getSetPointChangeOverDelay()));
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeChangeOverDelay(getChangeOverDelay01Presets());
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) changeOverDelay01 is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isChangeOverDelayModifiedOf(getAo3p02())) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: changeOverDelay02 is {2} instead of {3}",
                                        LOG_PREFIX, getDeviceName(),
                                        getAo3p02().getActiveChangeOverDelay(), getAo3p02().getSetPointChangeOverDelay()));
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeChangeOverDelay(getChangeOverDelay02Presets());
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) changeOverDelay02 is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isOverrunUpTimeModifiedOf(getAo3p01())) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: overrunUp01 is {2} instead of {3}",
                                            LOG_PREFIX, getDeviceName(),
                                            getAo3p01().getActiveChangeOverDelay(), getAo3p01().getSetPointChangeOverDelay()));
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeOverrunUpTime(getOverrunUpTime01Presets());
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) overrunUp01 is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isOverrunUpTimeModifiedOf(getAo3p02())) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: overrunUp02 is {2} instead of {3}",
                                            LOG_PREFIX, getDeviceName(),
                                            getAo3p02().getActiveOverrunUpTime(), getAo3p02().getSetPointOverrunUpTime()));
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeOverrunUpTime(getOverrunUpTime02Presets());
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) overrunUp01 is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isOverrunDownTimeModifiedOf(getAo3p01())) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: overrunDown01 is {2} instead of {3}",
                                            LOG_PREFIX, getDeviceName(),
                                            getAo3p01().getActiveOverrunUpTime(), getAo3p01().getSetPointOverrunUpTime()));
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeOverrunDownTime(getOverrunDownTime01Presets());
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) overrunDown01 is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

        if (isOverrunDownTimeModifiedOf(getAo3p02())) {
          LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: overrunDown02 is {2} instead of {3}",
                                            LOG_PREFIX, getDeviceName(),
                                            getAo3p02().getActiveOverrunUpTime(), getAo3p02().getSetPointOverrunUpTime()));
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeOverrunDownTime(getOverrunDownTime02Presets());
        }
        else {
          LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) overrunDown02 is up to date",
                                        LOG_PREFIX, getDeviceName()));
        }

      }
      else {
        LOG.warning(MessageFormat.format(
                    "{0}.checkConfiguration({1}) either ModuleType[{2} != {3}] or Baudrate[{4} != {5}] is modified",
                    LOG_PREFIX, getDeviceName(),
                    getActiveModuleType(), getSetPointModuleType(),
                    getActiveBaudrate(), getSetPointBaudrate()));
      }

      return newStatus;
    }

    @Override
    public void createLinksToRegister() {

        AsyncBasicConfiguration.createLinksToRegister(getDevice(), this);


        LOG.info(MessageFormat.format("{0}.createLinksToRegister({1}) {2}.{3}:create Links to Config Component",
                                        LOG_PREFIX, getDeviceDisplayName(null), getDeviceName(), this.getName()));


        BComponent ledRed = getRedLedValueRegister();
        if (null != ledRed) {
            Property srcProp = ledRed.getProperty("out");
            BLink activeLedRedValueLink = LinkUtils.createLink(ledRed, srcProp, this, activeLedRedValue);
            add("activeLedRedValueLink", activeLedRedValueLink);
        }
        else {
            LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get ActiveLedRedRegister",
                                            LOG_PREFIX, getDeviceDisplayName(null)));
        }


        BComponent ledRedPreset = getLedRedValuePreset();
        if (ledRedPreset instanceof BModbusClientPresetRegisters) {
            BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)ledRedPreset;
            AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "setPointLedRedValue");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get LedRedValuePreset",
                                            LOG_PREFIX, getDeviceDisplayName(null)));
        }


        BComponent ledGreen = getGreenLedValueRegister();
        if (null != ledGreen) {
          Property srcProp = ledGreen.getProperty("out");
          BLink activeLedGreenValueLink = LinkUtils.createLink(ledGreen, srcProp, this, activeLedGreenValue);
          add("activeLedGreenValueLink", activeLedGreenValueLink);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get ActiveLedGreenValueRegister",
                                            LOG_PREFIX, getDeviceDisplayName(null)));
        }


        BComponent ledGreenPreset = getLedGreenValuePreset();
        if (ledGreenPreset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)ledGreenPreset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "setPointLedGreenValue");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get LedGreenValuePreset",
                                            LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent switchLogic = getSwitchLogicValueRegister();
        if (null != switchLogic) {
          Property srcProp = switchLogic.getProperty("out");
          BLink activeSwitchLogicValueLink = LinkUtils.createLink(switchLogic, srcProp, this, activeSwitchLogicValue);
          add("activeSwitchLogicValueLink", activeSwitchLogicValueLink);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get ActiveSwitchLogicValueRegister",
                                            LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent switchLogicPreset = getSwitchLogicValuePreset();
        if (switchLogicPreset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)switchLogicPreset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "setPointSwitchLogicValue");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SwitchLogicValuePreset",
                                            LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent bouncProt = getBounceProtectionValueRegister();
        if (null != bouncProt) {
          Property srcProp = bouncProt.getProperty("out");
          BLink activeBounceProtectionValueLink = LinkUtils.createLink(bouncProt, srcProp, this, activeBounceProtectionValue);
          add("activeBounceProtectionValueLink", activeBounceProtectionValueLink);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get ActiveBounceProtectionValueRegister",
                                            LOG_PREFIX, getDeviceDisplayName(null)));
        }


        BComponent bouncProtPreset = getBounceProtectionValuePreset();
        if (bouncProtPreset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)bouncProtPreset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "setPointBounceProtectionValue");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get BounceProtectionValuePreset",
                                            LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent input = getInputRegister();
        if (null != input) {
          Property srcProp = input.getProperty("out");
          BLink inputLink = LinkUtils.createLink(input, srcProp, this, Input);
          add("inputLink", inputLink);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get InputRegister",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent onSideSwitchStatus = getOnSideSwitchStatusRegister();
        if (null != onSideSwitchStatus) {
          Property srcProp = onSideSwitchStatus.getProperty("out");
          BLink onSideSwitchLink = LinkUtils.createLink(onSideSwitchStatus, srcProp, this, activeOnSideSwitchStatus);
          add("activeonSideSwitchLink", onSideSwitchLink);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get onSideSwitchStatusRegister",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }


        BComponent onSideSwitchValue = getOnSideSwitchValueRegister();
        if (null != onSideSwitchValue) {
          Property srcProp = onSideSwitchValue.getProperty("out");
          BLink onSideSwitchCfgLink = LinkUtils.createLink(onSideSwitchValue, srcProp, this, activeOnSideSwitchValue);
          add("onSideSwitchCfgLink", onSideSwitchCfgLink);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get onSideSwitchValueRegister",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }


        BComponent feedback01 = getFeedback01Register();
        if (null != feedback01) {
          Property srcProp = feedback01.getProperty("out");
          Property destProp = getAo3p01().getProperty("feedbackValue");
          BLink feedback01Link = LinkUtils.createLink(feedback01, srcProp, getAo3p01(), destProp);
          getAo3p01().add("feedback01Link", feedback01Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Feedback01Register",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent feedback02 = getFeedback02Register();
        if (null != feedback02) {
          Property srcProp = feedback02.getProperty("out");
          Property destProp = getAo3p02().getProperty("feedbackValue");
          BLink feedback02Link = LinkUtils.createLink(feedback02, srcProp, getAo3p02(), destProp);
          getAo3p02().add("feedback02Link", feedback02Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get Feedback02Register",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }


        BComponent automaticCmd01 = getAutomaticCmd01Preset();
        if (automaticCmd01 instanceof BModbusClientPresetRegister) {
          BModbusClientPresetRegister preset = (BModbusClientPresetRegister)automaticCmd01;
          AsyncLinkUtils.linkToPreset(preset, (BComponent)getAo3p01(), "automaticCmd");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get automaticCmd01Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent automaticCmd02 = getAutomaticCmd02Preset();
        if (automaticCmd02 instanceof BModbusClientPresetRegister) {
          BModbusClientPresetRegister preset = (BModbusClientPresetRegister)automaticCmd02;
          AsyncLinkUtils.linkToPreset(preset, (BComponent)getAo3p02(), "automaticCmd");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get automaticCmd02Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent upTime01Preset = getUpTime01Presets();
        if (upTime01Preset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)upTime01Preset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo3p01(), "setPointRuntimeUp");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get UpTime01Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent upTime01 = getUpTime01Register();
        if (null != upTime01) {
          Property srcProp = upTime01.getProperty("out");
          Property destProp = getAo3p01().getProperty("activeRuntimeUp");
          BLink activeUpTime01Link = LinkUtils.createLink(upTime01, srcProp, getAo3p01(), destProp);
          getAo3p01().add("activeUpTime01Link", activeUpTime01Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get UpTime01Register",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent upTime02Preset = getUpTime02Presets();
        if (upTime02Preset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)upTime02Preset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo3p02(), "setPointRuntimeUp");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get UpTime02Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent upTime02 = getUpTime02Register();
        if (null != upTime02) {
          Property srcProp = upTime02.getProperty("out");
          Property destProp = getAo3p02().getProperty("activeRuntimeUp");
          BLink activeUpTime02Link = LinkUtils.createLink(upTime02, srcProp, getAo3p02(), destProp);
          getAo3p02().add("activeUpTime02Link", activeUpTime02Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get UpTime01Register",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent downTime01Preset = getDownTime01Presets();
        if (downTime01Preset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)downTime01Preset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo3p01(), "setPointRuntimeDown");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get DownTime01Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent downTime01 = getDownTime01Register();
        if (null != downTime01) {
          Property srcProp = downTime01.getProperty("out");
          Property destProp = getAo3p01().getProperty("activeRuntimeDown");
          BLink activeDownTime01Link = LinkUtils.createLink(downTime01, srcProp, getAo3p01(), destProp);
          getAo3p01().add("activeDownTime01Link", activeDownTime01Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get DownTime01Register",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent downTime02Preset = getDownTime02Presets();
        if (downTime02Preset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)downTime02Preset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo3p02(), "setPointRuntimeDown");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get DownTime02Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent downTime02 = getDownTime02Register();
        if (null != downTime02) {
          Property srcProp = downTime02.getProperty("out");
          Property destProp = getAo3p02().getProperty("activeRuntimeDown");
          BLink activeDownTime02Link = LinkUtils.createLink(downTime02, srcProp, getAo3p02(), destProp);
          getAo3p02().add("activeDownTime02Link", activeDownTime02Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get DownTime01Register",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent onOffDelay01Preset = getOnOffDelay01Presets();
        if (onOffDelay01Preset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)onOffDelay01Preset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo3p01(), "setPointOnOffDelay");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get onOffDelay01Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent downUp01 = getOnOffDelay01Register();
        if (null != downUp01) {
          Property srcProp = downUp01.getProperty("out");
          Property destProp = getAo3p01().getProperty("activeOnOffDelay");
          BLink activedownUp01Link = LinkUtils.createLink(downUp01, srcProp, getAo3p01(), destProp);
          getAo3p01().add("activeonOffDelay01Link", activedownUp01Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get downUp01Register",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent onOffDelay02Preset = getOnOffDelay02Presets();
        if (onOffDelay02Preset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)onOffDelay02Preset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo3p02(), "setPointOnOffDelay");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get onOffDelay02Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent downUp02 = getOnOffDelay02Register();
        if (null != downUp02) {
          Property srcProp = downUp02.getProperty("out");
          Property destProp = getAo3p02().getProperty("activeOnOffDelay");
          BLink activeDownUp02Link = LinkUtils.createLink(downUp02, srcProp, getAo3p02(), destProp);
          getAo3p02().add("activeDownUp02Link", activeDownUp02Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get downUp02Register",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }


        BComponent changeOverDelay01Preset = getChangeOverDelay01Presets();
        if (changeOverDelay01Preset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)changeOverDelay01Preset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo3p01(), "setPointChangeOverDelay");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get changeOverDelay01Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent upDown01 = getChangeOverDelay01Register();
        if (null != upDown01) {
          Property srcProp = upDown01.getProperty("out");
          Property destProp = getAo3p01().getProperty("activeChangeOverDelay");
          BLink activeChangeOverDelayn01Link = LinkUtils.createLink(upDown01, srcProp, getAo3p01(), destProp);
          getAo3p01().add("activeChangeOverDelay01Link", activeChangeOverDelayn01Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get changeOverDelay01Register",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent changeOverDelay02Preset = getChangeOverDelay02Presets();
        if (changeOverDelay02Preset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)changeOverDelay02Preset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo3p02(), "setPointChangeOverDelay");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get changeOverDelay02Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent upDown02 = getChangeOverDelay02Register();
        if (null != upDown02) {
          Property srcProp = upDown02.getProperty("out");
          Property destProp = getAo3p02().getProperty("activeChangeOverDelay");
          BLink activeChangeOverDelay02Link = LinkUtils.createLink(upDown02, srcProp, getAo3p02(), destProp);
          getAo3p02().add("activeUpDown02Link", activeChangeOverDelay02Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get changeOverDelay02Register",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent overrunUpTime01Preset = getOverrunUpTime01Presets();
        if (overrunUpTime01Preset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)overrunUpTime01Preset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo3p01(), "setPointOverrunUpTime");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get overrunUpTime01Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent overrunUpTime01 = getoverrunUpTime01Register();
        if (null != overrunUpTime01) {
          Property srcProp = overrunUpTime01.getProperty("out");
          Property destProp = getAo3p01().getProperty("activeOverrunUpTime");
          BLink overrunUpTime01Link = LinkUtils.createLink(overrunUpTime01, srcProp, getAo3p01(), destProp);
          getAo3p01().add("overrunUpTime01Link", overrunUpTime01Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get overrunUpTime01",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent overrunUpTime02Preset = getOverrunUpTime02Presets();
        if (overrunUpTime02Preset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)overrunUpTime02Preset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo3p02(), "setPointOverrunUpTime");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get overrunUpTime02Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent overrunUpTime02 = getoverrunUpTime02Register();
        if (null != overrunUpTime02) {
          Property srcProp = overrunUpTime02.getProperty("out");
          Property destProp = getAo3p02().getProperty("activeOverrunUpTime");
          BLink overrunUpTime02Link = LinkUtils.createLink(overrunUpTime02, srcProp, getAo3p02(), destProp);
          getAo3p02().add("overrunUpTime02Link", overrunUpTime02Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get overrunUpTime02Link",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent overrunDownTime01Preset = getOverrunDownTime01Presets();
        if (overrunDownTime01Preset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)overrunDownTime01Preset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo3p01(), "setPointOverrunDownTime");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get overrunDownTime01Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent overrunDownTime01 = getOverrunDownTime01Register();
        if (null != overrunDownTime01) {
          Property srcProp = overrunDownTime01.getProperty("out");
          Property destProp = getAo3p01().getProperty("activeOverrunDownTime");
          BLink overrunDownTime01Link = LinkUtils.createLink(overrunDownTime01, srcProp, getAo3p01(), destProp);
          getAo3p01().add("overrunDownTime01Link", overrunDownTime01Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get overrunDownTime01",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent overrunDownTime02Preset = getOverrunDownTime02Presets();
        if (overrunDownTime02Preset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)overrunDownTime02Preset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo3p02(), "setPointOverrunDownTime");
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get overrunDownTime02Preset",
          LOG_PREFIX,  getDeviceDisplayName(null)));
        }

        BComponent overrunDownTime02 = getOverrunDownTime02Register();
        if (null != overrunDownTime02) {
          Property srcProp = overrunDownTime02.getProperty("out");
          Property destProp = getAo3p02().getProperty("activeOverrunDownTime");
          BLink overrunDownTime02Link = LinkUtils.createLink(overrunDownTime02, srcProp, getAo3p02(), destProp);
          getAo3p02().add("overrunDownTime02Link", overrunDownTime02Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get overrunDownTime02Link",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent outputStatus01 = getOutputStatus01Register();
        if (null != outputStatus01) {
          Property srcProp = outputStatus01.getProperty("out");
          Property destProp = getAo3p01().getProperty("outputStatus");
          BLink outputStatus01Link = LinkUtils.createLink(outputStatus01, srcProp, getAo3p01(), destProp);
          getAo3p01().add("outputStatus01Link", outputStatus01Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get outputStatus01",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }

        BComponent outputStatus02 = getOutputStatus02Register();
        if (null != outputStatus02) {
          Property srcProp = outputStatus02.getProperty("out");
          Property destProp = getAo3p02().getProperty("outputStatus");
          BLink outputStatus02Link = LinkUtils.createLink(outputStatus02, srcProp, getAo3p02(), destProp);
          getAo3p02().add("outputStatus02Link", outputStatus02Link);
        }
        else {
          LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get outputStatus01",
                  LOG_PREFIX, getDeviceDisplayName(null)));
        }
    }

    public void writeRedLedValue() {
        BComponent ledRedPreset = getLedRedValuePreset();
        if(ledRedPreset instanceof BModbusClientPresetRegisters) {
            LOG.config(MessageFormat.format("{0}.updateLedRedColor({1}) update with {2}",
                    LOG_PREFIX, getDeviceDisplayName(null), getSetPointLedRedValue().getNumeric()));
            BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)ledRedPreset;
            AsyncBasicConfiguration.checkPresetWriteStatus(preset, this);

            preset.doWrite();
        }
        else {
            LOG.warning(MessageFormat.format("{0}.updateLedRedColor({1}) could not find Red Led Preset",
                    LOG_PREFIX, getDeviceDisplayName(null)));
        }
    }

    public void writeGreenLedValue() {
        BComponent ledGreenPreset = getLedGreenValuePreset();
        if(ledGreenPreset instanceof BModbusClientPresetRegisters) {
            LOG.config(MessageFormat.format("{0}.updateLedGreenColor({1}) update with {2}",
                    LOG_PREFIX, getDeviceDisplayName(null), getSetPointLedGreenValue().getNumeric()));
            BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)ledGreenPreset;
            AsyncBasicConfiguration.checkPresetWriteStatus(preset, this);
            preset.doWrite();
        }
        else {
            LOG.warning(MessageFormat.format("{0}.updateLedGreenColor({1}) could not find Green Led Preset",
                    LOG_PREFIX, getDeviceDisplayName(null)));
        }
    }

    private void writeSwitchLogicValue() {
        BComponent switchLogicPreset = getSwitchLogicValuePreset();
        if (switchLogicPreset instanceof BModbusClientPresetRegisters) {
            LOG.config(MessageFormat.format("{0}.updateSwitchLogic({1}) update with {2}",
                    LOG_PREFIX, getDeviceDisplayName(null), getSetPointSwitchLogicValue().getNumeric()));
            BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)switchLogicPreset;
            AsyncBasicConfiguration.checkPresetWriteStatus(preset, this);
            preset.doWrite();
        }
        else {
            LOG.warning(MessageFormat.format("{0}.updateSwitchLogic({1}) could not find SwitchLogic Preset",
                    LOG_PREFIX, getDeviceDisplayName(null)));
        }
    }

    private void writeBounceProtectionValue() {
        BComponent bounceProtectionPreset = getBounceProtectionValuePreset();
        if (bounceProtectionPreset instanceof BModbusClientPresetRegisters) {
            LOG.config(MessageFormat.format("{0}.updateBounceProtection({1}) update with {2}",
                    LOG_PREFIX, getDeviceDisplayName(null), getSetPointBounceProtectionValue().getNumeric()));
            BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)bounceProtectionPreset;
            AsyncBasicConfiguration.checkPresetWriteStatus(preset, this);
            preset.doWrite();
        }
        else {
            LOG.warning(MessageFormat.format("{0}.updateBounceProtection({1}) could not find BounceProtection Preset",
                    LOG_PREFIX, getDeviceDisplayName(null)));
        }
    }

    private void writeRuntimeUp(BComponent preset) {
      LOG.config(MessageFormat.format("{0}.writeRuntimeUp({1}) upate {2}",
                                      LOG_PREFIX, getDeviceDisplayName(null), preset.getName()));

      if(preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters runtimeUp = (BModbusClientPresetRegisters)preset;

        runtimeUp.doWrite();
      }
    }

    private void writeRuntimeDown(BComponent preset) {
      LOG.config(MessageFormat.format("{0}.writeRuntimeDown({1}) upate {2}",
                                      LOG_PREFIX, getDeviceDisplayName(null), preset.getName()));

      if(preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters runtimeDown = (BModbusClientPresetRegisters)preset;

        runtimeDown.doWrite();
      }
    }

    private void writeOnOffDelay(BComponent preset) {
      LOG.config(MessageFormat.format("{0}.writeOnOffDelay({1}) upate {2}",
                                      LOG_PREFIX, getDeviceDisplayName(null), preset.getName()));

      if(preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters onOffDelay = (BModbusClientPresetRegisters)preset;

        onOffDelay.doWrite();
      }
    }

    private void writeChangeOverDelay(BComponent preset) {
      LOG.config(MessageFormat.format("{0}.writeChangeOverDelay({1}) upate {2}",
                                      LOG_PREFIX, getDeviceDisplayName(null), preset.getName()));

      if(preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters changeOverDelay = (BModbusClientPresetRegisters)preset;

        changeOverDelay.doWrite();
      }
    }

    private void writeOverrunUpTime(BComponent preset) {
      LOG.config(MessageFormat.format("{0}.writeOverrunUpTime({1}) upate {2}",
                                      LOG_PREFIX, getDeviceDisplayName(null), preset.getName()));

      if(preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters overrunUp = (BModbusClientPresetRegisters)preset;

        overrunUp.doWrite();
      }
    }

    private void writeOverrunDownTime(BComponent preset) {
      LOG.config(MessageFormat.format("{0}.writeOverrunDownTime({1}) upate {2}",
                                      LOG_PREFIX, getDeviceDisplayName(null), preset.getName()));

      if(preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters overrunDown = (BModbusClientPresetRegisters)preset;

        overrunDown.doWrite();
      }
    }

    public boolean isRedLedValueModified() {
        BStatusNumeric active = getActiveLedRedValue();
        BStatusNumeric setPoint = getSetPointLedRedValue();

        return (active.getValue() != setPoint.getValue());
    }

    public boolean isGreenLedValueModified() {
        BStatusNumeric active = getActiveLedGreenValue();
        BStatusNumeric setPoint = getSetPointLedGreenValue();

        return (active.getValue() != setPoint.getValue());
    }

    private boolean isSwitchLogicValueModified() {
        BStatusNumeric active = getActiveSwitchLogicValue();
        BStatusNumeric setPoint = getSetPointSwitchLogicValue();

        return (active.getValue() != setPoint.getValue());
    }

    private boolean isBounceProtValueModified() {
        BStatusNumeric active = getActiveBounceProtectionValue();
        BStatusNumeric setPoint = getSetPointBounceProtectionValue();

        return (active.getValue() != setPoint.getValue());
    }

    private boolean isRuntimeUpModifiedOf(BAo3Point point) {
      BStatusNumeric active = point.getActiveRuntimeUp();
      BStatusNumeric setpoint = point.getSetPointRuntimeUp();

      return active.getValue() != setpoint.getValue();
    }

    private boolean isRuntimeDownModifiedOf(BAo3Point point) {
      BStatusNumeric active = point.getActiveRuntimeDown();
      BStatusNumeric setpoint = point.getSetPointRuntimeDown();

      return active.getValue() != setpoint.getValue();
    }

    private boolean isOnOffDelayModifiedOf(BAo3Point point) {
      BStatusNumeric active = point.getActiveOnOffDelay();
      BStatusNumeric setpoint = point.getSetPointOnOffDelay();

      return active.getValue() != setpoint.getValue();
    }

    private boolean isChangeOverDelayModifiedOf(BAo3Point point) {
      BStatusNumeric active = point.getActiveChangeOverDelay();
      BStatusNumeric setpoint = point.getSetPointChangeOverDelay();

      return active.getValue() != setpoint.getValue();
    }

    private boolean isOverrunUpTimeModifiedOf(BAo3Point point) {
      BStatusNumeric active = point.getActiveOverrunUpTime();
      BStatusNumeric setpoint = point.getSetPointOverrunUpTime();

      return active.getValue() != setpoint.getValue();
    }

    private boolean isOverrunDownTimeModifiedOf(BAo3Point point) {
      BStatusNumeric active = point.getActiveOverrunDownTime();
      BStatusNumeric setpoint = point.getSetPointOverrunDownTime();

      return active.getValue() != setpoint.getValue();
    }

    public BComponent getRedLedValueRegister() {
        return getRegisterByName("ledRedValue");
    }

    public BComponent getGreenLedValueRegister() {
        return getRegisterByName("ledGreenValue");
    }

    private BComponent getSwitchLogicValueRegister() {
        return getRegisterByName("switchLogicValue");
    }

    private BComponent getBounceProtectionValueRegister() {
        return getRegisterByName("bounceProtectionValue");
    }

    public BComponent getAutobaudingPreset() {
        return getRegisterByName("autobaudingPreset");
    }

    public BComponent getBaudratePreset() {
        return getRegisterByName("baudratePreset");
    }

    public BComponent getTimeoutPreset() {
        return getRegisterByName("timeoutSecPreset");
    }

    public BComponent getStatusLedPreset() {
        return getRegisterByName("statusLedPreset");
    }

    public BComponent getModuleCmdPreset() {
        return getRegisterByName("moduleCmdPreset");
    }

    private BComponent getLedRedValuePreset() {
        return getRegisterByName("ledRedValuePreset");
    }

    private BComponent getLedGreenValuePreset() {
        return getRegisterByName("ledGreenValuePreset");
    }

    private BComponent getSwitchLogicValuePreset() {
        return getRegisterByName("switchLogicValuePreset");
    }

    private BComponent getBounceProtectionValuePreset() {
        return getRegisterByName("bounceProtectionValuePreset");
    }

    private BComponent getInputRegister() {
        return getRegisterByName("Inputs");
    }

    private BComponent getFeedback01Register() {
        return  getRegisterByName("feedback01");
    }

    private BComponent getFeedback02Register() {
        return  getRegisterByName("feedback02");
    }

    public BComponent getOnSideSwitchValueRegister() {
      return getRegisterByName("onSideSwitchValue");
    }

    public BComponent getOnSideSwitchStatusRegister() {
      return getRegisterByName("onSideSwitchStatus");
    }

    private BComponent getAutomaticCmd01Presets() {
      return getRegisterByName("automaticCmd01Preset");
    }

    private BComponent getAutomaticCmd02Presets() {
      return getRegisterByName("automaticCmd02Preset");
    }

    private BComponent getAutomaticCmd01Preset() {
        BComponent container = getRegisterByName("automaticCmd01Preset");
        return (BComponent) container.get("autoCmd");
    }

    private BComponent getAutomaticCmd02Preset() {
        BComponent container = getRegisterByName("automaticCmd02Preset");
        return (BComponent) container.get("autoCmd");
    }

    private BComponent getUpTime01Presets() {
      return getRegisterByName("upTime01Preset");
    }

    private BComponent getUpTime01Register() {
      return getRegisterByName("upTime01");
    }

    private BComponent getUpTime02Presets() {
      return getRegisterByName("upTime02Preset");
    }

    private BComponent getUpTime02Register() {
      return getRegisterByName("upTime02");
    }

    private BComponent getDownTime01Presets() {
      return getRegisterByName("downTime01Preset");
    }

    private BComponent getDownTime01Register() {
      return getRegisterByName("downTime01");
    }

    private BComponent getDownTime02Presets() {
      return getRegisterByName("downTime02Preset");
    }

    private BComponent getDownTime02Register() {
      return getRegisterByName("downTime02");
    }

    private BComponent getOnOffDelay01Presets() {
      return getRegisterByName("onOffDelay01Preset");
    }

    private BComponent getOnOffDelay01Register() {
      return getRegisterByName("onOffDelay01");
    }

    private BComponent getOnOffDelay02Presets() {
      return getRegisterByName("onOffDelay02Preset");
    }

    private BComponent getOnOffDelay02Register() {
      return getRegisterByName("onOffDelay02");
    }
    private BComponent getChangeOverDelay01Presets() {
      return getRegisterByName("changeOverDelay01Preset");
    }

    private BComponent getChangeOverDelay01Register() {
      return getRegisterByName("changeOverDelay01");
    }

    private BComponent getChangeOverDelay02Presets() {
      return getRegisterByName("changeOverDelay02Preset");
    }

    private BComponent getChangeOverDelay02Register() {
      return getRegisterByName("changeOverDelay02");
    }

    @SuppressWarnings({"unused"}) // not needed yet but maybe in the future.
    private BComponent getEnableChannelsRegister() {
      return getRegisterByName("enableChannels");
    }

    private BComponent getOverrunUpTime01Presets() {
      return getRegisterByName("overrunUpTime01Preset");
    }

    private BComponent getoverrunUpTime01Register() {
      return getRegisterByName("overrunUpTime01");
    }

    private BComponent getOverrunUpTime02Presets() {
      return getRegisterByName("overrunUpTime02Preset");
    }

    private BComponent getoverrunUpTime02Register() {
      return getRegisterByName("overrunUpTime02");
    }

    private BComponent getOverrunDownTime01Presets() {
      return getRegisterByName("overrunDownTime01Preset");
    }

    private BComponent getOverrunDownTime01Register() {
      return getRegisterByName("overrunDownTime01");
    }

    private BComponent getOverrunDownTime02Presets() {
      return getRegisterByName("overrunDownTime02Preset");
    }

    private BComponent getOverrunDownTime02Register() {
      return getRegisterByName("overrunDownTime02");
    }

    private BComponent getOutputStatus01Register() {
      return getRegisterByName("outputStatus01");
    }

    private BComponent getOutputStatus01Preset() {
      return getRegisterByName("outputStatus01Preset");
    }

    private BComponent getOutputStatus02Register() {
      return getRegisterByName("outputStatus02");
    }
    private BComponent getOutputStatus02Preset() {
      return getRegisterByName("outputStatus02Preset");
    }


    public void updateSetPointLedColors() {
        int green = calcGreenLedValue();
        int red = calcRedLedValue();
        setSetPointLedGreenValue(new BStatusNumeric(green));
        setSetPointLedRedValue(new BStatusNumeric(red));
    }

    public void updateActiveOnSideSwitchValue() {

        Double onSideSwitchValuePosition = getActiveOnSideSwitchValue().getValue();
        BStatus status = getActiveOnSideSwitchValue().getStatus();
        int bits = onSideSwitchValuePosition.intValue();

        LOG.finer(MessageFormat.format("{0}.updateActiveOnSideSwitchValue({1}) current Value from {2} | {3}",
                                      LOG_PREFIX,
                                      getDeviceName(),
                                      Integer.toString(bits, 2),
                                      bits));

        getAo3p01().setActiveOnSideSwitchValue(
                                          new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 0), status));
        getAo3p02().setActiveOnSideSwitchValue(
                                          new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 2), status));

    }

    public void updateActiveOnSideSwitchStatus() {

        Double onSideSwitchStatusPosition = getActiveOnSideSwitchStatus().getValue();
        BStatus status = getActiveOnSideSwitchStatus().getStatus();
        int bits = onSideSwitchStatusPosition.intValue();

        LOG.finer(MessageFormat.format("{0}.updateActiveOnSideSwitchStatus({1}) current Value from {2} | {3}",
                                      LOG_PREFIX,
                                      getDeviceName(),
                                      Integer.toString(bits, 2),
                                      bits));

        getAo3p01().setActiveOnSideSwitchStatus(
                                        new BStatusBoolean(getOnSideSwitchStatusValue(bits, 0), status));
        getAo3p02().setActiveOnSideSwitchStatus(
                                        new BStatusBoolean(getOnSideSwitchStatusValue(bits, 1), status));
    }


    public void updateBounceProtection() {
        int temp = calcBounceProtection();

        setSetPointBounceProtectionValue(new BStatusNumeric(temp));
    }

    public void updateSwitchLogic() {
        int temp = calcSwitchLogic();

        setSetPointSwitchLogicValue(new BStatusNumeric(temp));
    }

    private void updateInputValues() {

      Double inputValuePosition = getInput().getValue();
      BStatus status = getInput().getStatus();
      int bits = inputValuePosition.intValue();

      LOG.finer(MessageFormat.format("{0}.updateInputValues({1}) current Value from {2} | {3}",
              LOG_PREFIX,
              getDeviceName(),
              Integer.toString(bits, 2),
              bits));

      getDi01().setInput(
              new BStatusBoolean(getBitFromInt(bits, 4), status));
      getDi02().setInput(
              new BStatusBoolean(getBitFromInt(bits, 5), status));
      getDi03().setInput(
              new BStatusBoolean(getBitFromInt(bits, 6), status));
      getDi04().setInput(
              new BStatusBoolean(getBitFromInt(bits, 7), status));
    }

    private boolean getBitFromInt (int inInt, int bitPosition) {
      int bit = (inInt >> bitPosition) & 1;
      return (bit == 1);
    }

    public int calcGreenLedValue() {

        int result = 0;

        int[] bits = new int[] {
                0, 0, 0, 0, // lower 4 bits will not be used.
                getDi01().getLedColor().getOrdinal(), getDi02().getLedColor().getOrdinal(),
                getDi03().getLedColor().getOrdinal(), getDi04().getLedColor().getOrdinal()
        };

        for (int index = 0; index < bits.length; index++) {
            result |= (bits[index] == BOasLedColor.GREEN || bits[index] == BOasLedColor.YELLOW) ? (1 << index) : 0;
        }

        return result;
    }

    public int calcRedLedValue() {
        int result = 0;

        int[] bits = new int[] {
                0, 0, 0, 0, // lower 4 bits will not be used.
                getDi01().getLedColor().getOrdinal(), getDi02().getLedColor().getOrdinal(),
                getDi03().getLedColor().getOrdinal(), getDi04().getLedColor().getOrdinal()
        };


        for (int index = 0; index < bits.length; index++) {
            result |= (bits[index] == BOasLedColor.RED || bits[index] == BOasLedColor.YELLOW) ? (1 << index) : 0;
        }


        return result;
    }

    private int calcSwitchLogic() {
        int result = 0;

        boolean[] bits = new boolean[] {
                getDi01().getSwitchLogic().getValue(), getDi02().getSwitchLogic().getValue(),
                getDi03().getSwitchLogic().getValue(), getDi04().getSwitchLogic().getValue()
        };


        for (int index = 0; index < bits.length; index++) {
            result |= (bits[index]) ? (1 << index) : 0;
        }

        return result;
    }

    private int calcBounceProtection() {
        int result = 0;

        boolean[] bits = new boolean[] {
                getDi01().getBounceProtection().getValue(), getDi02().getBounceProtection().getValue(),
                getDi03().getBounceProtection().getValue(), getDi04().getBounceProtection().getValue()
        };

        for (int index = 0; index < bits.length; index++) {
            result |= (bits[index]) ? (1 << index) : 0;
        }
        return result;
    }
    public BModbusAsyncDevice getDevice() {
        BComplex parent = getParent();
        if (parent instanceof BComponent) {
            return (BModbusAsyncDevice)parent;
        }
        return null;
    }

    protected BStatus getDeviceStatus() {

        return AsyncBasicConfiguration.getDeviceStatus(getDevice());
    }

    protected boolean isDeviceEnabeld() {

        return AsyncBasicConfiguration.isDeviceEnabeld(getDevice());
    }

    protected boolean isDeviceOk() {

        return AsyncBasicConfiguration.isDeviceOk(getDevice());
    }

    public String getDeviceName() {

        return AsyncBasicConfiguration.getDeviceName(getDevice());
    }

    @Override
    public String getDeviceDisplayName(Context ctx) {

        return AsyncBasicConfiguration.getDeviceDisplayName(getDevice(), ctx);
    }

    @Override
    protected BComponent[] getRegister() {

        return AsyncBasicConfiguration.getRegister(getDevice());
    }


    /**
   * Implemation of the channel1InitialRun100 Action
   */
  public void doChannel1InitialRun100() {
    BComponent outputStatus01Preset =  getOutputStatus01Preset();

    if (outputStatus01Preset instanceof BModbusClientPresetRegisters) {
      BModbusClientPresetRegisters presets = (BModbusClientPresetRegisters)outputStatus01Preset;
      BModbusClientPresetRegister preset = (BModbusClientPresetRegister)presets.get("ModbusClientPresetRegister");
      preset.setValue(BOasShutterStatus.INITIAL_RUN_HUNDRED);
      presets.write();
    }
    else {
      LOG.warning(MessageFormat.format(
        "{0}.doChannel1InitialRun100() could not execute initial run 100 on channel 01 because preset could not be found",
        LOG_PREFIX));
    }

  }

  /**
   * Implemation of the channel1InitialRun0 Action
   */
  public void doChannel1InitialRun0() {
    BComponent outputStatus01Preset =  getOutputStatus01Preset();

    if (outputStatus01Preset instanceof BModbusClientPresetRegisters) {
      BModbusClientPresetRegisters presets = (BModbusClientPresetRegisters)outputStatus01Preset;
      BModbusClientPresetRegister preset = (BModbusClientPresetRegister)presets.get("ModbusClientPresetRegister");
      preset.setValue(BOasShutterStatus.INITIAL_RUN_ZERO);
      presets.write();
    }
    else {
      LOG.warning(MessageFormat.format(
        "{0}.doChannel1InitialRun0() could not execute initial run 0 on channel 01 because preset could not be found",
        LOG_PREFIX));
    }
  }

  /**
   * Implemation of the channel2InitialRun100 Action
   */
  public void doChannel2InitialRun100() {
    BComponent outputStatus02Preset =  getOutputStatus02Preset();

    if (outputStatus02Preset instanceof BModbusClientPresetRegisters) {
      BModbusClientPresetRegisters presets = (BModbusClientPresetRegisters)outputStatus02Preset;
      BModbusClientPresetRegister preset = (BModbusClientPresetRegister)presets.get("ModbusClientPresetRegister");
      preset.setValue(BOasShutterStatus.INITIAL_RUN_HUNDRED);
      presets.write();
    }
    else {
      LOG.warning(MessageFormat.format(
        "{0}.doChannel2InitialRun100() could not execute initial run 100 on channel 02 because preset could not be found",
        LOG_PREFIX));
    }
  }
  /**
   * Implemation of the channel2InitialRun0 Action
   */
  public void doChannel2InitialRun0() {
    BComponent outputStatus02Preset =  getOutputStatus02Preset();

    if (outputStatus02Preset instanceof BModbusClientPresetRegisters) {
      BModbusClientPresetRegisters presets = (BModbusClientPresetRegisters)outputStatus02Preset;
      BModbusClientPresetRegister preset = (BModbusClientPresetRegister)presets.get("ModbusClientPresetRegister");
      preset.setValue(BOasShutterStatus.INITIAL_RUN_ZERO);
      presets.write();
    }
    else {
      LOG.warning(MessageFormat.format(
        "{0}.doChannel2InitialRun0() could not execute initial run 0 on channel 02 because preset could not be found",
        LOG_PREFIX));
    }
  }

  /**
   * execute the write action of the BModbusClientPresetRegisters returned by
   * {@link #getAutomaticCmdPresets()} and saves the execution time in {@link #lastAutoCmdWrite}
   * also reset the the {@link #maxSendTicket}
   */
  public void doWriteAutoCmd() {

    if (isConfigurationValid()) {

      BComponent autoCmd01 = getAutomaticCmd01Presets();

      if (autoCmd01 instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters autoCmdPresets = (BModbusClientPresetRegisters)autoCmd01;

        AsyncBasicConfiguration.checkPresetWriteStatus(autoCmdPresets, this);

        LOG.info(MessageFormat.format("{0}.doWriteAutoCmd({1}) write AutoCmd01 Presets", LOG_PREFIX, getDeviceName()));

        autoCmdPresets.write();
        restartMaxSendTicket();
      }
      else {
        LOG.warning(MessageFormat.format("{0}.doWriteAutoCmd(): Object was not of type BModbusClientPresetRegisters",
                                      LOG_PREFIX));
      }

      BComponent autoCmd02 = getAutomaticCmd02Presets();

      if (autoCmd02 instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters autoCmdPresets = (BModbusClientPresetRegisters)autoCmd02;

        AsyncBasicConfiguration.checkPresetWriteStatus(autoCmdPresets, this);

        LOG.info(MessageFormat.format("{0}.doWriteAutoCmd({1}) write AutoCmd02 Presets", LOG_PREFIX, getDeviceName()));

        autoCmdPresets.write();
        restartMaxSendTicket();
      }
      else {
        LOG.warning(MessageFormat.format("{0}.doWriteAutoCmd(): Object was not of type BModbusClientPresetRegisters",
                                      LOG_PREFIX));
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.doWriteAutoCmd(): Config was not valid. No write was executed", LOG_PREFIX));
    }
  }

  @Override
  public void autoCmdChanged() {
    synchronized (autoCmdMutex) {

      if (!isAutoCmdWriteLocked()) {
        LOG.info(MessageFormat.format("{0}.autoCmdChanged({1}) write is unlocked", LOG_PREFIX, getDeviceName()));
        lockAutoCmdWrite();
        doWriteAutoCmd();
      }
      else {
        LOG.info(MessageFormat.format("{0}.autoCmdChanged({1}) write is locked!!", LOG_PREFIX, getDeviceName()));
      }
    }
  }

  @Override
  public boolean isOnSideSwitchConfigured(int value, int status) {

    int config = ~value & status;

    int channel01 = config >> 0 & 1;
    int channel02 = config >> 2 & 1;

    return ( (channel01 == 1) || (channel02 == 1) );
 }

}
