package com.oas.modbusutilities.configuration;
import java.text.MessageFormat;
import java.util.logging.Logger;

import javax.baja.nre.annotations.NiagaraType;
import javax.baja.status.BStatus;
import javax.baja.status.BStatusBoolean;
import javax.baja.status.BStatusNumeric;
import javax.baja.sys.BComplex;
import javax.baja.sys.BComponent;
import javax.baja.sys.BDouble;
import javax.baja.sys.BLink;
import javax.baja.sys.Context;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.oas.modbusutilities.configuration.point.BAoPoint;
import com.oas.modbusutilities.enums.BOasAutobauding;
import com.oas.modbusutilities.enums.BOasBautrate;
import com.oas.modbusutilities.enums.BOasModuleCmd;
import com.oas.modbusutilities.utils.AsyncLinkUtils;
import com.oas.modbusutilities.utils.LinkUtils;
import com.tridium.modbusAsync.BModbusAsyncDevice;
import com.tridium.modbusAsync.BModbusAsyncDeviceFolder;
import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegister;
import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegisters;


/**
 * BAoConfiguration
 */
@SuppressWarnings({
  "squid:S2629", // Invoke LOGs conditionally will bloat up the code
  "squid:S110" // Niagara Framework has more inherit lvl than 6
})
@NiagaraType
public class BAsyncAoConfiguration
  extends BAbstractAoConfiguration
  implements AsyncConfiguration
{
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.oas.modbusutilities.configuration.BAsyncAoConfiguration(2979906276)1.0$ @*/
/* Generated Mon Mar 23 11:52:00 CET 2020 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////

  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BAsyncAoConfiguration.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/


  private static final Logger LOG = Logger.getLogger("oasModbusUtilities");
  private static final String LOG_PREFIX = BAsyncAoConfiguration.class.getSimpleName();

  @Override
  public boolean isParentLegal(BComponent parent) {

    return (parent instanceof BModbusAsyncDeviceFolder) ||
           (parent instanceof BModbusAsyncDevice);
  }


  @Override
  public void descendantsStarted() throws Exception {
    super.descendantsStarted();

    updateSetPointSavePositionStatus();

    restartMaxSendTicket();
  }


  @Override
  public void doWriteConfiguration() {
    super.doWriteConfiguration();
    if (!isRunning()) {
      return;
    }

    if (isModuleTypeValid()) {

      writeOnSideSwitchStatus();
      writeOnSideSwitchValue();
      writeSavePositionStatus();
      writeSavePositionValue();
    }
  }

  @Override
  public void doWriteModuleCmd(BOasModuleCmd cmd) {
    LOG.info(MessageFormat.format("{0}.doWriteModuleCmd(): upate to {1}", LOG_PREFIX, cmd));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getModuleCmdPreset(), this);
    AsyncBasicConfiguration.doWriteModuleCmd(getModuleCmdRegister(), getModuleCmdPreset(), cmd);

  }

  @Override
  protected void writeAutobinding(BOasAutobauding value) {
    LOG.info(MessageFormat.format("{0}.writeAutobinding(): upate to {1}", LOG_PREFIX, value));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getAutobaudingPreset(), this);
    AsyncBasicConfiguration.writeAutobinding(getAutobaudingPreset(), value);
  }

  @Override
  protected void writeBaudreate(BOasBautrate value) {
    LOG.info(MessageFormat.format("{0}.writeBaudreate(): upate to {1}", LOG_PREFIX, value));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getBaudratePreset(), this);
    AsyncBasicConfiguration.writeBaudreate(getBaudratePreset(), value);
  }

  @Override
  protected void writeTimeout(BDouble value) {
    LOG.info(MessageFormat.format("{0}.writeTimeout(): upate to {1}", LOG_PREFIX, value.getDouble()));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getTimeoutPreset(), this);
    AsyncBasicConfiguration.writeTimeout(getTimeoutPreset());
  }

  protected void writeLedStatus() {
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getStatusLedPreset(), this);
    AsyncBasicConfiguration.writeLedStatus(getStatusLedRegister(), getStatusLedPreset(), statusLedOneValue, statusLedTwoValue);

  }

  @Override
  public BConfigurationStatus checkConfiguration() {
    BConfigurationStatus newStatus = super.checkConfiguration();

    if (getDeviceStatus().isDisabled() || getDeviceStatus().isDown()) {
      return BConfigurationStatus.offline;
    }


    if (!isRunning()) {
      LOG.finest(MessageFormat.format("{0}.checkConfiguration({1}) station is not running",
                                        LOG_PREFIX,  getDeviceName()));
      return BConfigurationStatus.offline;
    }

    if (isConfigurationValid()) {


      // Check if the Points have
      if (isSavePositionStatusModifiedOf(getAo01())) {
        updateSetPointSavePositionStatus();
      }
      if (isSavePositionStatusModifiedOf(getAo02())) {
        updateSetPointSavePositionStatus();
      }
      if (isSavePositionStatusModifiedOf(getAo03())) {
        updateSetPointSavePositionStatus();
      }
      if (isSavePositionStatusModifiedOf(getAo04())) {
        updateSetPointSavePositionStatus();
      }
      if (isSavePositionStatusModifiedOf(getAo05())) {
        updateSetPointSavePositionStatus();
      }
      if (isSavePositionStatusModifiedOf(getAo06())) {
        updateSetPointSavePositionStatus();
      }
      if (isSavePositionStatusModifiedOf(getAo07())) {
        updateSetPointSavePositionStatus();
      }
      if (isSavePositionStatusModifiedOf(getAo08())) {
        updateSetPointSavePositionStatus();
      }

      if (isSavePositionStatusModified()) {
        LOG.warning(MessageFormat.format(
                    "{0}.checkConfiguration({1}) configuration missmatch: savePositionStatus is {2} instead of {3} ",
                    LOG_PREFIX, getDeviceName(),
                    getActiveSavePositionStatus(), getSetPointSavePositionStatus()));

          setConfigModifiedStatus();
          newStatus = BConfigurationStatus.makeModified(newStatus, true);
          writeSavePositionStatus();
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) SavePositionStatus  is up to date",
                                      LOG_PREFIX, getDeviceName()));

      }

      if (isSavePositionValueModifiedOf(getAo01())) {
        LOG.warning(MessageFormat.format(
                    "{0}.checkConfiguration({1}) configuration missmatch: SavePositionValue 01 is {2} instead of {3} ",
                    LOG_PREFIX, getDeviceName(),
                    getAo01().getActiveSavePositionValue(), getAo01().getSetPointSavePositionValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeSavePositionValueOf(getSavePositionValue01Preset());

      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) SavePositionValue 01  is up to date",
                                      LOG_PREFIX, getDeviceName()));

      }

      if (isSavePositionValueModifiedOf(getAo02())) {
        LOG.warning(MessageFormat.format(
                    "{0}.checkConfiguration({1}) configuration missmatch: SavePositionValue 02 is {2} instead of {3} ",
                    LOG_PREFIX, getDeviceName(),
                    getAo02().getActiveSavePositionValue(), getAo02().getSetPointSavePositionValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeSavePositionValueOf(getSavePositionValue02Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) SavePositionValue 02  is up to date",
                                      LOG_PREFIX, getDeviceName()));

      }
      if (isSavePositionValueModifiedOf(getAo03())) {
        LOG.warning(MessageFormat.format(
                    "{0}.checkConfiguration({1}) configuration missmatch: SavePositionValue 03 is {2} instead of {3} ",
                    LOG_PREFIX, getDeviceName(),
                    getAo03().getActiveSavePositionValue(), getAo03().getSetPointSavePositionValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeSavePositionValueOf(getSavePositionValue03Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) SavePositionValue 03  is up to date",
                                      LOG_PREFIX, getDeviceName()));

      }

      if (isSavePositionValueModifiedOf(getAo04())) {
        LOG.warning(MessageFormat.format(
                    "{0}.checkConfiguration({1}) configuration missmatch: SavePositionValue 04 is {2} instead of {3} ",
                    LOG_PREFIX, getDeviceName(),
                    getAo04().getActiveSavePositionValue(), getAo04().getSetPointSavePositionValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeSavePositionValueOf(getSavePositionValue04Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) SavePositionValue 04  is up to date",
                                      LOG_PREFIX, getDeviceName()));

      }

      if (isSavePositionValueModifiedOf(getAo05())) {
        LOG.warning(MessageFormat.format(
                    "{0}.checkConfiguration({1}) configuration missmatch: SavePositionValue 05 is {2} instead of {3} ",
                    LOG_PREFIX, getDeviceName(),
                    getAo05().getActiveSavePositionValue(), getAo05().getSetPointSavePositionValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeSavePositionValueOf(getSavePositionValue05Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) SavePositionValue 05  is up to date",
                                      LOG_PREFIX, getDeviceName()));

      }

      if (isSavePositionValueModifiedOf(getAo06())) {
        LOG.warning(MessageFormat.format(
                    "{0}.checkConfiguration({1}) configuration missmatch: SavePositionValue 06 is {2} instead of {3} ",
                    LOG_PREFIX, getDeviceName(),
                    getAo06().getActiveSavePositionValue(), getAo06().getSetPointSavePositionValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeSavePositionValueOf(getSavePositionValue06Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) SavePositionValue 06  is up to date",
                                      LOG_PREFIX, getDeviceName()));
      }

      if (isSavePositionValueModifiedOf(getAo07())) {
        LOG.warning(MessageFormat.format(
                    "{0}.checkConfiguration({1}) configuration missmatch: SavePositionValue 07 is {2} instead of {3} ",
                    LOG_PREFIX, getDeviceName(),
                    getAo07().getActiveSavePositionValue(), getAo07().getSetPointSavePositionValue()));
        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeSavePositionValueOf(getSavePositionValue07Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) SavePositionValue 07  is up to date",
                                      LOG_PREFIX, getDeviceName()));

      }

      if (isSavePositionValueModifiedOf(getAo08())) {
        LOG.warning(MessageFormat.format(
                    "{0}.checkConfiguration({1}) configuration missmatch: SavePositionValue 08 is {2} instead of {3} ",
                    LOG_PREFIX, getDeviceName(),
                    getAo08().getActiveSavePositionValue(), getAo08().getSetPointSavePositionValue()));

        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeSavePositionValueOf(getSavePositionValue08Preset());
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) SavePositionValue 08  is up to date", LOG_PREFIX, getDeviceName()));

      }

    }
    else {
      LOG.warning(MessageFormat.format(
                  "{0}.checkConfiguration({1}) either ModuleType[{2} != {3}] or Baudrate[{4} != {5}] is modified",
                  LOG_PREFIX, getDevice().getName(),
                  getActiveModuleType(), getSetPointModuleType(),
                  getActiveBaudrate(), getSetPointBaudrate()));
    }

    return newStatus;
  }

  @Override
  public void createLinksToRegister() {

    AsyncBasicConfiguration.createLinksToRegister(getDevice(), this);

    LOG.fine(MessageFormat.format("{0}.createLinksToRegister({1}) {2}.{3}:create Links to Config Component",
                                    LOG_PREFIX, getDeviceDisplayName(null), getDeviceName(), this.getName()));




    BComponent onSideSwitchStatus = getOnSideSwitchStatusRegister();
    if (null != onSideSwitchStatus) {
      loadSlots();
      Property srcProp = onSideSwitchStatus.getProperty("out");
      BLink onSideSwitchStatusLink = LinkUtils.createLink(onSideSwitchStatus, srcProp, this, activeOnSideSwitchStatus);
      add("onSideSwitchStatusLink", onSideSwitchStatusLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get onSideSwitchActiveRegister",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent onSideSwitchValue = getOnSideSwitchValueRegister();
    if (null != onSideSwitchValue) {
      loadSlots();
      Property srcProp = onSideSwitchValue.getProperty("out");
      BLink onSideSwitchValueLink = LinkUtils.createLink(onSideSwitchValue, srcProp, this, activeOnSideSwitchValue);
      add("onSideSwitchValueLink", onSideSwitchValueLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get OnSideSwitchValueRegister",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionStatus = getSavePositionStatusRegister();
    if (null != savePositionStatus) {
      Property srcProp = savePositionStatus.getProperty("out");
      BLink savePositionStatusLink = LinkUtils.createLink(savePositionStatus, srcProp, this, activeSavePositionStatus);
      add("savePositionStatusLink", savePositionStatusLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionStatusRegister",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionStatusPreset = getSavePositionStatusPreset();
    if (null != savePositionStatusPreset) {
      if (savePositionStatusPreset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)savePositionStatusPreset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "setPointSavePositionStatus");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionStatusPreset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent feedback01 = getFeedback01Register();
    if (null != feedback01) {
        Property srcProp = feedback01.getProperty("out");
        Property destProp = getAo01().getProperty("inFeedback");
        BLink feedback01Link = LinkUtils.createLink(feedback01, srcProp, getAo01(), destProp);
        getAo01().add("feedback01Link", feedback01Link);
    }
    else {
        LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get feedback01Register",
                LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent feedback02 = getFeedback02Register();
    if (null != feedback02) {
        Property srcProp = feedback02.getProperty("out");
        Property destProp = getAo02().getProperty("inFeedback");
        BLink feedback02Link = LinkUtils.createLink(feedback02, srcProp, getAo02(), destProp);
        getAo02().add("feedback02Link", feedback02Link);
    }
    else {
        LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get feedback02Register",
                LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent feedback03 = getFeedback03Register();
    if (null != feedback03) {
        Property srcProp = feedback03.getProperty("out");
        Property destProp = getAo03().getProperty("inFeedback");
        BLink feedback03Link = LinkUtils.createLink(feedback03, srcProp, getAo03(), destProp);
        getAo03().add("feedback03Link", feedback03Link);
    }
    else {
        LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get feedback03Register",
                LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent feedback04 = getFeedback04Register();
    if (null != feedback04) {
        Property srcProp = feedback04.getProperty("out");
        Property destProp = getAo04().getProperty("inFeedback");
        BLink feedback04Link = LinkUtils.createLink(feedback04, srcProp, getAo04(), destProp);
        getAo04().add("feedback04Link", feedback04Link);
    }
    else {
        LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get feedback04Register",
                LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent feedback05 = getFeedback05Register();
    if (null != feedback05) {
        Property srcProp = feedback05.getProperty("out");
        Property destProp = getAo05().getProperty("inFeedback");
        BLink feedback05Link = LinkUtils.createLink(feedback05, srcProp, getAo05(), destProp);
        getAo05().add("feedback05Link", feedback05Link);
    }
    else {
        LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get feedback05Register",
                LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent feedback06 = getFeedback06Register();
    if (null != feedback06) {
        Property srcProp = feedback06.getProperty("out");
        Property destProp = getAo06().getProperty("inFeedback");
        BLink feedback06Link = LinkUtils.createLink(feedback06, srcProp, getAo06(), destProp);
        getAo06().add("feedback06Link", feedback06Link);
    }
    else {
        LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get feedback06Register",
                LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent feedback07 = getFeedback07Register();
    if (null != feedback07) {
        Property srcProp = feedback07.getProperty("out");
        Property destProp = getAo07().getProperty("inFeedback");
        BLink feedback07Link = LinkUtils.createLink(feedback07, srcProp, getAo07(), destProp);
        getAo07().add("feedback07Link", feedback07Link);
    }
    else {
        LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get feedback07Register",
                LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent feedback08 = getFeedback08Register();
    if (null != feedback08) {
        Property srcProp = feedback08.getProperty("out");
        Property destProp = getAo08().getProperty("inFeedback");
        BLink feedback08Link = LinkUtils.createLink(feedback08, srcProp, getAo08(), destProp);
        getAo08().add("feedback08Link", feedback08Link);
    }
    else {
        LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get feedback08Register",
                LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent automaticCmd01Preset = getAutomaticCmd01Preset();
    if (automaticCmd01Preset instanceof BModbusClientPresetRegister) {
      BModbusClientPresetRegister preset = (BModbusClientPresetRegister)automaticCmd01Preset;
      AsyncLinkUtils.linkToPreset(preset, (BComponent)getAo01(), "outAutomaticCmd");
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get automaticCmd01Preset",
              LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent automaticCmd02Preset = getAutomaticCmd02Preset();
    if (automaticCmd02Preset instanceof BModbusClientPresetRegister) {
      BModbusClientPresetRegister preset = (BModbusClientPresetRegister)automaticCmd02Preset;
      AsyncLinkUtils.linkToPreset(preset, (BComponent)getAo02(), "outAutomaticCmd");
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get automaticCmd02Preset",
              LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent automaticCmd03Preset = getAutomaticCmd03Preset();
    if (automaticCmd03Preset instanceof BModbusClientPresetRegister) {
      BModbusClientPresetRegister preset = (BModbusClientPresetRegister)automaticCmd03Preset;
      AsyncLinkUtils.linkToPreset(preset, (BComponent)getAo03(), "outAutomaticCmd");
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get automaticCmd03Preset",
              LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent automaticCmd04Preset = getAutomaticCmd04Preset();
    if (automaticCmd04Preset instanceof BModbusClientPresetRegister) {
      BModbusClientPresetRegister preset = (BModbusClientPresetRegister)automaticCmd04Preset;
      AsyncLinkUtils.linkToPreset(preset, (BComponent)getAo04(), "outAutomaticCmd");
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get automaticCmd04Preset",
              LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent automaticCmd05Preset = getAutomaticCmd05Preset();
    if (automaticCmd05Preset instanceof BModbusClientPresetRegister) {
      BModbusClientPresetRegister preset = (BModbusClientPresetRegister)automaticCmd05Preset;
      AsyncLinkUtils.linkToPreset(preset, (BComponent)getAo05(), "outAutomaticCmd");
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get automaticCmd05Preset",
              LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent automaticCmd06Preset = getAutomaticCmd06Preset();
    if (automaticCmd06Preset instanceof BModbusClientPresetRegister) {
      BModbusClientPresetRegister preset = (BModbusClientPresetRegister)automaticCmd06Preset;
      AsyncLinkUtils.linkToPreset(preset, (BComponent)getAo06(), "outAutomaticCmd");
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get automaticCmd06Preset",
              LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent automaticCmd07Preset = getAutomaticCmd07Preset();
    if (automaticCmd07Preset instanceof BModbusClientPresetRegister) {
      BModbusClientPresetRegister preset = (BModbusClientPresetRegister)automaticCmd07Preset;
      AsyncLinkUtils.linkToPreset(preset, (BComponent)getAo07(), "outAutomaticCmd");
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get automaticCmd07Preset",
              LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent automaticCmd08Preset = getAutomaticCmd08Preset();
    if (automaticCmd08Preset instanceof BModbusClientPresetRegister) {
      BModbusClientPresetRegister preset = (BModbusClientPresetRegister)automaticCmd08Preset;
      AsyncLinkUtils.linkToPreset(preset, (BComponent)getAo08(), "outAutomaticCmd");
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get automaticCmd08Preset",
              LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue01 = getSavePositionValue01Register();
    if (null != savePositionValue01) {
      Property srcProp = savePositionValue01.getProperty("out");
      Property destProp = getAo01().getProperty("inSavePosition");
      BLink savePositionValue01Link = LinkUtils.createLink(savePositionValue01, srcProp, getAo01(), destProp);
      getAo01().add("savePositionValue01Link", savePositionValue01Link);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue01Register",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue01Preset = getSavePositionValue01Preset();
    if (null != savePositionValue01Preset) {
      if (savePositionValue01Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)savePositionValue01Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo01(), "outSavePosition");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue01Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue02 = getSavePositionValue02Register();
    if (null != savePositionValue02) {
      Property srcProp = savePositionValue02.getProperty("out");
      Property destProp = getAo02().getProperty("inSavePosition");
      BLink savePositionValue02Link = LinkUtils.createLink(savePositionValue02, srcProp, getAo02(), destProp);
      getAo02().add("savePositionValue02Link", savePositionValue02Link);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue02Register",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue02Preset = getSavePositionValue02Preset();
    if (null != savePositionValue02Preset) {
      if (savePositionValue02Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)savePositionValue02Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo02(), "outSavePosition");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue02Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue03 = getSavePositionValue03Register();
    if (null != savePositionValue03) {
      Property srcProp = savePositionValue03.getProperty("out");
      Property destProp = getAo03().getProperty("inSavePosition");
      BLink savePositionValue03Link = LinkUtils.createLink(savePositionValue03, srcProp, getAo03(), destProp);
      getAo03().add("savePositionValue03Link", savePositionValue03Link);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue03Register",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue03Preset = getSavePositionValue03Preset();
    if (null != savePositionValue03Preset) {
      if (savePositionValue03Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)savePositionValue03Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo03(), "outSavePosition");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue03Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }


    BComponent savePositionValue04 = getSavePositionValue04Register();
    if (null != savePositionValue04) {
      Property srcProp = savePositionValue04.getProperty("out");
      Property destProp = getAo04().getProperty("inSavePosition");
      BLink savePositionValue04Link = LinkUtils.createLink(savePositionValue04, srcProp, getAo04(), destProp);
      getAo04().add("savePositionValue04Link", savePositionValue04Link);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue04Register",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue04Preset = getSavePositionValue04Preset();
    if (null != savePositionValue04Preset) {
      if (savePositionValue04Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)savePositionValue04Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo04(), "outSavePosition");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue04Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue05 = getSavePositionValue05Register();
    if (null != savePositionValue05) {
      Property srcProp = savePositionValue05.getProperty("out");
      Property destProp = getAo05().getProperty("inSavePosition");
      BLink savePositionValue05Link = LinkUtils.createLink(savePositionValue05, srcProp, getAo05(), destProp);
      getAo05().add("savePositionValue05Link", savePositionValue05Link);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue05Register",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue05Preset = getSavePositionValue05Preset();
    if (null != savePositionValue05Preset) {
        if (savePositionValue05Preset instanceof BModbusClientPresetRegisters) {
          BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)savePositionValue05Preset;
          AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo05(), "outSavePosition");
        }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue05Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue06 = getSavePositionValue06Register();
    if (null != savePositionValue06) {
      Property srcProp = savePositionValue06.getProperty("out");
      Property destProp = getAo06().getProperty("inSavePosition");
      BLink savePositionValue06Link = LinkUtils.createLink(savePositionValue06, srcProp, getAo06(), destProp);
      getAo06().add("savePositionValue06Link", savePositionValue06Link);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue06Register",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue06Preset = getSavePositionValue06Preset();
    if (null != savePositionValue06Preset) {
      if (savePositionValue06Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)savePositionValue06Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo06(), "outSavePosition");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue06Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue07 = getSavePositionValue07Register();
    if (null != savePositionValue07) {
      Property srcProp = savePositionValue07.getProperty("out");
      Property destProp = getAo07().getProperty("inSavePosition");
      BLink savePositionValue07Link = LinkUtils.createLink(savePositionValue07, srcProp, getAo07(), destProp);
      getAo07().add("savePositionValue07Link", savePositionValue07Link);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue07Register",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue07Preset = getSavePositionValue07Preset();
    if (null != savePositionValue07Preset) {
      if (savePositionValue07Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)savePositionValue07Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo07(), "outSavePosition");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue07Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue08 = getSavePositionValue08Register();
    if (null != savePositionValue08) {
      Property srcProp = savePositionValue08.getProperty("out");
      Property destProp = getAo08().getProperty("inSavePosition");
      BLink savePositionValue08Link = LinkUtils.createLink(savePositionValue08, srcProp, getAo08(), destProp);
      getAo08().add("savePositionValue08Link", savePositionValue08Link);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue08Register",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }

    BComponent savePositionValue08Preset = getSavePositionValue08Preset();
    if (null != savePositionValue08Preset) {
      if (savePositionValue08Preset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)savePositionValue08Preset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)getAo08(), "outSavePosition");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionValue08Preset",
                                        LOG_PREFIX,  getDeviceDisplayName(null)));
    }
  }

  @Override
  public void changed(Property property, Context context) {
    super.changed(property, context);

    if(!isRunning()) {
      return;
    }

    if (property.equals(activeOnSideSwitchValue)) {
      updateActiveOnSideSwitchValue();
      int value = new Double(getActiveOnSideSwitchValue().getValue()).intValue();
      int status = new Double(getActiveOnSideSwitchStatus().getValue()).intValue();
      if (isOnSideSwitchConfigured(value, status)){

        setOnSideSwitchStatusOn();
      }
      else {
        setOnSideSwitchStatusOff();
      }
    }
    else if (property.equals(activeOnSideSwitchStatus)) {
      updateActiveOnSideSwitchStatus();
      int value = new Double(getActiveOnSideSwitchValue().getValue()).intValue();
      int status = new Double(getActiveOnSideSwitchStatus().getValue()).intValue();
      if (isOnSideSwitchConfigured(value, status)){

        setOnSideSwitchStatusOn();
      }
      else {
        setOnSideSwitchStatusOff();
      }
    }
    else if (property.equals(activeSavePositionStatus)) {
      updateActiveSavePositionStatus();
    }
    else if (property.equals(maxWriteTime)) {
      if (getDevice().getEnabled()) {
        restartMaxSendTicket();
      }
    }
  }

  public void updateActiveOnSideSwitchValue() {

    Double onSideSwitchValuePosition = getActiveOnSideSwitchValue().getValue();
    BStatus status = getActiveOnSideSwitchValue().getStatus();
    int bits = onSideSwitchValuePosition.intValue();

    LOG.finer(MessageFormat.format("{0}.updateActiveOnSideSwitchValue({1}) current Value from {2} | {3}",
                                    LOG_PREFIX,
                                    getDeviceName(),
                                    Integer.toString(bits, 2),
                                    bits));

    getAo01().setActiveOnSideSwitchValue(new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 0), status));
    getAo02().setActiveOnSideSwitchValue(new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 1), status));
    getAo03().setActiveOnSideSwitchValue(new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 2), status));
    getAo04().setActiveOnSideSwitchValue(new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 3), status));
    getAo05().setActiveOnSideSwitchValue(new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 4), status));
    getAo06().setActiveOnSideSwitchValue(new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 5), status));
    getAo07().setActiveOnSideSwitchValue(new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 6), status));
    getAo08().setActiveOnSideSwitchValue(new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 7), status));

  }

  public void updateActiveOnSideSwitchStatus() {

    Double onSideSwitchStatusPosition = getActiveOnSideSwitchStatus().getValue();
    BStatus status = getActiveOnSideSwitchStatus().getStatus();
    int bits = onSideSwitchStatusPosition.intValue();

    LOG.finer(MessageFormat.format("{0}.updateActiveOnSideSwitchStatus({1}) current Value from {2} | {3}",
                                  LOG_PREFIX,
                                  getDeviceName(),
                                  Integer.toString(bits, 2),
                                  bits));

    getAo01().setActiveOnSideSwitchStatus(new BStatusBoolean(getOnSideSwitchStatusValue(bits, 0), status));
    getAo02().setActiveOnSideSwitchStatus(new BStatusBoolean(getOnSideSwitchStatusValue(bits, 1), status));
    getAo03().setActiveOnSideSwitchStatus(new BStatusBoolean(getOnSideSwitchStatusValue(bits, 2), status));
    getAo04().setActiveOnSideSwitchStatus(new BStatusBoolean(getOnSideSwitchStatusValue(bits, 3), status));
    getAo05().setActiveOnSideSwitchStatus(new BStatusBoolean(getOnSideSwitchStatusValue(bits, 4), status));
    getAo06().setActiveOnSideSwitchStatus(new BStatusBoolean(getOnSideSwitchStatusValue(bits, 5), status));
    getAo07().setActiveOnSideSwitchStatus(new BStatusBoolean(getOnSideSwitchStatusValue(bits, 6), status));
    getAo08().setActiveOnSideSwitchStatus(new BStatusBoolean(getOnSideSwitchStatusValue(bits, 7), status));
  }


  /**
   * @deprecated
   */
  @Deprecated
  @Override
  public void updateSetPointOnSideSwitchStatus() {

    Double onSideSwitchPoistion = getActiveOnSideSwitchStatus().getValue();
    BStatus status = getActiveOnSideSwitchStatus().getStatus();
    int positionConfig = onSideSwitchPoistion.intValue();
    int newValue = positionConfig & 1;

    LOG.finer(MessageFormat.format("{0}.updateSetPointOnSideSwitchStatus({1}) onSideSwitchStatus of AO01 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo01().setActiveOnSideSwitchStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 1) & 1;
    LOG.finer(MessageFormat.format("{0}.updateSetPointOnSideSwitchStatus({1}) onSideSwitchStatus of AO02 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo02().setActiveOnSideSwitchStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 2) & 1;
    LOG.finer(MessageFormat.format("{0}.updateSetPointOnSideSwitchStatus({1}) onSideSwitchStatus of AO03 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo03().setActiveOnSideSwitchStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 3) & 1;
    LOG.finer(MessageFormat.format("{0}.updateSetPointOnSideSwitchStatus({1}) onSideSwitchStatus of AO04 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo04().setActiveOnSideSwitchStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 4) & 1;
    LOG.finer(MessageFormat.format("{0}.updateSetPointOnSideSwitchStatus({1}) onSideSwitchStatus of AO05 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo05().setActiveOnSideSwitchStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 5) & 1;
    LOG.finer(MessageFormat.format("{0}.updateSetPointOnSideSwitchStatus({1}) onSideSwitchStatus of AO06 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo06().setActiveOnSideSwitchStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 6) & 1;
    LOG.finer(MessageFormat.format("{0}.updateSetPointOnSideSwitchStatus({1}) onSideSwitchStatus of AO07 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo07().setActiveOnSideSwitchStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 7);
    LOG.finer(MessageFormat.format("{0}.updateSetPointOnSideSwitchStatus({1}) onSideSwitchStatus of AO08 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo08().setActiveOnSideSwitchStatus(new BStatusBoolean((newValue == 1), status));
  }



  public void updateActiveSavePositionStatus() {
    Double savePositionStatus = getActiveSavePositionStatus().getValue();
    BStatus status = getActiveSavePositionStatus().getStatus();
    int positionConfig = savePositionStatus.intValue();
    int newValue = positionConfig & 1;
    LOG.finer(MessageFormat.format("{0}.updateActiveSavePositionStatus({1}) onSideSwitchStatus of AO01 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo01().setActiveSavePositionStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 1) & 1;
    LOG.finer(MessageFormat.format("{0}.updateActiveSavePositionStatus({1}) onSideSwitchStatus of AO02 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo02().setActiveSavePositionStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 2) & 1;
    LOG.finer(MessageFormat.format("{0}.updateActiveSavePositionStatus({1}) onSideSwitchStatus of AO03 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo03().setActiveSavePositionStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 3) & 1;
    LOG.finer(MessageFormat.format("{0}.updateActiveSavePositionStatus({1}) onSideSwitchStatus of AO04 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo04().setActiveSavePositionStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 4) & 1;
    LOG.finer(MessageFormat.format("{0}.updateActiveSavePositionStatus({1}) onSideSwitchStatus of AO05 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo05().setActiveSavePositionStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 5) & 1;
    LOG.finer(MessageFormat.format("{0}.updateActiveSavePositionStatus({1}) onSideSwitchStatus of AO06 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo06().setActiveSavePositionStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 6) & 1;
    LOG.finer(MessageFormat.format("{0}.updateActiveSavePositionStatus({1}) onSideSwitchStatus of AO07 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo07().setActiveSavePositionStatus(new BStatusBoolean((newValue == 1), status));
    newValue = (positionConfig >> 7);
    LOG.finer(MessageFormat.format("{0}.updateActiveSavePositionStatus({1}) onSideSwitchStatus of AO08 is {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), newValue));
    getAo08().setActiveSavePositionStatus(new BStatusBoolean((newValue == 1), status));
  }

  public boolean isSavePositionStatusModified() {
    BStatusNumeric active = getActiveSavePositionStatus();
    BStatusNumeric setPoint = getSetPointSavePositionStatus();
    return (active.getValue() != setPoint.getValue());
  }

  private boolean isSavePositionStatusModifiedOf(BAoPoint point) {
    BStatusBoolean  active = point.getActiveSavePositionStatus();
    BStatusBoolean setpoint = point.getSetPointSavePositionStatus();
    return active.getValue() != setpoint.getValue();
  }

  private boolean isSavePositionValueModifiedOf(BAoPoint point) {
    BStatusNumeric active = point.getActiveSavePositionValue();
    BStatusNumeric setpoint = point.getSetPointSavePositionValue();

    return active.getValue() != setpoint.getValue();
  }


  private void writeOnSideSwitchStatus() {
    LOG.config(MessageFormat.format("{0}.writeonSideSwitchStatus({1}) upate onSideSwitchStatus",
                                    LOG_PREFIX, getDeviceDisplayName(null)));
    BComponent preset = getOnSideSwitchStatusPreset();
    if(preset instanceof BModbusClientPresetRegisters) {
      BModbusClientPresetRegisters onSideSwitchStatus = (BModbusClientPresetRegisters)preset;
      AsyncBasicConfiguration.checkPresetWriteStatus(onSideSwitchStatus, this);
      onSideSwitchStatus.doWrite();
    }
  }

  private void writeOnSideSwitchValue() {
    LOG.config(MessageFormat.format("{0}.writeonSideSwitchValue{1}) upate onSideSwitchValue",
                                    LOG_PREFIX, getDeviceDisplayName(null)));
    BComponent preset = getOnSideSwitchValuePreset();
    if(preset instanceof BModbusClientPresetRegisters) {
      BModbusClientPresetRegisters onSideSwitchValue = (BModbusClientPresetRegisters)preset;
      AsyncBasicConfiguration.checkPresetWriteStatus(onSideSwitchValue, this);
      onSideSwitchValue.doWrite();
    }
  }

  private void writeSavePositionStatus() {
    LOG.config(MessageFormat.format("{0}.writeSavePositionStatus({1}) upate SavePositionStatus",
                                    LOG_PREFIX, getDeviceDisplayName(null)));
    BComponent preset = getSavePositionStatusPreset();
    if(preset instanceof BModbusClientPresetRegisters) {
      BModbusClientPresetRegisters savePositionStatus = (BModbusClientPresetRegisters)preset;
      AsyncBasicConfiguration.checkPresetWriteStatus(savePositionStatus, this);
      savePositionStatus.doWrite();
    }
  }

  private void writeSavePositionValue() {
    writeSavePositionValueOf(getSavePositionValue01Preset());
    writeSavePositionValueOf(getSavePositionValue02Preset());
    writeSavePositionValueOf(getSavePositionValue03Preset());
    writeSavePositionValueOf(getSavePositionValue04Preset());
    writeSavePositionValueOf(getSavePositionValue05Preset());
    writeSavePositionValueOf(getSavePositionValue06Preset());
    writeSavePositionValueOf(getSavePositionValue07Preset());
    writeSavePositionValueOf(getSavePositionValue08Preset());

  }

  private void writeSavePositionValueOf(BComponent preset) {
    LOG.config(MessageFormat.format("{0}.writeSavePositionValueOf({1}) upate {2}",
                                    LOG_PREFIX, getDeviceDisplayName(null), preset.getName()));
    if(preset instanceof BModbusClientPresetRegisters) {
      BModbusClientPresetRegisters savePositionValue = (BModbusClientPresetRegisters)preset;

      savePositionValue.doWrite();
    }
  }


  public BComponent getOnSideSwitchStatusRegister() {
    return getRegisterByName("onSideSwitchStatus");
  }
  public BComponent getOnSideSwitchValueRegister() {
    return getRegisterByName("onSideSwitchValue");
  }
  public BComponent getSavePositionStatusRegister() {
    return getRegisterByName("savePositionStatus");
  }

  private BComponent getFeedback01Register() {return getRegisterByName("Feedback01");}
  private BComponent getFeedback02Register() {return getRegisterByName("Feedback02");}
  private BComponent getFeedback03Register() {return getRegisterByName("Feedback03");}
  private BComponent getFeedback04Register() {return getRegisterByName("Feedback04");}
  private BComponent getFeedback05Register() {return getRegisterByName("Feedback05");}
  private BComponent getFeedback06Register() {return getRegisterByName("Feedback06");}
  private BComponent getFeedback07Register() {return getRegisterByName("Feedback07");}
  private BComponent getFeedback08Register() {return getRegisterByName("Feedback08");}

  private BComponent getSavePositionValue01Register() {
    return getRegisterByName("savePositionValue01");
  }
  private BComponent getSavePositionValue02Register() {
    return getRegisterByName("savePositionValue02");
  }
  private BComponent getSavePositionValue03Register() {
    return getRegisterByName("savePositionValue03");
  }
  private BComponent getSavePositionValue04Register() {
    return getRegisterByName("savePositionValue04");
  }
  private BComponent getSavePositionValue05Register() {
    return getRegisterByName("savePositionValue05");
  }
  private BComponent getSavePositionValue06Register() {
    return getRegisterByName("savePositionValue06");
  }
  private BComponent getSavePositionValue07Register() {
    return getRegisterByName("savePositionValue07");
  }
  private BComponent getSavePositionValue08Register() {
    return getRegisterByName("savePositionValue08");
  }

  public BComponent getAutobaudingPreset() {
    return getRegisterByName("autobaudingPreset");
  }

  public BComponent getBaudratePreset() {
    return getRegisterByName("baudratePreset");
  }

  public BComponent getTimeoutPreset() {
    return getRegisterByName("timeoutSecPreset");
  }

  public BComponent getStatusLedPreset() {
    return getRegisterByName("statusLedPreset");
  }

  public BComponent getModuleCmdPreset() {
    return getRegisterByName("moduleCmdPreset");
  }

  private BComponent getOnSideSwitchStatusPreset() {
    return getRegisterByName("onSideSwitchStatusPreset");
  }

  private BComponent getOnSideSwitchValuePreset() {
    return getRegisterByName("onSideSwitchValuePreset");
  }

  private BComponent getSavePositionStatusPreset() {
    return getRegisterByName("savePositionStatusPreset");
  }

  private BComponent getSavePositionValue01Preset() {
    return getRegisterByName("savePositionValue01Preset");
  }

  private BComponent getSavePositionValue02Preset() {
    return getRegisterByName("savePositionValue02Preset");
  }

  private BComponent getSavePositionValue03Preset() {
    return getRegisterByName("savePositionValue03Preset");
  }

  private BComponent getSavePositionValue04Preset() {
    return getRegisterByName("savePositionValue04Preset");
  }

  private BComponent getSavePositionValue05Preset() {
    return getRegisterByName("savePositionValue05Preset");
  }

  private BComponent getSavePositionValue06Preset() {
    return getRegisterByName("savePositionValue06Preset");
  }

  private BComponent getSavePositionValue07Preset() {
    return getRegisterByName("savePositionValue07Preset");
  }

  private BComponent getSavePositionValue08Preset() {
    return getRegisterByName("savePositionValue08Preset");
  }

  private BComponent getAutomaticCmdPresets() {
    return getRegisterByName("automaticCmdPreset");
  }

  private BComponent getAutomaticCmd01Preset() {
    BComponent container = getAutomaticCmdPresets();
    return (BComponent)container.get("autoCmd01");
  }

  private BComponent getAutomaticCmd02Preset() {
    BComponent container = getAutomaticCmdPresets();
    return (BComponent)container.get("autoCmd02");
  }

  private BComponent getAutomaticCmd03Preset() {
    BComponent container = getAutomaticCmdPresets();
    return (BComponent)container.get("autoCmd03");
  }

  private BComponent getAutomaticCmd04Preset() {
    BComponent container = getAutomaticCmdPresets();
    return (BComponent)container.get("autoCmd04");
  }

  private BComponent getAutomaticCmd05Preset() {
    BComponent container = getAutomaticCmdPresets();
    return (BComponent)container.get("autoCmd05");
  }

  private BComponent getAutomaticCmd06Preset() {
    BComponent container = getAutomaticCmdPresets();
    return (BComponent)container.get("autoCmd06");
  }

  private BComponent getAutomaticCmd07Preset() {
    BComponent container = getAutomaticCmdPresets();
    return (BComponent)container.get("autoCmd07");
  }

  private BComponent getAutomaticCmd08Preset() {
    BComponent container = getAutomaticCmdPresets();
    return (BComponent)container.get("autoCmd08");
  }

  @Override
  public void updateSetPointSavePositionStatus() {

    setSetPointSavePositionStatus(new BStatusNumeric(calcSavePositionStatus()));
  }


  public int calcSavePositionStatus() {

    int result = 0;

    boolean[] bits = new boolean[] {
      getAo01().getSetPointSavePositionStatus().getValue(), getAo02().getSetPointSavePositionStatus().getValue(),
      getAo03().getSetPointSavePositionStatus().getValue(), getAo04().getSetPointSavePositionStatus().getValue(),
      getAo05().getSetPointSavePositionStatus().getValue(), getAo06().getSetPointSavePositionStatus().getValue(),
      getAo07().getSetPointSavePositionStatus().getValue(), getAo08().getSetPointSavePositionStatus().getValue()};

    for (int index = 0; index < bits.length; index++) {
      result |= (bits[index]) ? (1 << index) : 0;
    }
    return result;
  }

  public BModbusAsyncDevice getDevice() {
    BComplex parent = getParent();
    if (parent instanceof BComponent) {
      return (BModbusAsyncDevice)parent;
    }
    return null;
  }

  protected BStatus getDeviceStatus() {

    return AsyncBasicConfiguration.getDeviceStatus(getDevice());
  }

  protected boolean isDeviceEnabeld() {

    return AsyncBasicConfiguration.isDeviceEnabeld(getDevice());
  }

  protected boolean isDeviceOk() {

    return AsyncBasicConfiguration.isDeviceOk(getDevice());
  }

  public String getDeviceName() {

    return AsyncBasicConfiguration.getDeviceName(getDevice());
  }

  @Override
  public String getDeviceDisplayName(Context ctx) {

    return AsyncBasicConfiguration.getDeviceDisplayName(getDevice(), ctx);
  }

  @Override
  protected BComponent[] getRegister() {

    return AsyncBasicConfiguration.getRegister(getDevice());
  }

  /**
   * execute the write action of the BModbusClientPresetRegisters returned by
   * {@link #getAutomaticCmdPresets()} and saves the execution time in {@link #lastAutoCmdWrite}
   * also reset the the {@link #maxSendTicket}
   */
  public void doWriteAutoCmd() {

    if (isConfigurationValid()) {

      BComponent autoCmd = getAutomaticCmdPresets();

      if (autoCmd instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters autoCmdPresets = (BModbusClientPresetRegisters)autoCmd;

        AsyncBasicConfiguration.checkPresetWriteStatus(autoCmdPresets, this);

        LOG.finer(MessageFormat.format("{0}.doWriteAutoCmd({1}) write AutoCmd Presets", LOG_PREFIX, getDeviceName()));

        autoCmdPresets.write();
        restartMaxSendTicket();
      }
      else {
        LOG.warning(MessageFormat.format("{0}.doWriteAutoCmd(): Object was not of type BModbusClientPresetRegisters",
                                      LOG_PREFIX));
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.doWriteAutoCmd(): Config was not valid. No write was executed", LOG_PREFIX));
    }
  }

  @Override
  public void autoCmdChanged() {
    synchronized (autoCmdMutex) {

      if (!isAutoCmdWriteLocked()) {
        LOG.finer(MessageFormat.format("{0}.autoCmdChanged({1}) write is unlocked", LOG_PREFIX, getDeviceName()));
        lockAutoCmdWrite();
        doWriteAutoCmd();
      }
      else {
        LOG.finer(MessageFormat.format("{0}.autoCmdChanged({1}) write is locked!!", LOG_PREFIX, getDeviceName()));
      }
    }
  }
}
