package com.oas.modbusutilities.configuration;
import java.text.MessageFormat;
import java.util.logging.Logger;

import javax.baja.nre.annotations.NiagaraType;
import javax.baja.status.BStatus;
import javax.baja.status.BStatusBoolean;
import javax.baja.status.BStatusNumeric;
import javax.baja.sys.BComplex;
import javax.baja.sys.BComponent;
import javax.baja.sys.BDouble;
import javax.baja.sys.BLink;
import javax.baja.sys.Context;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.oas.modbusutilities.enums.BOasAutobauding;
import com.oas.modbusutilities.enums.BOasBautrate;
import com.oas.modbusutilities.enums.BOasLedColor;
import com.oas.modbusutilities.enums.BOasModuleCmd;
import com.oas.modbusutilities.utils.AsyncLinkUtils;
import com.oas.modbusutilities.utils.LinkUtils;
import com.tridium.modbusAsync.BModbusAsyncDevice;
import com.tridium.modbusAsync.BModbusAsyncDeviceFolder;
import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegisters;


/**
 * BDiConfiguration
 */
@SuppressWarnings({
  "squid:S2629", // Invoke LOGs conditionally will bloat up the code
  "squid:S110" // Niagara Framework has more inherit lvl than 6
})
@NiagaraType
public class BAsyncDiConfiguration
  extends BAbstractDiConfiguration
  implements AsyncConfiguration
{
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.oas.modbusutilities.configuration.BAsyncDiConfiguration(2979906276)1.0$ @*/
/* Generated Mon Mar 23 11:52:00 CET 2020 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////

  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BAsyncDiConfiguration.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/


  private static final Logger LOG = Logger.getLogger("oasModbusUtilities");
  private static final String LOG_PREFIX = BAsyncDiConfiguration.class.getSimpleName();

  @Override
  public boolean isParentLegal(BComponent parent) {

    return (parent instanceof BModbusAsyncDeviceFolder) ||
           (parent instanceof BModbusAsyncDevice);
  }

  @Override
  public void changed(Property property, Context context) {
    super.changed(property, context);

    if (!isRunning()) {
      return;
    }

    if (property.equals(Input)) {
      updateInputValues();
    }
    else if (property.equals(activeLedGreenValue) || property.equals(activeLedRedValue)) {
        check();
    }
    else if (property.equals(activeBounceProtectionValue)) {
        check();
    }
    else if (property.equals(activeSwitchLogicValue)) {
        check();
    }

  }


  @Override
  public void descendantsStarted() throws Exception {
    super.descendantsStarted();
    updateSetPointLedColors();
    updateBounceProtection();
    updateSwitchLogic();
  }

  @Override
  public void doWriteConfiguration() {
    super.doWriteConfiguration();
    if (!isRunning()) {
      return;
    }

    if (isModuleTypeValid()) {

      writeRedLedValue();
      writeGreenLedValue();
      writeSwitchLogicValue();
      writeBounceProtectionValue();
    }
  }

  @Override
  public void doWriteModuleCmd(BOasModuleCmd cmd) {
    LOG.info(MessageFormat.format("{0}.doWriteModuleCmd(): upate to {1}", LOG_PREFIX, cmd));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getModuleCmdPreset(), this);
    AsyncBasicConfiguration.doWriteModuleCmd(getModuleCmdRegister(), getModuleCmdPreset(), cmd);
  }

  @Override
  protected void writeAutobinding(BOasAutobauding value) {
    LOG.info(MessageFormat.format("{0}.writeAutobinding(): upate to {1}", LOG_PREFIX, value));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getAutobaudingPreset(), this);
    AsyncBasicConfiguration.writeAutobinding(getAutobaudingPreset(), value);
  }

  @Override
  protected void writeBaudreate(BOasBautrate value) {
    LOG.info(MessageFormat.format("{0}.writeBaudreate(): upate to {1}", LOG_PREFIX, value));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getBaudratePreset(), this);
    AsyncBasicConfiguration.writeBaudreate(getBaudratePreset(), value);
  }

  @Override
  protected void writeTimeout(BDouble value) {
    LOG.info(MessageFormat.format("{0}.writeTimeout(): upate to {1}", LOG_PREFIX, value.getDouble()));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getTimeoutPreset(), this);
    AsyncBasicConfiguration.writeTimeout(getTimeoutPreset());
  }

  protected void writeLedStatus() {
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getStatusLedPreset(), this);
    AsyncBasicConfiguration.writeLedStatus(getStatusLedRegister(), getStatusLedPreset(), statusLedOneValue, statusLedTwoValue);

  }

  @Override
  public void createLinksToRegister() {

    AsyncBasicConfiguration.createLinksToRegister(getDevice(), this);

    LOG.info(MessageFormat.format("{0}.createLinksToRegister({1}) {2}.{3}:create Links to Config Component",
                                    LOG_PREFIX, getDeviceDisplayName(null), getDeviceName(), this.getName()));

    BComponent ledRed = getRedLedValueRegister();
    if (null != ledRed) {
      Property srcProp = ledRed.getProperty("out");
      BLink activeLedRedValueLink = LinkUtils.createLink(ledRed, srcProp, this, activeLedRedValue);
      add("activeLedRedValueLink", activeLedRedValueLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get ActiveLedRedRegister",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent ledRedPreset = getLedRedValuePreset();
    if (null != ledRedPreset) {
      if (ledRedPreset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)ledRedPreset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "setPointLedRedValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get LedRedValuePreset",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent ledGreen = getGreenLedValueRegister();
    if (null != ledGreen) {
      Property srcProp = ledGreen.getProperty("out");
      BLink activeLedGreenValueLink = LinkUtils.createLink(ledGreen, srcProp, this, activeLedGreenValue);
      add("activeLedGreenValueLink", activeLedGreenValueLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get ActiveLedGreenValueRegister",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }


    BComponent ledGreenPreset = getLedGreenValuePreset();
    if (null != ledGreenPreset) {
      if (ledGreenPreset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)ledGreenPreset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "setPointLedGreenValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get LedGreenValuePreset",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }


    BComponent switchLogic = getSwitchLogicValueRegister();
    if (null != switchLogic) {
      Property srcProp = switchLogic.getProperty("out");
      BLink activeSwitchLogicValueLink = LinkUtils.createLink(switchLogic, srcProp, this, activeSwitchLogicValue);
      add("activeSwitchLogicValueLink", activeSwitchLogicValueLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get ActiveSwitchLogicValueRegister",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent switchLogicPreset = getSwitchLogicValuePreset();
    if (null != switchLogicPreset) {
      if (switchLogicPreset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)switchLogicPreset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "setPointSwitchLogicValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SwitchLogicValuePreset",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent bouncProt = getBounceProtectionValueRegister();
    if (null != bouncProt) {
      Property srcProp = bouncProt.getProperty("out");
      BLink activeBounceProtectionValueLink = LinkUtils.createLink(bouncProt, srcProp, this, activeBounceProtectionValue);
      add("activeBounceProtectionValueLink", activeBounceProtectionValueLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get ActiveBounceProtectionValueRegister",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }


    BComponent bouncProtPreset = getBounceProtectionValuePreset();
    if (null != bouncProtPreset) {
      if (bouncProtPreset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)bouncProtPreset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "setPointBounceProtectionValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get BounceProtectionValuePreset",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent input = getInputRegister();
    if (null != input) {
      Property srcProp = input.getProperty("out");
      BLink inputLink = LinkUtils.createLink(input, srcProp, this, Input);
      add("inputLink", inputLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get InputRegister",
              LOG_PREFIX, getDeviceDisplayName(null)));
    }
  }

  @Override
  public BConfigurationStatus checkConfiguration() {
    BConfigurationStatus newStatus = super.checkConfiguration();

    if (getDeviceStatus().isDisabled() || getDeviceStatus().isDown()) {
      return BConfigurationStatus.offline;
    }


    if (!isRunning()) {
      LOG.finest(MessageFormat.format("{0}.checkConfiguration({1}) station is not running",
                                        LOG_PREFIX, getDeviceName()));
      return BConfigurationStatus.offline;
    }

    if (isConfigurationValid()) {


      if (isRedLedValueModified()) {
        LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: RedLed is {2} instead of {3}",
                                        LOG_PREFIX, getDeviceName(),
                                        getActiveLedRedValue(), getSetPointLedRedValue()));
        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeRedLedValue();
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) RedLed is up to date", LOG_PREFIX, getDeviceName()));

      }

      if (isGreenLedValueModified()) {
        LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: GreenLed is {2} instead of {3}",
                                          LOG_PREFIX, getDeviceName(),
                                          getActiveLedGreenValue(), getSetPointLedGreenValue()));
        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeGreenLedValue();
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) GreenLed is up to date",
                                      LOG_PREFIX, getDeviceName()));

      }

      if (isSwitchLogicValueModified()) {
        LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: Switch Logic is {2} instead of {3}",
                                          LOG_PREFIX, getDeviceName(),
                                          getActiveSwitchLogicValue(), getSetPointSwitchLogicValue()));
        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeSwitchLogicValue();
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) SwitchLogic is up to date",
                                      LOG_PREFIX, getDeviceName()));

      }

      if (isBounceProtValueModified()) {
        LOG.warning(MessageFormat.format("{0}.doCheck({1}) configuration missmatch: Bounce Prot is {2} instead of {3}",
                                          LOG_PREFIX, getDeviceName(),
                                          getActiveBounceProtectionValue(), getSetPointBounceProtectionValue()));
        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeBounceProtectionValue();
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) BounceProt is up to date",
                                      LOG_PREFIX, getDeviceName()));

      }
    }
    else {
      LOG.warning(MessageFormat.format(
                  "{0}.checkConfiguration({1}) either ModuleType[{2} != {3}] or Baudrate[{4} != {5}] is modified",
                  LOG_PREFIX, getDeviceName(),
                  getActiveModuleType(), getSetPointModuleType(),
                  getActiveBaudrate(), getSetPointBaudrate()));
    }


    return newStatus;
  }


  public void writeRedLedValue() {
    BComponent ledRedPreset = getLedRedValuePreset();
    if(ledRedPreset instanceof BModbusClientPresetRegisters) {
      LOG.config(MessageFormat.format("{0}.updateLedRedColor({1}) update with {2}",
                                      LOG_PREFIX, getDeviceDisplayName(null), getSetPointLedRedValue().getNumeric()));
      BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)ledRedPreset;
      AsyncBasicConfiguration.checkPresetWriteStatus(preset, this);

      preset.doWrite();
    }
    else {
      LOG.warning(MessageFormat.format("{0}.updateLedRedColor({1}) could not find Red Led Preset",
                                      LOG_PREFIX, getDeviceDisplayName(null)));
    }
  }

  public void writeGreenLedValue() {
    BComponent ledGreenPreset = getLedGreenValuePreset();
    if(ledGreenPreset instanceof BModbusClientPresetRegisters) {
      LOG.config(MessageFormat.format("{0}.updateLedGreenColor({1}) update with {2}",
                                      LOG_PREFIX, getDeviceDisplayName(null), getSetPointLedGreenValue().getNumeric()));
      BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)ledGreenPreset;
      AsyncBasicConfiguration.checkPresetWriteStatus(preset, this);
      preset.doWrite();
    }
    else {
      LOG.warning(MessageFormat.format("{0}.updateLedGreenColor({1}) could not find Green Led Preset",
                                      LOG_PREFIX, getDeviceDisplayName(null)));
    }
  }

  private void writeSwitchLogicValue() {
    BComponent switchLogicPreset = getSwitchLogicValuePreset();
    if (switchLogicPreset instanceof BModbusClientPresetRegisters) {
      LOG.config(MessageFormat.format("{0}.updateSwitchLogic({1}) update with {2}",
                                      LOG_PREFIX, getDeviceDisplayName(null), getSetPointSwitchLogicValue().getNumeric()));
      BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)switchLogicPreset;
      AsyncBasicConfiguration.checkPresetWriteStatus(preset, this);
      preset.doWrite();
    }
    else {
      LOG.warning(MessageFormat.format("{0}.updateSwitchLogic({1}) could not find SwitchLogic Preset",
                                      LOG_PREFIX, getDeviceDisplayName(null)));
    }
  }


  private void writeBounceProtectionValue() {
    BComponent bounceProtectionPreset = getBounceProtectionValuePreset();
    if (bounceProtectionPreset instanceof BModbusClientPresetRegisters) {
      LOG.config(MessageFormat.format("{0}.updateBounceProtection({1}) update with {2}",
                                      LOG_PREFIX, getDeviceDisplayName(null), getSetPointBounceProtectionValue().getNumeric()));
      BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)bounceProtectionPreset;
      AsyncBasicConfiguration.checkPresetWriteStatus(preset, this);
      preset.doWrite();
    }
    else {
      LOG.warning(MessageFormat.format("{0}.updateBounceProtection({1}) could not find BounceProtection Preset",
                                      LOG_PREFIX, getDeviceDisplayName(null)));
    }
  }

  public boolean isRedLedValueModified() {
    BStatusNumeric active = getActiveLedRedValue();
    BStatusNumeric setPoint = getSetPointLedRedValue();

    return (active.getValue() != setPoint.getValue());
  }

  public boolean isGreenLedValueModified() {
    BStatusNumeric active = getActiveLedGreenValue();
    BStatusNumeric setPoint = getSetPointLedGreenValue();

    return (active.getValue() != setPoint.getValue());
  }

  private boolean isSwitchLogicValueModified() {
    BStatusNumeric active = getActiveSwitchLogicValue();
    BStatusNumeric setPoint = getSetPointSwitchLogicValue();

    return (active.getValue() != setPoint.getValue());
  }

  private boolean isBounceProtValueModified() {
    BStatusNumeric active = getActiveBounceProtectionValue();
    BStatusNumeric setPoint = getSetPointBounceProtectionValue();

    return (active.getValue() != setPoint.getValue());
  }

  public BComponent getRedLedValueRegister() {
    return getRegisterByName("ledRedValue");
  }

  public BComponent getGreenLedValueRegister() {
    return getRegisterByName("ledGreenValue");
  }

  private BComponent getSwitchLogicValueRegister() {
    return getRegisterByName("switchLogicValue");
  }

  private BComponent getBounceProtectionValueRegister() {
    return getRegisterByName("bounceProtectionValue");
  }

  public BComponent getAutobaudingPreset() {
    return getRegisterByName("autobaudingPreset");
  }

  public BComponent getBaudratePreset() {
    return getRegisterByName("baudratePreset");
  }

  public BComponent getTimeoutPreset() {
    return getRegisterByName("timeoutSecPreset");
  }

  public BComponent getStatusLedPreset() {
    return getRegisterByName("statusLedPreset");
  }

  public BComponent getModuleCmdPreset() {
    return getRegisterByName("moduleCmdPreset");
  }

  private BComponent getLedRedValuePreset() {
    return getRegisterByName("ledRedValuePreset");
  }

  private BComponent getLedGreenValuePreset() {
    return getRegisterByName("ledGreenValuePreset");
  }

  private BComponent getSwitchLogicValuePreset() {
    return getRegisterByName("switchLogicValuePreset");
  }

  private BComponent getBounceProtectionValuePreset() {
    return getRegisterByName("bounceProtectionValuePreset");
  }

  private BComponent getInputRegister() {
    return getRegisterByName("Inputs");
  }

  public void updateSetPointLedColors() {
    int green = calcGreenLedValue();
    int red = calcRedLedValue();
    LOG.info(MessageFormat.format("{0}.updateLedColor({1}) update green to {2} and red to {3}",
                                  LOG_PREFIX, getDeviceDisplayName(null), green, red));
    setSetPointLedGreenValue(new BStatusNumeric(green));
    setSetPointLedRedValue(new BStatusNumeric(red));
  }

  public void updateSwitchLogic() {
    int temp = calcSwitchLogic();

    setSetPointSwitchLogicValue(new BStatusNumeric(temp));
  }

  public void updateBounceProtection() {
    int temp = calcBounceProtection();

    setSetPointBounceProtectionValue(new BStatusNumeric(temp));
  }

  private void updateInputValues() {

    Double inputValuePosition = getInput().getValue();
    BStatus status = getInput().getStatus();
    int bits = inputValuePosition.intValue();

    LOG.finer(MessageFormat.format("{0}.updateInputValues({1}) current Value from {2} | {3}",
            LOG_PREFIX,
            getDeviceName(),
            Integer.toString(bits, 2),
            bits));

    getDi01().setInput(
            new BStatusBoolean(getBitFromInt(bits, 0), status));
    getDi02().setInput(
            new BStatusBoolean(getBitFromInt(bits, 1), status));
    getDi03().setInput(
            new BStatusBoolean(getBitFromInt(bits, 2), status));
    getDi04().setInput(
            new BStatusBoolean(getBitFromInt(bits, 3), status));
    getDi05().setInput(
            new BStatusBoolean(getBitFromInt(bits, 4), status));
    getDi06().setInput(
            new BStatusBoolean(getBitFromInt(bits, 5), status));
    getDi07().setInput(
            new BStatusBoolean(getBitFromInt(bits, 6), status));
    getDi08().setInput(
            new BStatusBoolean(getBitFromInt(bits, 7), status));
    getDi09().setInput(
            new BStatusBoolean(getBitFromInt(bits, 8), status));
    getDi10().setInput(
            new BStatusBoolean(getBitFromInt(bits, 9), status));
    getDi11().setInput(
            new BStatusBoolean(getBitFromInt(bits, 10), status));
    getDi12().setInput(
            new BStatusBoolean(getBitFromInt(bits, 11), status));
    getDi13().setInput(
            new BStatusBoolean(getBitFromInt(bits, 12), status));
    getDi14().setInput(
            new BStatusBoolean(getBitFromInt(bits, 13), status));
    getDi15().setInput(
            new BStatusBoolean(getBitFromInt(bits, 14), status));
    getDi16().setInput(
            new BStatusBoolean(getBitFromInt(bits, 15), status));
  }

  private boolean getBitFromInt (int inInt, int bitPosition) {
    int bit = (inInt >> bitPosition) & 1;
    return (bit == 1);
  }

  public int calcGreenLedValue() {

    int result = 0;

    int[] bits = new int[] {
      getDi01().getLedColor().getOrdinal(), getDi02().getLedColor().getOrdinal(),
      getDi03().getLedColor().getOrdinal(), getDi04().getLedColor().getOrdinal(),
      getDi05().getLedColor().getOrdinal(), getDi06().getLedColor().getOrdinal(),
      getDi07().getLedColor().getOrdinal(), getDi08().getLedColor().getOrdinal(),
      getDi09().getLedColor().getOrdinal(), getDi10().getLedColor().getOrdinal(),
      getDi11().getLedColor().getOrdinal(), getDi12().getLedColor().getOrdinal(),
      getDi13().getLedColor().getOrdinal(), getDi14().getLedColor().getOrdinal(),
      getDi15().getLedColor().getOrdinal(), getDi16().getLedColor().getOrdinal()
    };

    for (int index = 0; index < bits.length; index++) {
      result |= (bits[index] == BOasLedColor.GREEN || bits[index] == BOasLedColor.YELLOW) ? (1 << index) : 0;
    }

    return result;
  }

  public int calcRedLedValue() {
    int result = 0;

    int[] bits = new int[] {
      getDi01().getLedColor().getOrdinal(), getDi02().getLedColor().getOrdinal(),
      getDi03().getLedColor().getOrdinal(), getDi04().getLedColor().getOrdinal(),
      getDi05().getLedColor().getOrdinal(), getDi06().getLedColor().getOrdinal(),
      getDi07().getLedColor().getOrdinal(), getDi08().getLedColor().getOrdinal(),
      getDi09().getLedColor().getOrdinal(), getDi10().getLedColor().getOrdinal(),
      getDi11().getLedColor().getOrdinal(), getDi12().getLedColor().getOrdinal(),
      getDi13().getLedColor().getOrdinal(), getDi14().getLedColor().getOrdinal(),
      getDi15().getLedColor().getOrdinal(), getDi16().getLedColor().getOrdinal(),
    };


    for (int index = 0; index < bits.length; index++) {
      result |= (bits[index] == BOasLedColor.RED || bits[index] == BOasLedColor.YELLOW) ? (1 << index) : 0;
    }


    return result;
  }

  private int calcSwitchLogic() {
    int result = 0;

    boolean[] bits = new boolean[] {
      getDi01().getSwitchLogic().getValue(), getDi02().getSwitchLogic().getValue(),
      getDi03().getSwitchLogic().getValue(), getDi04().getSwitchLogic().getValue(),
      getDi05().getSwitchLogic().getValue(), getDi06().getSwitchLogic().getValue(),
      getDi07().getSwitchLogic().getValue(), getDi08().getSwitchLogic().getValue(),
      getDi09().getSwitchLogic().getValue(), getDi10().getSwitchLogic().getValue(),
      getDi11().getSwitchLogic().getValue(), getDi12().getSwitchLogic().getValue(),
      getDi13().getSwitchLogic().getValue(), getDi14().getSwitchLogic().getValue(),
      getDi15().getSwitchLogic().getValue(), getDi16().getSwitchLogic().getValue()
    };


    for (int index = 0; index < bits.length; index++) {
      result |= (bits[index]) ? (1 << index) : 0;
    }

    return result;
  }

  private int calcBounceProtection() {
    int result = 0;

    boolean[] bits = new boolean[] {
      getDi01().getBounceProtection().getValue(), getDi02().getBounceProtection().getValue(),
      getDi03().getBounceProtection().getValue(), getDi04().getBounceProtection().getValue(),
      getDi05().getBounceProtection().getValue(), getDi06().getBounceProtection().getValue(),
      getDi07().getBounceProtection().getValue(), getDi08().getBounceProtection().getValue(),
      getDi09().getBounceProtection().getValue(), getDi10().getBounceProtection().getValue(),
      getDi11().getBounceProtection().getValue(), getDi12().getBounceProtection().getValue(),
      getDi13().getBounceProtection().getValue(), getDi14().getBounceProtection().getValue(),
      getDi15().getBounceProtection().getValue(), getDi16().getBounceProtection().getValue()
    };

    for (int index = 0; index < bits.length; index++) {
      result |= (bits[index]) ? (1 << index) : 0;
    }
    return result;
  }

  public BModbusAsyncDevice getDevice() {
    BComplex parent = getParent();
    if (parent instanceof BComponent) {
      return (BModbusAsyncDevice)parent;
    }
    return null;
  }

  protected BStatus getDeviceStatus() {

    return AsyncBasicConfiguration.getDeviceStatus(getDevice());
  }

  protected boolean isDeviceEnabeld() {

    return AsyncBasicConfiguration.isDeviceEnabeld(getDevice());
  }

  protected boolean isDeviceOk() {

    return AsyncBasicConfiguration.isDeviceOk(getDevice());
  }

  public String getDeviceName() {

    return AsyncBasicConfiguration.getDeviceName(getDevice());
  }

  @Override
  public String getDeviceDisplayName(Context ctx) {

    return AsyncBasicConfiguration.getDeviceDisplayName(getDevice(), ctx);
  }

  @Override
  protected BComponent[] getRegister() {

    return AsyncBasicConfiguration.getRegister(getDevice());

  }



}
