package com.oas.modbusutilities.configuration;
import java.text.MessageFormat;
import java.util.logging.Logger;

import javax.baja.nre.annotations.NiagaraType;
import javax.baja.status.BStatus;
import javax.baja.status.BStatusBoolean;
import javax.baja.status.BStatusNumeric;
import javax.baja.sys.BComplex;
import javax.baja.sys.BComponent;
import javax.baja.sys.BDouble;
import javax.baja.sys.BLink;
import javax.baja.sys.Context;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.oas.modbusutilities.enums.BOasAutobauding;
import com.oas.modbusutilities.enums.BOasBautrate;
import com.oas.modbusutilities.enums.BOasLedColor;
import com.oas.modbusutilities.enums.BOasModuleCmd;
import com.oas.modbusutilities.utils.AsyncLinkUtils;
import com.oas.modbusutilities.utils.LinkUtils;
import com.tridium.modbusAsync.BModbusAsyncDevice;
import com.tridium.modbusAsync.BModbusAsyncDeviceFolder;
import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegisters;


/**
 * BDoConfiguration
 */

@SuppressWarnings({
  "squid:S2629", // Invoke LOGs conditionally will bloat up the code
  "squid:S110" // Niagara Framework has more inherit lvl than 6
})
@NiagaraType
public class BAsyncDoConfiguration
  extends BAbstractDoConfiguration
  implements AsyncConfiguration
{
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.oas.modbusutilities.configuration.BAsyncDoConfiguration(2979906276)1.0$ @*/
/* Generated Thu Apr 02 16:24:03 CEST 2020 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////

  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BAsyncDoConfiguration.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/


  private static final Logger LOG = Logger.getLogger("oasModbusUtilities");
  private static final String LOG_PREFIX = BAsyncDoConfiguration.class.getSimpleName();

  @Override
  public boolean isParentLegal(BComponent parent) {

    return (parent instanceof BModbusAsyncDeviceFolder) ||
           (parent instanceof BModbusAsyncDevice);
  }

  @Override
  public void changed(Property property, Context context) {
    super.changed(property, context);

    if (!isRunning()) {
      return;
    }

    if (property.equals(activeOnSideSwitchValue)) {
      updateActiveOnSideSwitchValue();

      Double onSideSwitchValue = getActiveOnSideSwitchValue().getValue();
      int value = onSideSwitchValue.intValue();

      Double onSideSwitchStatus = getActiveOnSideSwitchStatus().getValue();
      int status = onSideSwitchStatus.intValue();

      if (isOnSideSwitchConfigured(value, status)){

        setOnSideSwitchStatusOn();
      }
      else {
        setOnSideSwitchStatusOff();
      }
    }
    else if (property.equals(activeOnSideSwitchStatus)) {
      updateActiveOnSideSwitchStatus();

      Double onSideSwitchValue = getActiveOnSideSwitchValue().getValue();
      int value = onSideSwitchValue.intValue();

      Double onSideSwitchStatus = getActiveOnSideSwitchStatus().getValue();
      int status = onSideSwitchStatus.intValue();

      if (isOnSideSwitchConfigured(value, status)){

        setOnSideSwitchStatusOn();
      }
      else {
        setOnSideSwitchStatusOff();
      }
    }
    else if (property.equals(feedbackValue)) {
      updateFeedbackValue();
    }
    else if (property.equals(maxWriteTime)) {
      BModbusAsyncDevice device = getDevice();
      if (device.getEnabled()) {
        restartMaxSendTicket();
      }
    }
  }


  @Override
  public void descendantsStarted() throws Exception {
    super.descendantsStarted();

    updateSetPointLedColors();

    updateSetPointSavePositionStatus();
    updateSetPointSavePositionValue();

    updateFeedbackValue();

    updateAutomaticCmd();
    restartMaxSendTicket();
  }



  @Override
  public void doWriteConfiguration() {
    super.doWriteConfiguration();

    if (!isRunning()) {
      return;
    }

    if (isModuleTypeValid()) {

      writeRedLedValue();
      writeGreenLedValue();
      writeSavePositionStatus();
      writeSavePositionValue();
    }
  }

  @Override
  public void doWriteModuleCmd(BOasModuleCmd cmd) {
    LOG.info(MessageFormat.format("{0}.doWriteModuleCmd(): upate to {1}", LOG_PREFIX, cmd));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getModuleCmdPreset(), this);
    AsyncBasicConfiguration.doWriteModuleCmd(getModuleCmdRegister(), getModuleCmdPreset(), cmd);
  }

  @Override
  protected void writeAutobinding(BOasAutobauding value) {
    LOG.info(MessageFormat.format("{0}.writeAutobinding(): upate to {1}", LOG_PREFIX, value));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getAutobaudingPreset(), this);
    AsyncBasicConfiguration.writeAutobinding(getAutobaudingPreset(), value);
  }

  @Override
  protected void writeBaudreate(BOasBautrate value) {
    LOG.info(MessageFormat.format("{0}.writeBaudreate(): upate to {1}", LOG_PREFIX, value));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getBaudratePreset(), this);
    AsyncBasicConfiguration.writeBaudreate(getBaudratePreset(), value);
  }

  @Override
  protected void writeTimeout(BDouble value) {
    LOG.info(MessageFormat.format("{0}.writeTimeout(): upate to {1}", LOG_PREFIX, value.getDouble()));
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getTimeoutPreset(), this);
    AsyncBasicConfiguration.writeTimeout(getTimeoutPreset());
  }

  protected void writeLedStatus() {
    AsyncBasicConfiguration.checkPresetWriteStatus((BModbusClientPresetRegisters)getStatusLedPreset(), this);
    AsyncBasicConfiguration.writeLedStatus(getStatusLedRegister(), getStatusLedPreset(), statusLedOneValue, statusLedTwoValue);
  }


  @Override
  public void createLinksToRegister() {

    AsyncBasicConfiguration.createLinksToRegister(getDevice(), this);

    LOG.info(MessageFormat.format("{0}.createLinksToRegister({1}) {2}.{3}:create Links to Config Component",
                                    LOG_PREFIX, getDeviceDisplayName(null), getDeviceName(), this.getName()));


    BComponent ledRed = getRedLedValueRegister();
    if (null != ledRed) {
      Property srcProp = ledRed.getProperty("out");
      BLink activeLedRedValueLink = LinkUtils.createLink(ledRed, srcProp, this, activeLedRedValue);
      add("activeLedRedValueLink", activeLedRedValueLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get ActiveLedRedValueRegister",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }


    BComponent ledRedPreset = getLedRedValuePreset();
    if (null != ledRedPreset) {
      if (ledRedPreset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)ledRedPreset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "setPointLedRedValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get ActiveLedRedValuePreset",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent ledGreen = getGreenLedValueRegister();
    if (null != ledGreen) {
      Property srcProp = ledGreen.getProperty("out");
      BLink activeLedGreenValueLink = LinkUtils.createLink(ledGreen, srcProp, this, activeLedGreenValue);
      add("activeLedGreenValueLink", activeLedGreenValueLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get LedGreenValueRegister",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent ledGreenPreset = getLedGreenValuePreset();
    if (null != ledGreenPreset) {
      if (ledGreenPreset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)ledGreenPreset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "setPointLedGreenValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get ActiveLedGreenValuePreset",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent onSideSwitchStatus = getOnSideSwitchStatusRegister();
    if (null != onSideSwitchStatus) {
      Property srcProp = onSideSwitchStatus.getProperty("out");
      BLink onSideSwitchLink = LinkUtils.createLink(onSideSwitchStatus, srcProp, this, activeOnSideSwitchStatus);
      add("activeonSideSwitchLink", onSideSwitchLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get onSideSwitchStatusRegister",
              LOG_PREFIX, getDeviceDisplayName(null)));
    }


    BComponent onSideSwitchValue = getOnSideSwitchValueRegister();
    if (null != onSideSwitchValue) {
      Property srcProp = onSideSwitchValue.getProperty("out");
      BLink onSideSwitchCfgLink = LinkUtils.createLink(onSideSwitchValue, srcProp, this, activeOnSideSwitchValue);
      add("onSideSwitchCfgLink", onSideSwitchCfgLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get onSideSwitchValueRegister",
              LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent savePositionCfgStatus = getSavePositionStatusRegister();
    if (null != savePositionCfgStatus) {
      Property srcProp = savePositionCfgStatus.getProperty("out");
      BLink savePositionStatusLink = LinkUtils.createLink(savePositionCfgStatus, srcProp, this, activeSavePositionStatus);
      add("savePositionStatusLink", savePositionStatusLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionCfgStatusRegister",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent savePositionCfgStatusPreset = getActiveSavePositionCfgStatusPreset();
    if (null != savePositionCfgStatusPreset) {
      if (savePositionCfgStatusPreset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)savePositionCfgStatusPreset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "setPointSavePositionStatus");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get savePositionCfgStatusPreset",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent savePositionCfgValue = getSavePositionValueRegister();
    if (null != savePositionCfgValue) {
      Property srcProp = savePositionCfgValue.getProperty("out");
      BLink savePositionValueLink = LinkUtils.createLink(savePositionCfgValue, srcProp, this, activeSavePositionValue);
      add("savePositionValueLink", savePositionValueLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get SavePositionCfgValueRegister",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent savePositionCfgValuePreset = getSavePositionCfgValuePreset();
    if (null != savePositionCfgValuePreset) {
      if (savePositionCfgValuePreset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)savePositionCfgValuePreset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "setPointSavePositionValue");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get savePositionCfgValuePreset",
                                        LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent feedback = getFeedbackRegister();
    if (null != feedback) {
      Property srcProp = feedback.getProperty("out");
      BLink feedbackLink = LinkUtils.createLink(feedback, srcProp, this, feedbackValue);
      add("feedbackLink", feedbackLink);
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get FeedbackRegister",
              LOG_PREFIX, getDeviceDisplayName(null)));
    }

    BComponent automaticCmdPreset = getAutomaticCmdPresets();
    if (null != automaticCmdPreset) {
      if (automaticCmdPreset instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters preset = (BModbusClientPresetRegisters)automaticCmdPreset;
        AsyncLinkUtils.linkToPresets(preset, (BComponent)this, "automaticCmd");
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.createLinksToRegister({1}) could not get automaticCmdPreset",
              LOG_PREFIX, getDeviceDisplayName(null)));
    }
  }

  @Override
  public BConfigurationStatus checkConfiguration() {

    BConfigurationStatus newStatus = super.checkConfiguration();

    if (getDeviceStatus().isDisabled() || getDeviceStatus().isDown()) {
      return BConfigurationStatus.offline;
    }


    if (!isRunning()) {
      LOG.finest(MessageFormat.format("{0}.checkConfiguration({1}) station is not running",
                                        LOG_PREFIX,  getDeviceName()));
      return BConfigurationStatus.offline;
    }



    if (isConfigurationValid()) {


      if (isRedLedValueModified()) {
        LOG.warning(MessageFormat.format(
                              "{0}.checkConfiguration({1}) configuration missmatch: Red Led is {2} instead of {3}",
                              LOG_PREFIX, getDeviceDisplayName(null), getActiveLedRedValue(), getSetPointLedRedValue()));
        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeRedLedValue();
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) Red Led is up to date",
                                      LOG_PREFIX, getDeviceDisplayName(null)));

      }
      // setPointCfgGreenLed & activeCfgGreenLed
      if (isGreenLedValueModified()) {
        LOG.fine(MessageFormat.format(
                            "{0}.checkConfiguration({1}) configuration missmatch: Cfg Green Led is {2} instead of {3}",
                            LOG_PREFIX, getDeviceDisplayName(null), getActiveLedGreenValue(), getSetPointLedGreenValue()));
        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeGreenLedValue();
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) Green Led is up to date",
                                      LOG_PREFIX, getDeviceDisplayName(null)));

      }


      if (isSavePositionStatusModified()) {
        LOG.warning(MessageFormat.format(
                        "{0}.checkConfiguration({1}) configuration missmatch: savePositionStatus is {2} instead of {3}",
                        LOG_PREFIX, getDeviceDisplayName(null), getActiveSavePositionStatus(), getSetPointSavePositionStatus()));
        setConfigModifiedStatus();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        writeSavePositionStatus();
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) Save Position Status is up to date",
                                      LOG_PREFIX, getDeviceDisplayName(null)));

      }

      if (isSavePositionValueModified()) {
        LOG.warning(MessageFormat.format(
                        "{0}.checkConfiguration({1}) configuration missmatch: savePositionValue() is {2} instead of {3}",
                        LOG_PREFIX, getDeviceDisplayName(null), getActiveSavePositionValue(),
                        getSetPointSavePositionValue()));
        writeSavePositionValue();
        newStatus = BConfigurationStatus.makeModified(newStatus, true);
        setConfigModifiedStatus();
      }
      else {
        LOG.fine(MessageFormat.format("{0}.checkConfiguration({1}) Save Position Value is up to date",
                                      LOG_PREFIX, getDeviceDisplayName(null)));

      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.checkConfiguration({1}) either ModuleType[{2} != {3}] or Baudrate[{4} != {5}] is modified",
                                        LOG_PREFIX, getDevice().getName(), getActiveModuleType(), getSetPointModuleType(),
                                        getActiveBaudrate(), getSetPointBaudrate()));
    }


    return newStatus;

  }

  public void writeRedLedValue() {
    BComponent reg = getLedRedValuePreset();
    if(reg instanceof BModbusClientPresetRegisters) {
      BModbusClientPresetRegisters activeLedRed = (BModbusClientPresetRegisters)reg;
      AsyncBasicConfiguration.checkPresetWriteStatus(activeLedRed, this);
      activeLedRed.doWrite();
    }
  }

  public void writeGreenLedValue() {
    BComponent reg = getLedGreenValuePreset();
    if(reg instanceof BModbusClientPresetRegisters) {
      BModbusClientPresetRegisters activeLedGreen = (BModbusClientPresetRegisters)reg;
      AsyncBasicConfiguration.checkPresetWriteStatus(activeLedGreen, this);
      activeLedGreen.doWrite();
    }
  }

  private void writeSavePositionStatus() {
    BComponent reg = getActiveSavePositionCfgStatusPreset();
    if(reg instanceof BModbusClientPresetRegisters) {
      BModbusClientPresetRegisters savePositionStatus = (BModbusClientPresetRegisters)reg;
      AsyncBasicConfiguration.checkPresetWriteStatus(savePositionStatus, this);
      savePositionStatus.doWrite();
    }
  }

  private void writeSavePositionValue() {
    BComponent reg = getSavePositionCfgValuePreset();
    if(reg instanceof BModbusClientPresetRegisters) {
      BModbusClientPresetRegisters savePositionValue = (BModbusClientPresetRegisters)reg;
      AsyncBasicConfiguration.checkPresetWriteStatus(savePositionValue, this);
      savePositionValue.doWrite();
    }
  }

  public boolean isRedLedValueModified() {
    BStatusNumeric active = getActiveLedRedValue();
    BStatusNumeric setPoint = getSetPointLedRedValue();

    return (active.getValue() != setPoint.getValue());
  }

  public boolean isGreenLedValueModified() {
    BStatusNumeric active = getActiveLedGreenValue();
    BStatusNumeric setPoint = getSetPointLedGreenValue();

    return (active.getValue() != setPoint.getValue());
  }

  public boolean isSavePositionStatusModified() {
    BStatusNumeric active = getActiveSavePositionStatus();
    BStatusNumeric setPoint = getSetPointSavePositionStatus();
    return (active.getValue() != setPoint.getValue());
  }

  public boolean isSavePositionValueModified() {
    BStatusNumeric active = getActiveSavePositionValue();
    BStatusNumeric setPoint = getSetPointSavePositionValue();
    return (active.getValue() != setPoint.getValue());
  }

  public BComponent getRedLedValueRegister() {
    return getRegisterByName("ledRedValue");
  }

  public BComponent getGreenLedValueRegister() {
    return getRegisterByName("ledGreenValue");
  }

  public BComponent getSavePositionStatusRegister() {
    return getRegisterByName("savePositionStatus");
  }

  @Override
  public BComponent getSavePositionValueRegister() {
    return getRegisterByName("savePositionValue");
  }

  public BComponent getOnSideSwitchValueRegister() {
    return getRegisterByName("onSideSwitchValue");
  }

  public BComponent getOnSideSwitchStatusRegister() {
    return getRegisterByName("onSideSwitchStatus");
  }

  public BComponent getAutobaudingPreset() {
    return getRegisterByName("autobaudingPreset");
  }

  public BComponent getBaudratePreset() {
    return getRegisterByName("baudratePreset");
  }

  public BComponent getTimeoutPreset() {
    return getRegisterByName("timeoutSecPreset");
  }

  public BComponent getStatusLedPreset() {
    return getRegisterByName("statusLedPreset");
  }

  public BComponent getModuleCmdPreset() {
    return getRegisterByName("moduleCmdPreset");
  }

  private BComponent getLedRedValuePreset() {
    return getRegisterByName("ledRedValuePreset");
  }

  private BComponent getLedGreenValuePreset() {
    return getRegisterByName("ledGreenValuePreset");
  }

  private BComponent getActiveSavePositionCfgStatusPreset() {
    return getRegisterByName("savePositionStatusPreset");
  }

  private BComponent getSavePositionCfgValuePreset() {
    return getRegisterByName("savePositionValuePreset");
  }

  private BComponent getFeedbackRegister() { return  getRegisterByName("feedback"); }

  private BComponent getAutomaticCmdPresets() { return getRegisterByName("automaticCmdPreset"); }

  public void updateSetPointLedColors() {
    int green = calcGreenLedValue();
    int red = calcRedLedValue();
    setSetPointLedGreenValue(new BStatusNumeric(green));
    setSetPointLedRedValue(new BStatusNumeric(red));
  }

  public void updateActiveOnSideSwitchValue() {

    Double onSideSwitchValuePosition = getActiveOnSideSwitchValue().getValue();
    BStatus status = getActiveOnSideSwitchValue().getStatus();
    int bits = onSideSwitchValuePosition.intValue();

    LOG.finer(MessageFormat.format("{0}.updateActiveOnSideSwitchValue({1}) current Value from {2} | {3}",
                                  LOG_PREFIX,
                                  getDeviceName(),
                                  Integer.toString(bits, 2),
                                  bits));

    getDo01().setActiveOnSideSwitchValue(
                                      new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 0), status));
    getDo02().setActiveOnSideSwitchValue(
                                      new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 1), status));
    getDo03().setActiveOnSideSwitchValue(
                                      new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 2), status));
    getDo04().setActiveOnSideSwitchValue(
                                      new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 3), status));
    getDo05().setActiveOnSideSwitchValue(
                                      new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 4), status));
    getDo06().setActiveOnSideSwitchValue(
                                      new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 5), status));
    getDo07().setActiveOnSideSwitchValue(
                                      new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 6), status));
    getDo08().setActiveOnSideSwitchValue(
                                      new BStatusBoolean(!getOnSideSwitchAutoValue(bits, 7), status));

  }

  public void updateActiveOnSideSwitchStatus() {

    Double onSideSwitchStatusPosition = getActiveOnSideSwitchStatus().getValue();
    BStatus status = getActiveOnSideSwitchStatus().getStatus();
    int bits = onSideSwitchStatusPosition.intValue();

    LOG.finer(MessageFormat.format("{0}.updateActiveOnSideSwitchStatus({1}) current Value from {2} | {3}",
                                  LOG_PREFIX,
                                  getDeviceName(),
                                  Integer.toString(bits, 2),
                                  bits));

    getDo01().setActiveOnSideSwitchStatus(
                                    new BStatusBoolean(getOnSideSwitchStatusValue(bits, 0), status));
    getDo02().setActiveOnSideSwitchStatus(
                                    new BStatusBoolean(getOnSideSwitchStatusValue(bits, 1), status));
    getDo03().setActiveOnSideSwitchStatus(
                                    new BStatusBoolean(getOnSideSwitchStatusValue(bits, 2), status));
    getDo04().setActiveOnSideSwitchStatus(
                                    new BStatusBoolean(getOnSideSwitchStatusValue(bits, 3), status));
    getDo05().setActiveOnSideSwitchStatus(
                                    new BStatusBoolean(getOnSideSwitchStatusValue(bits, 4), status));
    getDo06().setActiveOnSideSwitchStatus(
                                    new BStatusBoolean(getOnSideSwitchStatusValue(bits, 5), status));
    getDo07().setActiveOnSideSwitchStatus(
                                    new BStatusBoolean(getOnSideSwitchStatusValue(bits, 6), status));
    getDo08().setActiveOnSideSwitchStatus(
                                    new BStatusBoolean(getOnSideSwitchStatusValue(bits, 7), status));
  }

  public void updateFeedbackValue() {

    Double feedbackValuePosition = getFeedbackValue().getValue();
    BStatus status = getFeedbackValue().getStatus();
    int bits = feedbackValuePosition.intValue();

    LOG.finer(MessageFormat.format("{0}.updateFeedbackValue({1}) current Value from {2} | {3}",
            LOG_PREFIX,
            getDeviceName(),
            Integer.toString(bits, 2),
            bits));

    getDo01().setFeedbackValue(
            new BStatusBoolean(getBitFromInt(bits, 0), status));
    getDo02().setFeedbackValue(
            new BStatusBoolean(getBitFromInt(bits, 1), status));
    getDo03().setFeedbackValue(
            new BStatusBoolean(getBitFromInt(bits, 2), status));
    getDo04().setFeedbackValue(
            new BStatusBoolean(getBitFromInt(bits, 3), status));
    getDo05().setFeedbackValue(
            new BStatusBoolean(getBitFromInt(bits, 4), status));
    getDo06().setFeedbackValue(
            new BStatusBoolean(getBitFromInt(bits, 5), status));
    getDo07().setFeedbackValue(
            new BStatusBoolean(getBitFromInt(bits, 6), status));
    getDo08().setFeedbackValue(
            new BStatusBoolean(getBitFromInt(bits, 7), status));
  }

  private boolean getBitFromInt (int inInt, int bitPosition) {
    int bit = (inInt >> bitPosition) & 1;
    return (bit == 1);
  }

  public void updateSetPointSavePositionStatus() {
    int temp = calcSavePositionStatus();
    setSetPointSavePositionStatus(new BStatusNumeric(temp));
  }

  @Override
  public void updateSetPointSavePositionValue() {
    int temp = calcSavePositionValue();
    setSetPointSavePositionValue(new BStatusNumeric(temp));
  }

  public void updateAutomaticCmd() {
    int temp = calcAutomaticCmd();
    setAutomaticCmd(new BStatusNumeric(temp));
  }

  public int calcGreenLedValue() {

    int result = 0;

    int[] bits = new int[] {
      getDo01().getLedColor().getOrdinal(), getDo02().getLedColor().getOrdinal(),
      getDo03().getLedColor().getOrdinal(), getDo04().getLedColor().getOrdinal(),
      getDo05().getLedColor().getOrdinal(), getDo06().getLedColor().getOrdinal(),
      getDo07().getLedColor().getOrdinal(), getDo08().getLedColor().getOrdinal()
    };

    for (int index = 0; index < bits.length; index++) {
      result |= (bits[index] == BOasLedColor.GREEN || bits[index] == BOasLedColor.YELLOW) ? (1 << index) : 0;
    }

    return result;
  }

  public int calcRedLedValue() {
    int result = 0;

    int[] bits = new int[] {
      getDo01().getLedColor().getOrdinal(), getDo02().getLedColor().getOrdinal(),
      getDo03().getLedColor().getOrdinal(), getDo04().getLedColor().getOrdinal(),
      getDo05().getLedColor().getOrdinal(), getDo06().getLedColor().getOrdinal(),
      getDo07().getLedColor().getOrdinal(), getDo08().getLedColor().getOrdinal()
    };


    for (int index = 0; index < bits.length; index++) {
      result |= (bits[index] == BOasLedColor.RED || bits[index] == BOasLedColor.YELLOW) ? (1 << index) : 0;
    }


    return result;
  }

  public int calcSavePositionStatus() {

    int result = 0;

    boolean[] bits = new boolean[] {
      getDo01().getSavePositionStatus().getValue(), getDo02().getSavePositionStatus().getValue(),
      getDo03().getSavePositionStatus().getValue(), getDo04().getSavePositionStatus().getValue(),
      getDo05().getSavePositionStatus().getValue(), getDo06().getSavePositionStatus().getValue(),
      getDo07().getSavePositionStatus().getValue(), getDo08().getSavePositionStatus().getValue()};

      for (int index = 0; index < bits.length; index++) {
        result |= (bits[index]) ? (1 << index) : 0;
      }
    return result;
  }

  @Override
  public int calcSavePositionValue() {

    int result = 0;

    boolean[] bits = new boolean[] {
            getDo01().getSavePositionValue().getValue(), getDo02().getSavePositionValue().getValue(),
            getDo03().getSavePositionValue().getValue(), getDo04().getSavePositionValue().getValue(),
            getDo05().getSavePositionValue().getValue(), getDo06().getSavePositionValue().getValue(),
            getDo07().getSavePositionValue().getValue(), getDo08().getSavePositionValue().getValue()};

    for (int index = 0; index < bits.length; index++) {
      result |= (bits[index]) ? (1 << index) : 0;
    }
    return result;
  }

  private int calcAutomaticCmd() {

    int result = 0;

    boolean[] bits = new boolean[] {
            getDo01().getAutomaticCmd().getValue(), getDo02().getAutomaticCmd().getValue(),
            getDo03().getAutomaticCmd().getValue(), getDo04().getAutomaticCmd().getValue(),
            getDo05().getAutomaticCmd().getValue(), getDo06().getAutomaticCmd().getValue(),
            getDo07().getAutomaticCmd().getValue(), getDo08().getAutomaticCmd().getValue()};

    for (int index = 0; index < bits.length; index++) {
      result |= (bits[index]) ? (1 << index) : 0;
    }
    return result;
  }

  public BModbusAsyncDevice getDevice() {
    BComplex parent = getParent();
    if (parent instanceof BComponent) {
      return (BModbusAsyncDevice)parent;
    }
    return null;
  }

  protected BStatus getDeviceStatus() {

    return AsyncBasicConfiguration.getDeviceStatus(getDevice());
  }

  protected boolean isDeviceEnabeld() {

    return AsyncBasicConfiguration.isDeviceEnabeld(getDevice());
  }

  protected boolean isDeviceOk() {

    return AsyncBasicConfiguration.isDeviceOk(getDevice());
  }

  public String getDeviceName() {

    return AsyncBasicConfiguration.getDeviceName(getDevice());
  }

  @Override
  public String getDeviceDisplayName(Context ctx) {

    return AsyncBasicConfiguration.getDeviceDisplayName(getDevice(), ctx);
  }

  @Override
  protected BComponent[] getRegister() {

    return AsyncBasicConfiguration.getRegister(getDevice());

  }

  /**
   * execute the write action of the BModbusClientPresetRegisters returned by
   * {@link #getAutomaticCmdPresets()} and saves the execution time in {@link #lastAutoCmdWrite}
   * also reset the the {@link #maxSendTicket}
   */
  public void doWriteAutoCmd() {

    if (isConfigurationValid()) {

      BComponent autoCmd = getAutomaticCmdPresets();

      if (autoCmd instanceof BModbusClientPresetRegisters) {
        BModbusClientPresetRegisters autoCmdPresets = (BModbusClientPresetRegisters)autoCmd;

        AsyncBasicConfiguration.checkPresetWriteStatus(autoCmdPresets, this);

        LOG.finer(MessageFormat.format("{0}.doWriteAutoCmd({1}) write AutoCmd Presets", LOG_PREFIX, getDeviceName()));

        autoCmdPresets.write();
        restartMaxSendTicket();
      }
      else {
        LOG.warning(MessageFormat.format("{0}.doWriteAutoCmd(): Object was not of type BModbusClientPresetRegisters",
                                      LOG_PREFIX));
      }
    }
    else {
      LOG.warning(MessageFormat.format("{0}.doWriteAutoCmd(): Config was not valid. No write was executed", LOG_PREFIX));
    }
  }

  @Override
  public void autoCmdChanged() {
    synchronized (autoCmdMutex) {

      if (!isAutoCmdWriteLocked()) {
        LOG.finer(MessageFormat.format("{0}.autoCmdChanged({1}) write is unlocked", LOG_PREFIX, getDeviceName()));
        lockAutoCmdWrite();
        doWriteAutoCmd();
      }
      else {
        LOG.finer(MessageFormat.format("{0}.autoCmdChanged({1}) write is locked!!", LOG_PREFIX, getDeviceName()));
      }
    }
  }

}
