package com.oas.modbusutilities.devices;

import javax.baja.control.BNumericWritable;
import javax.baja.driver.util.BPollFrequency;
import javax.baja.nre.annotations.NiagaraProperty;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.status.BStatus;
import javax.baja.sys.BComponent;
import javax.baja.sys.BIcon;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.oas.modbusutilities.configuration.BAbstractConfiguration;
import com.oas.modbusutilities.configuration.BAsyncAiConfiguration;
import com.oas.modbusutilities.configuration.BConfigurationStatus;
import com.oas.modbusutilities.network.BOasModbusAsyncNetwork;
import com.oas.modbusutilities.register.AsyncRegisterFactory;
import com.oas.modbusutilities.utils.AsyncLinkUtils;
import com.tridium.modbusAsync.BModbusAsyncDevice;
import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegisters;
import com.tridium.modbusCore.client.point.BModbusClientPointDeviceExt;
import com.tridium.modbusCore.client.point.BModbusClientPointFolder;
import com.tridium.modbusCore.datatypes.BFlexAddress;
import com.tridium.modbusCore.enums.BAddressFormatEnum;

@NiagaraProperty(
  name="createFromTemplate",
  type="boolean",
  defaultValue="true",
  flags=Flags.READONLY | Flags.HIDDEN
)

@NiagaraProperty(
  name="pingAddress",
  type="BFlexAddress",
  defaultValue="new BFlexAddress(BAddressFormatEnum.decimal, \"10\")"
)

@NiagaraProperty(
  name="configuration",
  type="BAsyncAiConfiguration",
  defaultValue="new BAsyncAiConfiguration()"
)

@SuppressWarnings({
  "squid:S110" // Niagara Framework has more inherit lvl than 6
})
@NiagaraType
public class BAsyncAiModule extends BModbusAsyncDevice {
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.oas.modbusutilities.devices.BAsyncAiModule(1502846535)1.0$ @*/
/* Generated Fri Dec 04 09:54:09 CET 2020 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Property "createFromTemplate"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code createFromTemplate} property.
   * @see #getCreateFromTemplate
   * @see #setCreateFromTemplate
   */
  public static final Property createFromTemplate = newProperty(Flags.READONLY | Flags.HIDDEN, true, null);
  
  /**
   * Get the {@code createFromTemplate} property.
   * @see #createFromTemplate
   */
  public boolean getCreateFromTemplate() { return getBoolean(createFromTemplate); }
  
  /**
   * Set the {@code createFromTemplate} property.
   * @see #createFromTemplate
   */
  public void setCreateFromTemplate(boolean v) { setBoolean(createFromTemplate, v, null); }

////////////////////////////////////////////////////////////////
// Property "pingAddress"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code pingAddress} property.
   * @see #getPingAddress
   * @see #setPingAddress
   */
  public static final Property pingAddress = newProperty(0, new BFlexAddress(BAddressFormatEnum.decimal, "10"), null);
  
  /**
   * Get the {@code pingAddress} property.
   * @see #pingAddress
   */
  public BFlexAddress getPingAddress() { return (BFlexAddress)get(pingAddress); }
  
  /**
   * Set the {@code pingAddress} property.
   * @see #pingAddress
   */
  public void setPingAddress(BFlexAddress v) { set(pingAddress, v, null); }

////////////////////////////////////////////////////////////////
// Property "configuration"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code configuration} property.
   * @see #getConfiguration
   * @see #setConfiguration
   */
  public static final Property configuration = newProperty(0, new BAsyncAiConfiguration(), null);
  
  /**
   * Get the {@code configuration} property.
   * @see #configuration
   */
  public BAsyncAiConfiguration getConfiguration() { return (BAsyncAiConfiguration)get(configuration); }
  
  /**
   * Set the {@code configuration} property.
   * @see #configuration
   */
  public void setConfiguration(BAsyncAiConfiguration v) { set(configuration, v, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BAsyncAiModule.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/


  private BStatus lastStatus = BStatus.DEFAULT;
  private static final BIcon icon = BIcon.make("module://oasModbusUtilitiesCore/res/icon/module_round.png");

  @Override
  public BIcon getIcon() {
    return icon;
  }

  @Override
  public void started() throws Exception {
    super.started();


    if(Sys.isStationStarted() && getCreateFromTemplate()) {
      createRegister();
    }
  }

  @Override
  public void descendantsStarted() throws Exception {
    super.descendantsStarted();
    if(Sys.isStationStarted() && getCreateFromTemplate()) {
      setCreateFromTemplate(false);
    }

    BAbstractConfiguration config = getConfiguration();

    if (!config.isLinked()) {
      config.createLinksToRegister();
    }
  }


  @Override
  public void doPing() {
    super.doPing();

    BAbstractConfiguration config = getConfiguration();
    if (null != config) {
      if (config.isConfigurationValid() && config.isConfigurationModified()) {
        config.writeConfiguration();
      }
      else if (!config.isConfigurationValid()) {
        config.setConfigInvalidStatus();

      }
    }
  }

  @Override
  public void changed(Property property, Context context) {
    super.changed(property, context);


    if (property.equals(status)) {

      BAbstractConfiguration config = getConfiguration();
      if (getStatus().isOk()) {
        if (null != config) {
          if (lastStatus.isDown()){
              config.startCheckConfigTimer();
          }
          else if (lastStatus.isDisabled()) {
              config.startCheckConfigTimer();
              config.startStatusLedTimer();
          }
        }
      }
      else if (getStatus().isDisabled()) {
        config.setStatus(BConfigurationStatus.offline);
      }
      lastStatus = getStatus();
    }
    else if (property.equals(deviceAddress)) {
      BAbstractConfiguration config = getConfiguration();

      config.doCheck();
    }
  }

  @Override
  public boolean isParentLegal(BComponent parent) {
    return (parent instanceof BOasModbusAsyncNetwork);
  }


  private final void createRegister() {
    BModbusClientPointDeviceExt      pointExt = getPoints();

    //move all Register and Presets into it's own PointFolder
    BModbusClientPointFolder pointFolder = new BModbusClientPointFolder();
    pointExt.add("Points", pointFolder);

    pointFolder.add("autobaudingPreset",  AsyncRegisterFactory.makeSingleIntegerPresets("3", false));
    pointFolder.add("Autobauding",        AsyncRegisterFactory.makeNumericPointRegister("3", BPollFrequency.slow));

    pointFolder.add("baudratePreset",     AsyncRegisterFactory.makeSingleIntegerPresets("2", false));
    pointFolder.add("Baudrate",           AsyncRegisterFactory.makeNumericPointRegister("22", BPollFrequency.slow));

    pointFolder.add("ModuleType",         AsyncRegisterFactory.makeNumericPointRegister("10", BPollFrequency.slow));
    pointFolder.add("FirmwareVersion",    AsyncRegisterFactory.makeNumericPointRegister("12", BPollFrequency.slow));

    pointFolder.add("timeoutSecPreset",   AsyncRegisterFactory.makeSingleIntegerPresets("4", false));
    pointFolder.add("TimeoutSecs",        AsyncRegisterFactory.makeNumericPointRegister("4", BPollFrequency.slow));


    pointFolder.add("Input01",          AsyncRegisterFactory.makeSignedNumericPointRegister("501", BPollFrequency.normal));
    pointFolder.add("Input02",          AsyncRegisterFactory.makeSignedNumericPointRegister("502", BPollFrequency.normal));
    pointFolder.add("Input03",          AsyncRegisterFactory.makeSignedNumericPointRegister("503", BPollFrequency.normal));
    pointFolder.add("Input04",          AsyncRegisterFactory.makeSignedNumericPointRegister("504", BPollFrequency.normal));
    pointFolder.add("Input05",          AsyncRegisterFactory.makeSignedNumericPointRegister("505", BPollFrequency.normal));
    pointFolder.add("Input06",          AsyncRegisterFactory.makeSignedNumericPointRegister("506", BPollFrequency.normal));
    pointFolder.add("Input07",          AsyncRegisterFactory.makeSignedNumericPointRegister("507", BPollFrequency.normal));
    pointFolder.add("Input08",          AsyncRegisterFactory.makeSignedNumericPointRegister("508", BPollFrequency.normal));


    pointFolder.add("Characteristic01Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1501", false));
    pointFolder.add("Characteristic01",   AsyncRegisterFactory.makeNumericPointRegister("1501", BPollFrequency.slow));

    pointFolder.add("Characteristic02Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1502", false));
    pointFolder.add("Characteristic02", AsyncRegisterFactory.makeNumericPointRegister("1502", BPollFrequency.slow));

    pointFolder.add("Characteristic03Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1503", false));
    pointFolder.add("Characteristic03", AsyncRegisterFactory.makeNumericPointRegister("1503", BPollFrequency.slow));

    pointFolder.add("Characteristic04Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1504", false));
    pointFolder.add("Characteristic04", AsyncRegisterFactory.makeNumericPointRegister("1504", BPollFrequency.slow));

    pointFolder.add("Characteristic05Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1505", false));
    pointFolder.add("Characteristic05", AsyncRegisterFactory.makeNumericPointRegister("1505", BPollFrequency.slow));

    pointFolder.add("Characteristic06Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1506", false));
    pointFolder.add("Characteristic06", AsyncRegisterFactory.makeNumericPointRegister("1506", BPollFrequency.slow));

    pointFolder.add("Characteristic07Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1507", false));
    pointFolder.add("Characteristic07", AsyncRegisterFactory.makeNumericPointRegister("1507", BPollFrequency.slow));

    pointFolder.add("Characteristic08Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1508", false));
    pointFolder.add("Characteristic08", AsyncRegisterFactory.makeNumericPointRegister("1508", BPollFrequency.slow));


    BNumericWritable statusLedInterface = AsyncRegisterFactory.makeNumericWritableInterface();
    pointFolder.add("statusLed", statusLedInterface);

    BModbusClientPresetRegisters statusLedPreset = AsyncRegisterFactory.makeSingleIntegerPresets("210", false);
    pointFolder.add("statusLedPreset", statusLedPreset );

    AsyncLinkUtils.linkToPresets(statusLedPreset, statusLedInterface, "out");

    BNumericWritable moduleCmdInterface = AsyncRegisterFactory.makeNumericWritableInterface();
    pointFolder.add("moduleCmd", moduleCmdInterface);

    BModbusClientPresetRegisters moduleCmdPreset =  AsyncRegisterFactory.makeSingleIntegerPresets("6", false);
    pointFolder.add("moduleCmdPreset", moduleCmdPreset );

    AsyncLinkUtils.linkToPresets(moduleCmdPreset, moduleCmdInterface, "out");
  }
}
