package com.oas.modbusutilities.devices;

import javax.baja.control.BNumericWritable;
import javax.baja.driver.util.BPollFrequency;
import javax.baja.nre.annotations.NiagaraProperty;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.status.BStatus;
import javax.baja.sys.BComponent;
import javax.baja.sys.BIcon;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.oas.modbusutilities.configuration.BAbstractConfiguration;
import com.oas.modbusutilities.configuration.BAsyncAo3Configuration;
import com.oas.modbusutilities.configuration.BConfigurationStatus;
import com.oas.modbusutilities.network.BOasModbusAsyncNetwork;
import com.oas.modbusutilities.register.AsyncRegisterFactory;
import com.oas.modbusutilities.utils.AsyncLinkUtils;
import com.tridium.modbusAsync.BModbusAsyncDevice;
import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegisters;
import com.tridium.modbusCore.client.point.BModbusClientPointDeviceExt;
import com.tridium.modbusCore.client.point.BModbusClientPointFolder;
import com.tridium.modbusCore.datatypes.BFlexAddress;
import com.tridium.modbusCore.enums.BAddressFormatEnum;


@NiagaraProperty(
  name="createFromTemplate",
  type="boolean",
  defaultValue="true",
  flags= Flags.READONLY | Flags.HIDDEN
)

@NiagaraProperty(
  name="pingAddress",
  type="BFlexAddress",
  defaultValue="new BFlexAddress(BAddressFormatEnum.decimal, \"10\")"
)


@NiagaraProperty(
  name="configuration",
  type="BAsyncAo3Configuration",
  defaultValue="new BAsyncAo3Configuration()"
)

@SuppressWarnings({
  "squid:S2629", // Invoke LOGs conditionally will bloat up the code
  "squid:S110" // Niagara Framework has more inherit lvl than 6
})
@NiagaraType
public class BAsyncAo3Module extends BModbusAsyncDevice {




/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.oas.modbusutilities.devices.BAsyncAo3Module(344560673)1.0$ @*/
/* Generated Fri Dec 04 09:54:09 CET 2020 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Property "createFromTemplate"
////////////////////////////////////////////////////////////////

  /**
   * Slot for the {@code createFromTemplate} property.
   * @see #getCreateFromTemplate
   * @see #setCreateFromTemplate
   */
  public static final Property createFromTemplate = newProperty(Flags.READONLY | Flags.HIDDEN, true, null);

  /**
   * Get the {@code createFromTemplate} property.
   * @see #createFromTemplate
   */
  public boolean getCreateFromTemplate() { return getBoolean(createFromTemplate); }

  /**
   * Set the {@code createFromTemplate} property.
   * @see #createFromTemplate
   */
  public void setCreateFromTemplate(boolean v) { setBoolean(createFromTemplate, v, null); }

////////////////////////////////////////////////////////////////
// Property "pingAddress"
////////////////////////////////////////////////////////////////

  /**
   * Slot for the {@code pingAddress} property.
   * @see #getPingAddress
   * @see #setPingAddress
   */
  public static final Property pingAddress = newProperty(0, new BFlexAddress(BAddressFormatEnum.decimal, "10"), null);

  /**
   * Get the {@code pingAddress} property.
   * @see #pingAddress
   */
  public BFlexAddress getPingAddress() { return (BFlexAddress)get(pingAddress); }

  /**
   * Set the {@code pingAddress} property.
   * @see #pingAddress
   */
  public void setPingAddress(BFlexAddress v) { set(pingAddress, v, null); }

////////////////////////////////////////////////////////////////
// Property "configuration"
////////////////////////////////////////////////////////////////

  /**
   * Slot for the {@code configuration} property.
   * @see #getConfiguration
   * @see #setConfiguration
   */
  public static final Property configuration = newProperty(0, new BAsyncAo3Configuration(), null);

  /**
   * Get the {@code configuration} property.
   * @see #configuration
   */
  public BAsyncAo3Configuration getConfiguration() { return (BAsyncAo3Configuration)get(configuration); }

  /**
   * Set the {@code configuration} property.
   * @see #configuration
   */
  public void setConfiguration(BAsyncAo3Configuration v) { set(configuration, v, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////

  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BAsyncAo3Module.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

    private BStatus lastStatus = BStatus.DEFAULT;
    private static final BIcon icon = BIcon.make("module://oasModbusUtilitiesCore/res/icon/module_round.png");

    @Override
    public BIcon getIcon() {
        return icon;
    }

    @Override
    public void started() throws Exception {
        super.started();

        if(Sys.isStationStarted() && getCreateFromTemplate()) {
            createRegister();
        }
    }

    @Override
    public void descendantsStarted() throws Exception {
        super.descendantsStarted();
        if(Sys.isStationStarted() && getCreateFromTemplate()) {
            setCreateFromTemplate(false);
        }

        BAbstractConfiguration config = getConfiguration();

        if (!config.isLinked()) {
            config.createLinksToRegister();
        }
    }


    @Override
    public void doPing() {
        super.doPing();

        BAbstractConfiguration config = getConfiguration();
        if (null != config) {
            if (config.isConfigurationValid() && config.isConfigurationModified()) {
                config.writeConfiguration();
            }
            else if (!config.isConfigurationValid()) {
                config.setConfigInvalidStatus();

            }
        }
    }


  @Override
  public void changed(Property property, Context context) {
    super.changed(property, context);


    if (property.equals(status)) {
      BAbstractConfiguration config = getConfiguration();

      if (null != config) {
        if (getStatus().isOk()) {
          if (lastStatus.isDown()){
            config.startCheckConfigTimer();
          }
          else if (lastStatus.isDisabled()) {
            config.startCheckConfigTimer();
            config.startStatusLedTimer();
          }
        }
        else if (getStatus().isDisabled()) {
          config.setStatus(BConfigurationStatus.offline);
        }
      }
      lastStatus = getStatus();
    }
    else if (property.equals(deviceAddress)) {
      BAbstractConfiguration config = getConfiguration();

      config.doCheck();
    }
    else if (property.equals(enabled)) {

      if (getEnabled()) {
        getConfiguration().restartMaxSendTicket();
      }
      else {
        getConfiguration().cancelMaxSendTicket();
      }

    }
  }

  @Override
  public boolean isParentLegal(BComponent parent) {
    return (parent instanceof BOasModbusAsyncNetwork);
  }


  private final void createRegister() {
    BModbusClientPointDeviceExt pointExt = getPoints();

    //move all Register and Presets into it's own PointFolder
    BModbusClientPointFolder pointFolder = new BModbusClientPointFolder();
    pointExt.add("Points", pointFolder);

    pointFolder.add("autobaudingPreset", AsyncRegisterFactory.makeSingleIntegerPresets("3", false));
    pointFolder.add("Autobauding",       AsyncRegisterFactory.makeNumericPointRegister("3", BPollFrequency.slow));

    pointFolder.add("baudratePreset", AsyncRegisterFactory.makeSingleIntegerPresets("2", false));
    pointFolder.add("Baudrate",       AsyncRegisterFactory.makeNumericPointRegister("22", BPollFrequency.slow));

    pointFolder.add("ModuleType",      AsyncRegisterFactory.makeNumericPointRegister("10", BPollFrequency.slow));

    pointFolder.add("FirmwareVersion", AsyncRegisterFactory.makeNumericPointRegister("12", BPollFrequency.slow));

    pointFolder.add("timeoutSecPreset", AsyncRegisterFactory.makeSingleIntegerPresets("4", false));
    pointFolder.add("TimeoutSecs",      AsyncRegisterFactory.makeNumericPointRegister("4", BPollFrequency.slow));

    pointFolder.add("ledRedValuePreset", AsyncRegisterFactory.makeSingleIntegerPresets("1341", false));
    pointFolder.add("ledRedValue",       AsyncRegisterFactory.makeNumericPointRegister("1341", BPollFrequency.slow));

    pointFolder.add("ledGreenValuePreset", AsyncRegisterFactory.makeSingleIntegerPresets("1342", false));
    pointFolder.add("ledGreenValue",       AsyncRegisterFactory.makeNumericPointRegister("1342", BPollFrequency.slow));

    pointFolder.add("Inputs", AsyncRegisterFactory.makeNumericPointRegister("101", BPollFrequency.fast));

    pointFolder.add("switchLogicValuePreset", AsyncRegisterFactory.makeSingleIntegerPresets("1100", false));
    pointFolder.add("switchLogicValue",       AsyncRegisterFactory.makeNumericPointRegister("1100", BPollFrequency.slow));

    pointFolder.add("bounceProtectionValuePreset",  AsyncRegisterFactory.makeSingleIntegerPresets("1101", false));
    pointFolder.add("bounceProtectionValue",        AsyncRegisterFactory.makeNumericPointRegister("1101", BPollFrequency.slow));

    pointFolder.add("bounceProtectionTimeValuePreset", AsyncRegisterFactory.makeSingleIntegerPresets("1111", false));
    pointFolder.add("bounceProtectionTimeValue",       AsyncRegisterFactory.makeNumericPointRegister("1111", BPollFrequency.slow));

    pointFolder.add("feedback01", AsyncRegisterFactory.makeNumericPointRegister("1840", BPollFrequency.normal));
    pointFolder.add("feedback02", AsyncRegisterFactory.makeNumericPointRegister("1841", BPollFrequency.normal));

    pointFolder.add("automaticCmd01Preset", AsyncRegisterFactory.makeMultiIntegerPresets("1800", true, "autoCmd"));
    pointFolder.add("automaticCmd02Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1801", true, "autoCmd"));

    pointFolder.add("onSideSwitchValue", AsyncRegisterFactory.makeNumericPointRegister("153", BPollFrequency.normal));

    pointFolder.add("onSideSwitchStatusPreset", AsyncRegisterFactory.makeSingleIntegerPresets("135", false, 255.0f));
    pointFolder.add("onSideSwitchStatus",       AsyncRegisterFactory.makeNumericPointRegister("135", BPollFrequency.slow));

    pointFolder.add("upTime01Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1920", false));
    pointFolder.add("upTime01",       AsyncRegisterFactory.makeNumericPointRegister("1920", BPollFrequency.normal));

    pointFolder.add("upTime02Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1921", false));
    pointFolder.add("upTime02",       AsyncRegisterFactory.makeNumericPointRegister("1921", BPollFrequency.normal));

    pointFolder.add("downTime01Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1930", false));
    pointFolder.add("downTime01",       AsyncRegisterFactory.makeNumericPointRegister("1930", BPollFrequency.normal));

    pointFolder.add("downTime02Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1931", false));
    pointFolder.add("downTime02",       AsyncRegisterFactory.makeNumericPointRegister("1931", BPollFrequency.normal));

    pointFolder.add("onOffDelay01Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1970", false));
    pointFolder.add("onOffDelay01",       AsyncRegisterFactory.makeNumericPointRegister("1970", BPollFrequency.normal));

    pointFolder.add("onOffDelay02Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1971", false));
    pointFolder.add("onOffDelay02",       AsyncRegisterFactory.makeNumericPointRegister("1971", BPollFrequency.normal));

    pointFolder.add("changeOverDelay01Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1980", false));
    pointFolder.add("changeOverDelay01",       AsyncRegisterFactory.makeNumericPointRegister("1980", BPollFrequency.normal));

    pointFolder.add("changeOverDelay02Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1981", false));
    pointFolder.add("changeOverDelay02",       AsyncRegisterFactory.makeNumericPointRegister("1981", BPollFrequency.normal));

    pointFolder.add("enableChannelsPreset", AsyncRegisterFactory.makeSingleIntegerPresets("1900", false));
    pointFolder.add("enableChannels",       AsyncRegisterFactory.makeNumericWritableRegister("1900", BPollFrequency.normal, 3));

    pointFolder.add("overrunUpTime01Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1940", false));
    pointFolder.add("overrunUpTime01",       AsyncRegisterFactory.makeNumericPointRegister("1940", BPollFrequency.normal));

    pointFolder.add("overrunUpTime02Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1941", false));
    pointFolder.add("overrunUpTime02",       AsyncRegisterFactory.makeNumericPointRegister("1941", BPollFrequency.normal));

    pointFolder.add("overrunDownTime01Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1950", false));
    pointFolder.add("overrunDownTime01",       AsyncRegisterFactory.makeNumericPointRegister("1950", BPollFrequency.normal));

    pointFolder.add("overrunDownTime02Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1951", false));
    pointFolder.add("overrunDownTime02",       AsyncRegisterFactory.makeNumericPointRegister("1951", BPollFrequency.normal));

    pointFolder.add("outputStatus01", AsyncRegisterFactory.makeNumericPointRegister("1820", BPollFrequency.normal));
    pointFolder.add("outputStatus01Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1820", false));

    pointFolder.add("outputStatus02", AsyncRegisterFactory.makeNumericPointRegister("1821", BPollFrequency.normal));
    pointFolder.add("outputStatus02Preset", AsyncRegisterFactory.makeSingleIntegerPresets("1821", false));


    BModbusClientPresetRegisters statusLedPreset = AsyncRegisterFactory.makeSingleIntegerPresets("210", false);
    pointFolder.add("statusLedPreset",       statusLedPreset );

    BNumericWritable statusLed = AsyncRegisterFactory.makeNumericWritableInterface();
    pointFolder.add("statusLed", statusLed);
    AsyncLinkUtils.linkToPresets(statusLedPreset, statusLed, "out");

    BNumericWritable moduleCmdInterface = AsyncRegisterFactory.makeNumericWritableInterface();
    pointFolder.add("moduleCmd", moduleCmdInterface);

    BModbusClientPresetRegisters moduleCmdPreset =  AsyncRegisterFactory.makeSingleIntegerPresets("6", false);
    pointFolder.add("moduleCmdPreset", moduleCmdPreset );
    AsyncLinkUtils.linkToPresets(moduleCmdPreset, moduleCmdInterface, "out");
  }
}
