package com.oas.modbusutilities.devices;

import javax.baja.control.BNumericWritable;
import javax.baja.driver.util.BPollFrequency;
import javax.baja.nre.annotations.NiagaraProperty;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.status.BStatus;
import javax.baja.sys.BComponent;
import javax.baja.sys.BIcon;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.oas.modbusutilities.configuration.BAbstractConfiguration;
import com.oas.modbusutilities.configuration.BAsyncAoConfiguration;
import com.oas.modbusutilities.configuration.BConfigurationStatus;
import com.oas.modbusutilities.network.BOasModbusAsyncNetwork;
import com.oas.modbusutilities.register.AsyncRegisterFactory;
import com.oas.modbusutilities.utils.AsyncLinkUtils;
import com.tridium.modbusAsync.BModbusAsyncDevice;
import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegisters;
import com.tridium.modbusCore.client.point.BModbusClientPointDeviceExt;
import com.tridium.modbusCore.client.point.BModbusClientPointFolder;
import com.tridium.modbusCore.datatypes.BFlexAddress;
import com.tridium.modbusCore.enums.BAddressFormatEnum;


@NiagaraProperty(
  name="createFromTemplate",
  type="boolean",
  defaultValue="true",
  flags=Flags.READONLY | Flags.HIDDEN
)

@NiagaraProperty(
  name="pingAddress",
  type="BFlexAddress",
  defaultValue="new BFlexAddress(BAddressFormatEnum.decimal, \"10\")"
)

@NiagaraProperty(
  name="configuration",
  type="BAsyncAoConfiguration",
  defaultValue="new BAsyncAoConfiguration()"
)

@SuppressWarnings({
  "squid:S110" // Niagara Framework has more inherit lvl than 6
})
@NiagaraType
public class BAsyncAoModule extends BModbusAsyncDevice {
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.oas.modbusutilities.devices.BAsyncAoModule(1641420996)1.0$ @*/
/* Generated Fri Dec 04 09:54:09 CET 2020 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Property "createFromTemplate"
////////////////////////////////////////////////////////////////

  /**
   * Slot for the {@code createFromTemplate} property.
   * @see #getCreateFromTemplate
   * @see #setCreateFromTemplate
   */
  public static final Property createFromTemplate = newProperty(Flags.READONLY | Flags.HIDDEN, true, null);

  /**
   * Get the {@code createFromTemplate} property.
   * @see #createFromTemplate
   */
  public boolean getCreateFromTemplate() { return getBoolean(createFromTemplate); }

  /**
   * Set the {@code createFromTemplate} property.
   * @see #createFromTemplate
   */
  public void setCreateFromTemplate(boolean v) { setBoolean(createFromTemplate, v, null); }

////////////////////////////////////////////////////////////////
// Property "pingAddress"
////////////////////////////////////////////////////////////////

  /**
   * Slot for the {@code pingAddress} property.
   * @see #getPingAddress
   * @see #setPingAddress
   */
  public static final Property pingAddress = newProperty(0, new BFlexAddress(BAddressFormatEnum.decimal, "10"), null);

  /**
   * Get the {@code pingAddress} property.
   * @see #pingAddress
   */
  public BFlexAddress getPingAddress() { return (BFlexAddress)get(pingAddress); }

  /**
   * Set the {@code pingAddress} property.
   * @see #pingAddress
   */
  public void setPingAddress(BFlexAddress v) { set(pingAddress, v, null); }

////////////////////////////////////////////////////////////////
// Property "configuration"
////////////////////////////////////////////////////////////////

  /**
   * Slot for the {@code configuration} property.
   * @see #getConfiguration
   * @see #setConfiguration
   */
  public static final Property configuration = newProperty(0, new BAsyncAoConfiguration(), null);

  /**
   * Get the {@code configuration} property.
   * @see #configuration
   */
  public BAsyncAoConfiguration getConfiguration() { return (BAsyncAoConfiguration)get(configuration); }

  /**
   * Set the {@code configuration} property.
   * @see #configuration
   */
  public void setConfiguration(BAsyncAoConfiguration v) { set(configuration, v, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////

  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BAsyncAoModule.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/


  private BStatus lastStatus = BStatus.DEFAULT;

  private static final BIcon icon = BIcon.make("module://oasModbusUtilitiesCore/res/icon/module_round.png");

  @Override
  public BIcon getIcon() {
    return icon;
  }

  @Override
  public void started() throws Exception {
    super.started();



    if(Sys.isStationStarted() && getCreateFromTemplate()) {
      createRegister();
    }
  }

  @Override
  public void descendantsStarted() throws Exception {
    super.descendantsStarted();
    if(Sys.isStationStarted() && getCreateFromTemplate()) {
      setCreateFromTemplate(false);
    }

    BAbstractConfiguration config = getConfiguration();

    if (!config.isLinked()) {
      config.createLinksToRegister();
    }
  }

  @Override
  public void doPing() {
    super.doPing();

    BAbstractConfiguration config = getConfiguration();
    if (null != config) {
      if (config.isConfigurationValid() && config.isConfigurationModified()) {
        config.writeConfiguration();
      }
      else if (!config.isConfigurationValid()) {
        config.setConfigInvalidStatus();
      }
    }
  }

  @Override
  public void changed(Property property, Context context) {
    super.changed(property, context);


    if (property.equals(status)) {

      if (getStatus().isOk()) {
        if (lastStatus.isDown()){
          BAbstractConfiguration config = getConfiguration();
          if (null != config) {
            config.startCheckConfigTimer();
          }
        }
        else if (lastStatus.isDisabled()) {
          BAbstractConfiguration config = getConfiguration();
          if (null != config) {
            config.startCheckConfigTimer();
            config.startStatusLedTimer();
          }
        }
      }
      else if (getStatus().isDisabled()) {
        getConfiguration().setStatus(BConfigurationStatus.offline);
      }
      lastStatus = getStatus();
    }
    else if (property.equals(deviceAddress)) {
      BAbstractConfiguration config = getConfiguration();

      config.doCheck();
    }
    else if (property.equals(enabled)) {

      if (getEnabled()) {
        getConfiguration().restartMaxSendTicket();
      }
      else {
        getConfiguration().cancelMaxSendTicket();
      }

    }
  }

  @Override
  public boolean isParentLegal(BComponent parent) {
    return (parent instanceof BOasModbusAsyncNetwork);
  }


  private final void createRegister() {
    BModbusClientPointDeviceExt pointExt = getPoints();

    //move all Register and Presets into it's own PointFolder
    BModbusClientPointFolder pointFolder = new BModbusClientPointFolder();
    pointExt.add("Points", pointFolder);

    pointFolder.add("autobaudingPreset",  AsyncRegisterFactory.makeSingleIntegerPresets("3", false));
    pointFolder.add("Autobauding",        AsyncRegisterFactory.makeNumericPointRegister("3", BPollFrequency.slow));

    pointFolder.add("baudratePreset",     AsyncRegisterFactory.makeSingleIntegerPresets("2", false));
    pointFolder.add("Baudrate",           AsyncRegisterFactory.makeNumericPointRegister("22", BPollFrequency.slow));

    pointFolder.add("ModuleType",         AsyncRegisterFactory.makeNumericPointRegister("10", BPollFrequency.slow));
    pointFolder.add("FirmwareVersion",    AsyncRegisterFactory.makeNumericPointRegister("12", BPollFrequency.slow));

    pointFolder.add("timeoutSecPreset",   AsyncRegisterFactory.makeSingleIntegerPresets("4", false));
    pointFolder.add("TimeoutSecs",        AsyncRegisterFactory.makeNumericPointRegister("4", BPollFrequency.slow));


    pointFolder.add("onSideSwitchValue",  AsyncRegisterFactory.makeNumericPointRegister("152", BPollFrequency.normal));

    pointFolder.add("onSideSwitchStatusPreset", AsyncRegisterFactory.makeSingleIntegerPresets("535", false, 255.0f));
    pointFolder.add("onSideSwitchStatus", AsyncRegisterFactory.makeNumericPointRegister("535", BPollFrequency.slow));

    pointFolder.add("savePositionStatusPreset", AsyncRegisterFactory.makeSingleIntegerPresets("2535", false));
    pointFolder.add("savePositionStatus", AsyncRegisterFactory.makeNumericPointRegister("2535", BPollFrequency.slow));

    pointFolder.add("Feedback01",        AsyncRegisterFactory.makeNumericPointRegister("511", BPollFrequency.normal));
    pointFolder.add("Feedback02",        AsyncRegisterFactory.makeNumericPointRegister("512", BPollFrequency.normal));
    pointFolder.add("Feedback03",        AsyncRegisterFactory.makeNumericPointRegister("513", BPollFrequency.normal));
    pointFolder.add("Feedback04",        AsyncRegisterFactory.makeNumericPointRegister("514", BPollFrequency.normal));
    pointFolder.add("Feedback05",        AsyncRegisterFactory.makeNumericPointRegister("515", BPollFrequency.normal));
    pointFolder.add("Feedback06",        AsyncRegisterFactory.makeNumericPointRegister("516", BPollFrequency.normal));
    pointFolder.add("Feedback07",        AsyncRegisterFactory.makeNumericPointRegister("517", BPollFrequency.normal));
    pointFolder.add("Feedback08",        AsyncRegisterFactory.makeNumericPointRegister("518", BPollFrequency.normal));

    pointFolder.add("savePositionValue01Preset", AsyncRegisterFactory.makeSingleIntegerPresets("2521", false));
    pointFolder.add("savePositionValue01", AsyncRegisterFactory.makeNumericPointRegister("2521", BPollFrequency.slow));

    pointFolder.add("savePositionValue02Preset", AsyncRegisterFactory.makeSingleIntegerPresets("2522", false));
    pointFolder.add("savePositionValue02", AsyncRegisterFactory.makeNumericPointRegister("2522", BPollFrequency.slow));

    pointFolder.add("savePositionValue03Preset", AsyncRegisterFactory.makeSingleIntegerPresets("2523", false));
    pointFolder.add("savePositionValue03", AsyncRegisterFactory.makeNumericPointRegister("2523", BPollFrequency.slow));

    pointFolder.add("savePositionValue04Preset", AsyncRegisterFactory.makeSingleIntegerPresets("2524", false));
    pointFolder.add("savePositionValue04", AsyncRegisterFactory.makeNumericPointRegister("2524", BPollFrequency.slow));

    pointFolder.add("savePositionValue05Preset", AsyncRegisterFactory.makeSingleIntegerPresets("2525", false));
    pointFolder.add("savePositionValue05", AsyncRegisterFactory.makeNumericPointRegister("2525", BPollFrequency.slow));

    pointFolder.add("savePositionValue06Preset", AsyncRegisterFactory.makeSingleIntegerPresets("2526", false));
    pointFolder.add("savePositionValue06", AsyncRegisterFactory.makeNumericPointRegister("2526", BPollFrequency.slow));

    pointFolder.add("savePositionValue07Preset", AsyncRegisterFactory.makeSingleIntegerPresets("2527", false));
    pointFolder.add("savePositionValue07", AsyncRegisterFactory.makeNumericPointRegister("2527", BPollFrequency.slow));

    pointFolder.add("savePositionValue08Preset", AsyncRegisterFactory.makeSingleIntegerPresets("2528", false));
    pointFolder.add("savePositionValue08", AsyncRegisterFactory.makeNumericPointRegister("2528", BPollFrequency.slow));

    BModbusClientPresetRegisters autoCmdPrest = AsyncRegisterFactory.makeMultiIntegerPresets("521", false, "autoCmd01",
                                                                                                           "autoCmd02",
                                                                                                           "autoCmd03",
                                                                                                           "autoCmd04",
                                                                                                           "autoCmd05",
                                                                                                           "autoCmd06",
                                                                                                           "autoCmd07",
                                                                                                           "autoCmd08");

    // autoCmdContainer.add( AsyncRegisterFactory.makePresetRegister());
    // autoCmdContainer.add( AsyncRegisterFactory.makePresetRegister());
    // autoCmdContainer.add( AsyncRegisterFactory.makePresetRegister());
    // autoCmdContainer.add( AsyncRegisterFactory.makePresetRegister());
    // autoCmdContainer.add( AsyncRegisterFactory.makePresetRegister());
    // autoCmdContainer.add( AsyncRegisterFactory.makePresetRegister());
    // autoCmdContainer.add( AsyncRegisterFactory.makePresetRegister());
    // autoCmdContainer.add( AsyncRegisterFactory.makePresetRegister());

    pointFolder.add("automaticCmdPreset", autoCmdPrest);

    BNumericWritable statusLedInterface = AsyncRegisterFactory.makeNumericWritableInterface();
    pointFolder.add("statusLed", statusLedInterface);

    BModbusClientPresetRegisters statusLedPreset = AsyncRegisterFactory.makeSingleIntegerPresets("210", false);
    pointFolder.add("statusLedPreset", statusLedPreset );

    AsyncLinkUtils.linkToPresets(statusLedPreset, statusLedInterface, "out");

    BNumericWritable moduleCmdInterface = AsyncRegisterFactory.makeNumericWritableInterface();
    pointFolder.add("moduleCmd", moduleCmdInterface);

    BModbusClientPresetRegisters moduleCmdPreset =  AsyncRegisterFactory.makeSingleIntegerPresets("6", false);
    pointFolder.add("moduleCmdPreset", moduleCmdPreset );

    AsyncLinkUtils.linkToPresets(moduleCmdPreset, moduleCmdInterface, "out");

  }
}
