package com.oas.modbusutilities.devices;

import javax.baja.control.BNumericWritable;
import javax.baja.driver.util.BPollFrequency;
import javax.baja.nre.annotations.NiagaraProperty;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.status.BStatus;
import javax.baja.sys.BComponent;
import javax.baja.sys.BIcon;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.oas.modbusutilities.configuration.BAbstractConfiguration;
import com.oas.modbusutilities.configuration.BAsyncDoConfiguration;
import com.oas.modbusutilities.configuration.BConfigurationStatus;
import com.oas.modbusutilities.network.BOasModbusAsyncNetwork;
import com.oas.modbusutilities.register.AsyncRegisterFactory;
import com.oas.modbusutilities.utils.AsyncLinkUtils;
import com.tridium.modbusAsync.BModbusAsyncDevice;
import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegisters;
import com.tridium.modbusCore.client.point.BModbusClientPointDeviceExt;
import com.tridium.modbusCore.client.point.BModbusClientPointFolder;
import com.tridium.modbusCore.datatypes.BFlexAddress;
import com.tridium.modbusCore.enums.BAddressFormatEnum;



@NiagaraProperty(
  name="createFromTemplate",
  type="boolean",
  defaultValue="true",
  flags=Flags.READONLY | Flags.HIDDEN
)

@NiagaraProperty(
  name="pingAddress",
  type="BFlexAddress",
  defaultValue="new BFlexAddress(BAddressFormatEnum.decimal, \"10\")"
)

@NiagaraProperty(
  name="configuration",
  type="BAsyncDoConfiguration",
  defaultValue="new BAsyncDoConfiguration()"
)

@SuppressWarnings({
  "squid:S110" // Niagara Framework has more inherit lvl than 6
})
@NiagaraType
public class BAsyncDoModule extends BModbusAsyncDevice {
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.oas.modbusutilities.devices.BAsyncDoModule(790316118)1.0$ @*/
/* Generated Fri Dec 04 09:54:09 CET 2020 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Property "createFromTemplate"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code createFromTemplate} property.
   * @see #getCreateFromTemplate
   * @see #setCreateFromTemplate
   */
  public static final Property createFromTemplate = newProperty(Flags.READONLY | Flags.HIDDEN, true, null);
  
  /**
   * Get the {@code createFromTemplate} property.
   * @see #createFromTemplate
   */
  public boolean getCreateFromTemplate() { return getBoolean(createFromTemplate); }
  
  /**
   * Set the {@code createFromTemplate} property.
   * @see #createFromTemplate
   */
  public void setCreateFromTemplate(boolean v) { setBoolean(createFromTemplate, v, null); }

////////////////////////////////////////////////////////////////
// Property "pingAddress"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code pingAddress} property.
   * @see #getPingAddress
   * @see #setPingAddress
   */
  public static final Property pingAddress = newProperty(0, new BFlexAddress(BAddressFormatEnum.decimal, "10"), null);
  
  /**
   * Get the {@code pingAddress} property.
   * @see #pingAddress
   */
  public BFlexAddress getPingAddress() { return (BFlexAddress)get(pingAddress); }
  
  /**
   * Set the {@code pingAddress} property.
   * @see #pingAddress
   */
  public void setPingAddress(BFlexAddress v) { set(pingAddress, v, null); }

////////////////////////////////////////////////////////////////
// Property "configuration"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code configuration} property.
   * @see #getConfiguration
   * @see #setConfiguration
   */
  public static final Property configuration = newProperty(0, new BAsyncDoConfiguration(), null);
  
  /**
   * Get the {@code configuration} property.
   * @see #configuration
   */
  public BAsyncDoConfiguration getConfiguration() { return (BAsyncDoConfiguration)get(configuration); }
  
  /**
   * Set the {@code configuration} property.
   * @see #configuration
   */
  public void setConfiguration(BAsyncDoConfiguration v) { set(configuration, v, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BAsyncDoModule.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/


  private BStatus lastStatus = BStatus.DEFAULT;
  private static final BIcon icon = BIcon.make("module://oasModbusUtilitiesCore/res/icon/module_round.png");

  @Override
  public BIcon getIcon() {
    return icon;
  }

  @Override
  public void started() throws Exception {
    super.started();



    if(Sys.isStationStarted() && getCreateFromTemplate()) {
      createRegister();
    }
  }

  @Override
  public void descendantsStarted() throws Exception {
    super.descendantsStarted();
    if(Sys.isStationStarted() && getCreateFromTemplate()) {
      setCreateFromTemplate(false);
    }

    BAbstractConfiguration config = getConfiguration();

    if (!config.isLinked()) {
      config.createLinksToRegister();
    }
  }

  @Override
  public void doPing() {
    super.doPing();

    BAbstractConfiguration config = getConfiguration();
    if (null != config) {
      if (config.isConfigurationValid() && config.isConfigurationModified()) {
        config.writeConfiguration();
      }
      else if (!config.isConfigurationValid()) {
        config.setConfigInvalidStatus();
      }
    }
  }

  @Override
  public void changed(Property property, Context context) {
    super.changed(property, context);


    if (property.equals(status)) {

      if (getStatus().isOk()) {
        if (lastStatus.isDown()){
          BAbstractConfiguration config = getConfiguration();
          if (null != config) {
            config.startCheckConfigTimer();
          }
        }
        else if (lastStatus.isDisabled()) {
          BAbstractConfiguration config = getConfiguration();
          if (null != config) {
            config.startCheckConfigTimer();
            config.startStatusLedTimer();
          }
        }
      }
      else if (getStatus().isDisabled()) {
        getConfiguration().setStatus(BConfigurationStatus.offline);
      }
      lastStatus = getStatus();
    }
    else if (property.equals(deviceAddress)) {
      BAbstractConfiguration config = getConfiguration();

      config.doCheck();
    }
    else if (property.equals(enabled)) {

      if (getEnabled()) {
        getConfiguration().restartMaxSendTicket();
      }
      else {
        getConfiguration().cancelMaxSendTicket();
      }

    }
  }


  @Override
  public boolean isParentLegal(BComponent parent) {
    return (parent instanceof BOasModbusAsyncNetwork);
  }



  private final void createRegister() {

    BModbusClientPointDeviceExt pointExt = getPoints();

    BModbusClientPointFolder pointFolder = new BModbusClientPointFolder();
    pointExt.add("Points", pointFolder);

    pointFolder.add("autobaudingPreset",  AsyncRegisterFactory.makeSingleIntegerPresets("3", false));
    pointFolder.add("Autobauding",        AsyncRegisterFactory.makeNumericPointRegister("3", BPollFrequency.slow));

    pointFolder.add("baudratePreset",     AsyncRegisterFactory.makeSingleIntegerPresets("2", false));
    pointFolder.add("Baudrate",           AsyncRegisterFactory.makeNumericPointRegister("22", BPollFrequency.slow));

    pointFolder.add("ModuleType",         AsyncRegisterFactory.makeNumericPointRegister("10", BPollFrequency.slow));
    pointFolder.add("FirmwareVersion",    AsyncRegisterFactory.makeNumericPointRegister("12", BPollFrequency.slow));

    pointFolder.add("timeoutSecPreset",   AsyncRegisterFactory.makeSingleIntegerPresets("4", false));
    pointFolder.add("TimeoutSecs",        AsyncRegisterFactory.makeNumericPointRegister("4", BPollFrequency.slow));


    pointFolder.add("onSideSwitchValue", AsyncRegisterFactory.makeNumericWritableRegister("153", BPollFrequency.normal));

    pointFolder.add("onSideSwitchStatus", AsyncRegisterFactory.makeNumericPointRegister("135", BPollFrequency.slow));
    pointFolder.add("onSideSwitchStatusPreset", AsyncRegisterFactory.makeSingleIntegerPresets("135", false, 255.0f));

    pointFolder.add("savePositionStatus", AsyncRegisterFactory.makeNumericWritableRegister("2135", BPollFrequency.slow));
    pointFolder.add("savePositionStatusPreset", AsyncRegisterFactory.makeSingleIntegerPresets("2135", false));

    pointFolder.add("savePositionValue",  AsyncRegisterFactory.makeNumericWritableRegister("2121", BPollFrequency.slow));
    pointFolder.add("savePositionValuePreset", AsyncRegisterFactory.makeSingleIntegerPresets("2121", false));

    pointFolder.add("ledRedValue", AsyncRegisterFactory.makeNumericWritableRegister("1341", BPollFrequency.slow));
    pointFolder.add("ledRedValuePreset", AsyncRegisterFactory.makeSingleIntegerPresets("1341", false));

    pointFolder.add("ledGreenValue", AsyncRegisterFactory.makeNumericWritableRegister("1342", BPollFrequency.slow));
    pointFolder.add("ledGreenValuePreset", AsyncRegisterFactory.makeSingleIntegerPresets("1342", false));

    pointFolder.add("feedback", AsyncRegisterFactory.makeNumericWritableRegister("111", BPollFrequency.normal));

    pointFolder.add("automaticCmdPreset", AsyncRegisterFactory.makeSingleIntegerPresets("121", false));

    BModbusClientPresetRegisters statusLedPreset = AsyncRegisterFactory.makeSingleIntegerPresets("210", false);
    pointFolder.add("statusLedPreset", statusLedPreset );

    BNumericWritable statusLed = AsyncRegisterFactory.makeNumericWritableInterface();
    pointFolder.add("statusLed", statusLed);

    AsyncLinkUtils.linkToPresets(statusLedPreset, statusLed, "out");

    BNumericWritable moduleCmd = AsyncRegisterFactory.makeNumericWritableInterface();
    pointFolder.add("moduleCmd", moduleCmd);

    BModbusClientPresetRegisters moduleCmdPreset =  AsyncRegisterFactory.makeSingleIntegerPresets("6", false);
    pointFolder.add("moduleCmdPreset", moduleCmdPreset );

    AsyncLinkUtils.linkToPresets(moduleCmdPreset, moduleCmd, "out");
  }
}
