package com.oas.modbusutilities.network;
import java.text.MessageFormat;
import java.util.logging.Logger;

import javax.baja.driver.BDevice;
import javax.baja.license.Feature;
import javax.baja.license.LicenseException;
import javax.baja.naming.BOrd;
import javax.baja.nre.annotations.NiagaraAction;
import javax.baja.nre.annotations.NiagaraProperty;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.status.BStatus;
import javax.baja.sys.Action;
import javax.baja.sys.BFacets;
import javax.baja.sys.BRelTime;
import javax.baja.sys.BString;
import javax.baja.sys.BValue;
import javax.baja.sys.Clock;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.LocalizableRuntimeException;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;
import javax.baja.util.BNotification;
import javax.baja.util.Lexicon;

import com.tridium.modbusAsync.BModbusAsyncDevice;
import com.tridium.modbusAsync.BModbusAsyncNetwork;

@NiagaraProperty(
  name="homeOrd",
  type="baja:Ord",
  defaultValue="BOrd.DEFAULT"
)

@NiagaraProperty(
  name="logoOrd",
  type="baja:Ord",
  defaultValue="BOrd.make(\"local:|module://oasModbusUtilitiesCore/rc/img/logo/your_icon.svg\")"
)

@NiagaraAction(
  name = "demoTimeExpired",
  flags = Flags.HIDDEN | Flags.ASYNC
)

@SuppressWarnings({
  "squid:S2629", // Invoke LOGs conditionally will bloat up the code
  "squid:S110" // Niagara Framework has more inherit lvl than 6
})

@NiagaraType
public class BOasModbusAsyncNetwork extends BModbusAsyncNetwork implements OasModbusNetwork {
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.oas.modbusutilities.network.BOasModbusAsyncNetwork(1272391725)1.0$ @*/
/* Generated Thu Dec 17 17:14:52 CET 2020 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Property "homeOrd"
////////////////////////////////////////////////////////////////

  /**
   * Slot for the {@code homeOrd} property.
   * @see #getHomeOrd
   * @see #setHomeOrd
   */
  public static final Property homeOrd = newProperty(0, BOrd.DEFAULT, null);

  /**
   * Get the {@code homeOrd} property.
   * @see #homeOrd
   */
  public BOrd getHomeOrd() { return (BOrd)get(homeOrd); }

  /**
   * Set the {@code homeOrd} property.
   * @see #homeOrd
   */
  public void setHomeOrd(BOrd v) { set(homeOrd, v, null); }

////////////////////////////////////////////////////////////////
// Property "logoOrd"
////////////////////////////////////////////////////////////////

  /**
   * Slot for the {@code logoOrd} property.
   * @see #getLogoOrd
   * @see #setLogoOrd
   */
  public static final Property logoOrd = newProperty(0, BOrd.make("local:|module://oasModbusUtilitiesCore/rc/img/logo/your_icon.svg"), null);

  /**
   * Get the {@code logoOrd} property.
   * @see #logoOrd
   */
  public BOrd getLogoOrd() { return (BOrd)get(logoOrd); }

  /**
   * Set the {@code logoOrd} property.
   * @see #logoOrd
   */
  public void setLogoOrd(BOrd v) { set(logoOrd, v, null); }

////////////////////////////////////////////////////////////////
// Action "demoTimeExpired"
////////////////////////////////////////////////////////////////

  /**
   * Slot for the {@code demoTimeExpired} action.
   * @see #demoTimeExpired()
   */
  public static final Action demoTimeExpired = newAction(Flags.HIDDEN | Flags.ASYNC, null);

  /**
   * Invoke the {@code demoTimeExpired} action.
   * @see #demoTimeExpired
   */
  public void demoTimeExpired() { invoke(demoTimeExpired, null, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////

  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BOasModbusAsyncNetwork.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  private boolean deviceLimitReached = false;

  private static final String LEXICON_NAME = "oasModbusUtilitiesCore";
  private static final Lexicon LEX = Lexicon.make(LEXICON_NAME);

  private static final Logger LOG = Logger.getLogger("oasModbusUtilities");
  private static final String LOG_PREFIX = BOasModbusAsyncNetwork.class.getSimpleName();

  private static final Object mutexDemoTimer = new Object();
  private static final int DEMO_RUNNING_TIME_SECONDS = 60 * 60; // one hour running time when demo mode
  private Clock.Ticket demoTimerTicket = null;

  @Override
  public void started() throws Exception {
    // necessary to start serial com
    super.started();

    if (Sys.isStationStarted()) {

        checkLicense();
        checkDeviceLimit(false);
    }
  }

  @Override
  public void atSteadyState() throws Exception {
    super.atSteadyState();

    checkLicense();
    checkDeviceLimit(false);

  }


  @Override
  public void checkAdd(String name, BValue value, int flags, BFacets facets, Context context) {
    super.checkAdd(name, value, flags, facets, context);

    if (Sys.isStation() && (value instanceof BModbusAsyncDevice)) {
      checkDeviceLimit(true);
    }
  }

  @Override
  public void added(Property property, Context context) {

    if (!isRunning()) {
      return;
    }

    BDevice[] devices = getDevices();
    if (devices.length >= SUPPORTED_DEVICE_LIMIT && !deviceLimitReached) {

      BNotification warning = new BNotification();
      warning.add("title", BString.make(LEX.get("DeviceLimitReached.title", "Supported Device Limit reached")));
      warning.add("message", BString.make(
        MessageFormat.format(
          LEX.get("DeviceLimitReached.message",
                  "You reached the supported Device limit of {0} Device for the Network {1}!"),
          SUPPORTED_DEVICE_LIMIT, getName()
      )));
      warning.raise(true);
      deviceLimitReached = true;
    }
  }


  @Override
  public void changed(Property prop, Context ctx) {
    super.changed(prop, ctx);

    if (!Sys.isStationStarted()) {
      return;
    }

    if (prop.equals(enabled)) {
      checkLicense();
    }
  }

  public void doDemoTimeExpired() {
    checkLicense();
  }


  private void checkLicense()
  {
    synchronized (mutexDemoTimer)
    {
      try
      {
        Feature feature = Sys.getLicenseManager().getFeature(VENDOR, FEATURE);
        feature.check();
      }
      catch (LicenseException exc)
      {
        final String MSG_PREFIX = "OAS Modbus Utilities";
        if (demoTimerTicket == null)
        {
          demoTimerTicket = Clock.schedule(this, BRelTime.makeSeconds(DEMO_RUNNING_TIME_SECONDS), demoTimeExpired,
                                           null);
        }
        if (!demoTimerTicket.isExpired())
        {
          LOG.warning(MessageFormat.format("{0} not licensed: {1}", MSG_PREFIX, exc));
          LOG.warning("OAS Modbus Utilities: demo mode");
          setFaultCause(LEX.getText("License.Demo"));
          return;
        }

        LOG.severe(MessageFormat.format("{0} not licensed: {1}", MSG_PREFIX, exc));
        LOG.severe(MessageFormat.format("{0}: demo mode expired", MSG_PREFIX));
        setEnabled(false);
        setStatus(BStatus.fault);
        setFaultCause(LEX.getText("License.Missing"));
        stop();
      }
    }
  }

  @SuppressWarnings({"squid:S2139"})
  public void checkOASLicenseFeature() {
    // check tridium modbus license (throws an exception when not licensed)
    try {
      getLicenseFeature();
    }
    catch (LicenseException exc) {
      LOG.severe(MessageFormat.format("problem with tridium modbus license: {0}", exc));
      handleLicenseIsMissing();
      throw exc;
    }

    Feature feature;
    try {
      feature = Sys.getLicenseManager().getFeature(VENDOR, FEATURE);
      feature.check();
    }
    catch (LicenseException exc) {
      LOG.severe(MessageFormat.format("problem with oasModbusUtilities license: {0}", exc));
      handleLicenseIsMissing();
      throw new LocalizableRuntimeException(LEXICON_NAME, "License.Missing");
    }
  }

  @SuppressWarnings({"squid:S2139"})
  public void checkDeviceLimit(boolean checkAddDevice) {

    int deviceLimit = getLicenseDeviceLimit();

    try {

      BModbusAsyncDevice[] children = getChildren(BModbusAsyncDevice.class);
      int deviceCount = children.length + ((checkAddDevice) ? 1 : 0);

      if (deviceCount > deviceLimit) {
        throw new LocalizableRuntimeException(LEXICON_NAME, "License.DeviceLimitExceeded");
      }
    }
    catch (LocalizableRuntimeException exc) {
      throw exc;
    }
    catch (Exception exc) {
      LOG.severe(MessageFormat.format("{0}.checkDeviceLimit() cannot check device limit: {1}", LOG_PREFIX, exc));
      throw new LocalizableRuntimeException(LEXICON_NAME, "License.CannotCheckDeviceLimit", new Object[] {exc});
    }
  }



  public final int getLicenseDeviceLimit() {
    Feature feature;
    try {
      feature = Sys.getLicenseManager().getFeature(VENDOR, FEATURE);
      feature.check();

      String deviceLimit = feature.get("device.limit");

      if (deviceLimit.equals("none")) {

        return SUPPORTED_DEVICE_LIMIT;
      }

      return Integer.parseInt(deviceLimit);
    }
    catch (LicenseException exc) {
      LOG.warning("No oasModbusUtilities license found: use 16 as device Limit");
      return SUPPORTED_DEVICE_LIMIT;
    }
  }


  private void handleLicenseIsMissing() {
    setEnabled(false);
  }


}
