package com.oas.modbusutilities.register;

import javax.baja.control.BNumericPoint;
import javax.baja.control.BNumericWritable;
import javax.baja.driver.util.BPollFrequency;
import javax.baja.status.BStatusNumeric;

import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegister;
import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegisters;
import com.tridium.modbusCore.client.point.BModbusClientNumericProxyExt;
import com.tridium.modbusCore.datatypes.BFlexAddress;
import com.tridium.modbusCore.enums.BAddressFormatEnum;
import com.tridium.modbusCore.enums.BDataTypeEnum;
import com.tridium.modbusCore.enums.BRegisterTypeEnum;

/**
 * AsyncRegisterFactory
 *
 * Factory which helps creating Modbus Async Register, Preset and NumericWritables
 */
public class AsyncRegisterFactory {

  private AsyncRegisterFactory() { }
  /**
   *
   * @param address
   * @return
   */
  public static BNumericWritable makeNumericWritableRegister(String address) {
    BNumericWritable typeRegister = new BNumericWritable();
    typeRegister.setProxyExt(makeIntHoldingProxyExt(makeDecimalRegisterAddress(address)));
    return typeRegister;
  }

  /**
   *
   * @param address
   * @param frequency
   * @return
   */
  public static BNumericWritable makeNumericWritableRegister(String address, BPollFrequency frequency) {
    BNumericWritable typeRegister = new BNumericWritable();
    typeRegister.setProxyExt(makeIntHoldingProxyExt(makeDecimalRegisterAddress(address), frequency));
    return typeRegister;
  }

  /**
   *
   * @param address
   * @param frequency
   * @return
   */
  public static BNumericWritable makeNumericWritableRegister(String address, BPollFrequency frequency, double value) {
    BNumericWritable typeRegister = new BNumericWritable();
    typeRegister.setFallback(new BStatusNumeric(value));
    typeRegister.setProxyExt(makeIntHoldingProxyExt(makeDecimalRegisterAddress(address), frequency));
    return typeRegister;
  }

  /**
   *
   * @return
   */
  public static BNumericWritable makeNumericWritableInterface() {
    return new BNumericWritable();
  }

  /**
   *
   * @param address
   * @return
   */
  public static BNumericPoint makeNumericPointRegister(String address) {
    BNumericPoint typeRegister = new BNumericPoint();
    typeRegister.setProxyExt(makeIntHoldingProxyExt(makeDecimalRegisterAddress(address)));
    return typeRegister;
  }
  /**
   *
   * @param address
   * @param frequency
   * @return
   */
  public static BNumericPoint makeNumericPointRegister(String address, BPollFrequency frequency) {
    BNumericPoint typeRegister = new BNumericPoint();
    typeRegister.setProxyExt(makeIntHoldingProxyExt(makeDecimalRegisterAddress(address), frequency));
    return typeRegister;
  }
  /**
   *
   * @param address
   * @param frequency
   * @return
   */
  public static BNumericPoint makeSignedNumericPointRegister(String address, BPollFrequency frequency) {
    BNumericPoint typeRegister = new BNumericPoint();
    typeRegister.setProxyExt(makeSignedIntHoldingProxyExt(makeDecimalRegisterAddress(address), frequency));
    return typeRegister;
  }

  /**
   * Convinient method to create a integer type BModbusClientPresetRegisters with startingAddress of <code>address</code>
   * , <code>writeOnChange</code> and multible ModbusClientPresetRegister.
   * @param address startingAddress of the ModbusClientPresetRegisters Component
   * @param writeOnChange dataType of the ModbusClientPresetRegisters Component
   * @param defaultValue value which will be set on creation
   * @param names List of names for ModbusClientPresetRegister
   * @return multi Preset integer type BModbusClientPresetRegisters Component
   */
  public static BModbusClientPresetRegisters makeMultiIntegerPresets(String address, boolean writeOnChange, float defaultValue, String ...names) {
    BModbusClientPresetRegisters presets = makePresetRegisters(address, BDataTypeEnum.integerType, writeOnChange);

    for (String name : names) {
      presets.add(name, makePresetRegister(defaultValue));
    }

    return presets;
  }

  /**
   * Convinient method to create a integer type BModbusClientPresetRegisters with startingAddress of <code>address</code>
   * , <code>writeOnChange</code> and multible ModbusClientPresetRegister.
   * @param address startingAddress of the ModbusClientPresetRegisters Component
   * @param writeOnChange dataType of the ModbusClientPresetRegisters Component
   * @param names List of names for ModbusClientPresetRegister
   * @return multi Preset integer type BModbusClientPresetRegisters Component
   */
  public static BModbusClientPresetRegisters makeMultiIntegerPresets(String address, boolean writeOnChange, String ...names) {
    BModbusClientPresetRegisters presets = makePresetRegisters(address, BDataTypeEnum.integerType, writeOnChange);

    for (String name : names) {
      presets.add(name, makePresetRegister());
    }

    return presets;
  }

  /**
   * Convinient method to create a integer type BModbusClientPresetRegisters with startingAddress of <code>address</code>
   * , <code>writeOnChange</code> and a single ModbusClientPresetRegister.
   * @param address startingAddress of the ModbusClientPresetRegisters Component
   * @param writeOnChange dataType of the ModbusClientPresetRegisters Component
   * @param defaultValue value which will be set on creation
   * @return single Preset integer type BModbusClientPresetRegisters Component
   */
  public static BModbusClientPresetRegisters makeSingleIntegerPresets(String address, boolean writeOnChange, float defaultValue) {
    return makeSingleIntegerPresets(address, writeOnChange, defaultValue, "ModbusClientPresetRegister");
  }

  /**
   * Convinient method to create a integer type BModbusClientPresetRegisters with startingAddress of <code>address</code>
   * , <code>writeOnChange</code> and a single ModbusClientPresetRegister.
   * @param address startingAddress of the ModbusClientPresetRegisters Component
   * @param writeOnChange dataType of the ModbusClientPresetRegisters Component
   * @param defaultValue value which will be set on creation
   * @return single Preset integer type BModbusClientPresetRegisters Component
   */
  public static BModbusClientPresetRegisters makeSingleIntegerPresets(String address, boolean writeOnChange, float defaultValue, String presetName) {
    return makeMultiIntegerPresets(address, writeOnChange, defaultValue, presetName);
  }

  /**
   * Convinient method to create a integer type BModbusClientPresetRegisters with startingAddress of <code>address</code>
   * , <code>writeOnChange</code> and a single ModbusClientPresetRegister.
   * @param address startingAddress of the ModbusClientPresetRegisters Component
   * @param writeOnChange dataType of the ModbusClientPresetRegisters Component
   * @return single Preset integer type BModbusClientPresetRegisters Component
   */
  public static BModbusClientPresetRegisters makeSingleIntegerPresets(String address, boolean writeOnChange) {
    return makeSingleIntegerPresets(address, writeOnChange, "ModbusClientPresetRegister");
  }
  /**
   * Convinient method to create a integer type BModbusClientPresetRegisters with startingAddress of <code>address</code>
   * , <code>writeOnChange</code> and a single ModbusClientPresetRegister.
   * @param address startingAddress of the ModbusClientPresetRegisters Component
   * @param writeOnChange dataType of the ModbusClientPresetRegisters Component
   * @param presetName Name of the Presets subcomponent
   * @return single Preset integer type BModbusClientPresetRegisters Component
   */
  public static BModbusClientPresetRegisters makeSingleIntegerPresets(String address, boolean writeOnChange, String presetName) {

    return makeMultiIntegerPresets(address, writeOnChange, presetName);
  }

  /**
   * Convinient methode to create a integer type BModbusClientPresetRegisters with startingAddress of <code>address</code>
   * and <code>writeOnChange</code>
   * @param address startingAddress of the ModbusClientPresetRegisters Component
   * @param writeOnChange dataType of the ModbusClientPresetRegisters Component
   * @return empty integer type BModbusClientPresetRegisters Component
   */
  public static BModbusClientPresetRegisters makeIntegerPresetRegisters(String address, boolean writeOnChange) {
    return makePresetRegisters(address, BDataTypeEnum.integerType, writeOnChange);
  }

  /**
   * Convinient method to create a BModbusClientPresetRegisters of Data Type <code>dataType</code>, starting Address
   * <code>address</code> and <code>writeOnChange</code>
   * @param address startingAddress of the ModbusClientPresetRegisters Component
   * @param dataType dataType of the ModbusClientPresetRegisters Component
   * @param writeOnChange writeOnInputChange of the ModbusClientPresetRegisters Component
   * @param presetName Name of the Presets subcomponent
   * @return empty BModbusClientPresetRegisters Component
   */
  public static BModbusClientPresetRegisters makePresetRegisters(String address, BDataTypeEnum dataType, boolean writeOnChange) {
    BModbusClientPresetRegisters presets = new BModbusClientPresetRegisters();
    presets.setStartingAddress(makeDecimalRegisterAddress(address));
    presets.setWriteOnInputChange(writeOnChange);
    presets.setDataType(dataType);
    return presets;
  }

  /**
   * Convinient method to create a BModbusClientPresetRegister Component
   * @return Preset Register of type BModbusClientPresetRegister
   */
  public static BModbusClientPresetRegister makePresetRegister() {
    return makePresetRegister(0.0f);
  }

  /**
   * Convinient method to create a BModbusClientPresetRegister Component with a <code>defaultValue</code>
   * @param defaultValue value with will be set on creation
   * @return BModbusClientPresetRegister
   */
  public static BModbusClientPresetRegister makePresetRegister(float defaultValue) {
    BModbusClientPresetRegister preset = new BModbusClientPresetRegister();
    preset.setValue(defaultValue);
    return preset;
  }

  /**
   *
   * @param address
   * @return
   */
  private static BFlexAddress makeDecimalRegisterAddress(String address) {
    return makeRegisterAddress(BAddressFormatEnum.decimal, address);
  }

  /**
   *
   * @param fromat
   * @param address
   * @return
   */
  private static BFlexAddress makeRegisterAddress(BAddressFormatEnum fromat, String address) {
    return new BFlexAddress(fromat, address);
  }

  /**
   *
   * @param address
   * @return
   */
  private static BModbusClientNumericProxyExt makeIntHoldingProxyExt(BFlexAddress address) {
    return makeProxyExt(address, BRegisterTypeEnum.holding, BDataTypeEnum.integerType);
  }

  /**
   *
   * @param address
   * @param frequency
   * @return
   */
  private static BModbusClientNumericProxyExt makeIntHoldingProxyExt(BFlexAddress address, BPollFrequency frequency) {
    return makeProxyExt(address, BRegisterTypeEnum.holding, BDataTypeEnum.integerType, frequency);
  }

  /**
   *
   * @param address
   * @param frequency
   * @return
   */
  private static BModbusClientNumericProxyExt makeSignedIntHoldingProxyExt(BFlexAddress address, BPollFrequency frequency) {
    return makeProxyExt(address, BRegisterTypeEnum.holding, BDataTypeEnum.signedInteger, frequency);
  }

  /**
   * Convinient Method to create an HoldingRegsiter of Integer DataType with an Address specified by <code>format</code> and <code>address</code>
   * @param format Format of the Register Address
   * @param address Address of the Register
   * @return <code>BModbusClientNumericProxyExt</code>
   *
   * @see BAddressFormatEnum
   * @see BModbusClientNumericProxyExt
   */
  @SuppressWarnings("unused")
  private static BModbusClientNumericProxyExt makeIntHoldingProxyExt(BAddressFormatEnum format, String address) {
    return makeProxyExt(format, address, BRegisterTypeEnum.holding, BDataTypeEnum.integerType);
  }


  /**
   * Convinient Method which requiere the <code>format</code> of the Register Address
   *
   * @param fromat Specifies the Format of the Address (Hex, Dec, Modbus)
   * @param address Specifies the Register Address
   * @param regType Specifies the Type of the Register (Holding, Coil, etc.)
   * @param dataType Specifies the Type of the Register Data (int, float, long, signed unsigned )
   * @return <code>BModbusClientNumericProxyExt</code> of <code>format</code> and <code>address</code>, <code>regType</code> and <code>dataType</code>
   * @see AsyncRegisterFactory#makeProxyExt(BFlexAddress, BRegisterTypeEnum, BDataTypeEnum)
   */
  private static BModbusClientNumericProxyExt makeProxyExt(BAddressFormatEnum fromat, String address, BRegisterTypeEnum regType, BDataTypeEnum dataType) {
    return makeProxyExt(makeRegisterAddress(fromat, address), regType, dataType);
  }

  /**
   * Convinient Method to create a <code>ModbusClientNumericProxyExt</code>
   * @param address Modbus Address of the ProxyExt
   * @param regType Modbus Register Type
   * @param dataType Data Type of the Register
   * @return <code>BModbusClientNumericProxyExt</code> of <code>address</code>, <code>regType</code> and <code>dataType</code>
   */
  private static BModbusClientNumericProxyExt makeProxyExt(BFlexAddress address, BRegisterTypeEnum regType, BDataTypeEnum dataType) {
    BModbusClientNumericProxyExt proxyExt = new BModbusClientNumericProxyExt();
    proxyExt.setDataAddress(address);
    proxyExt.setRegType(regType);
    proxyExt.setDataType(dataType);

    return proxyExt;
  }

  /**
   * Convinient Method to create a <code>ModbusClientNumericProxyExt</code>
   * @param address Modbus Address of the ProxyExt
   * @param regType Modbus Register Type
   * @param dataType Data Type of the Register
   * @param frequency frequency to poll the Register
   * @return <code>BModbusClientNumericProxyExt</code> of <code>address</code>, <code>regType</code> and <code>dataType</code>
   */
  private static BModbusClientNumericProxyExt makeProxyExt(BFlexAddress address, BRegisterTypeEnum regType, BDataTypeEnum dataType, BPollFrequency frequency) {
    BModbusClientNumericProxyExt proxyExt = new BModbusClientNumericProxyExt();
    proxyExt.setDataAddress(address);
    proxyExt.setRegType(regType);
    proxyExt.setDataType(dataType);
    proxyExt.setPollFrequency(frequency);

    return proxyExt;
  }
}
