package com.oas.modbusutilities.utils;

import java.text.MessageFormat;
import java.util.logging.Logger;

import javax.baja.sys.BComponent;
import javax.baja.sys.Property;


import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegister;
import com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegisters;


/**
 * LinkUtils
 */
@SuppressWarnings({
  "squid:S2629", // Invoke LOGs conditionally will bloat up the code
})
public class AsyncLinkUtils {

  private AsyncLinkUtils() {}

  private static final String LOG_PREFIX = AsyncLinkUtils.class.getSimpleName();
  private static final Logger LOG = Logger.getLogger("oasModbusUtilities");

  /**
   * Convinient method when PresetRegister is available.
   * @param preset
   * @param comp
   * @param prop
   * @return
   */
  public static void linkToPreset(BModbusClientPresetRegister preset, BComponent comp, String prop) {

    Property srcSlot = comp.getProperty(prop);
    if (null == srcSlot) {
      LOG.warning(MessageFormat.format("{0}.linkInterToPreset() Property >>{1}<< of {2} could not be found", LOG_PREFIX, prop, comp.getName()));
      return;
    }

    Property destSlot = preset.getProperty("value");
    if (null == destSlot) {
      LOG.warning(MessageFormat.format("{0}.linkInterToPreset() Property >>Value<< of {1} could not be found", LOG_PREFIX, preset.getName()));
      return;
    }


    LOG.fine(MessageFormat.format("{0}.linkInterToPreset() create Link from {1}.{2} to {3}", LOG_PREFIX, comp.getName(), srcSlot.getName(), destSlot.getName()));
    preset.add(preset.getName() + "link", LinkUtils.createLink(comp, srcSlot, preset, destSlot));
  }



  /**
   * Convinient Method to created a link from any Property to Preset created by
   * {@link com.oas.modbusutilities.register.AsyncRegisterFactory#makeIntegerPresetRegisters(String, boolean)}.
   *
   * @param preset Modbus Preset to link to
   * @param comp Source component
   * @param prop Source Property
   */
  public static void linkToPresets(BModbusClientPresetRegisters presets, BComponent comp, String prop) {

    AsyncLinkUtils.linkToPresets(presets, "ModbusClientPresetRegister", comp, prop);
  }


  /**
   * Convinient Method to created a link from any Property to Preset Register Container of type
   * BModbusClientPresetRegisters
   *
   * @param presets Modbus Preset Registers which contains the preset to link.
   * @param presetName Name of the preset to link.
   * @param comp Source component.
   * @param prop Source Property.
   * @see com.tridium.modbusCore.client.datatypes.BModbusClientPresetRegisters
   */
  public static void linkToPresets(BModbusClientPresetRegisters presets, String presetName, BComponent comp, String prop) {

    Property srcSlot = comp.getProperty(prop);
    if (null == srcSlot) {
      LOG.warning(MessageFormat.format("{0}.linkInterToPreset() Property >>{1}<< of {2} could not be found", LOG_PREFIX, prop, comp.getName()));
      return;
    }

    Property presetRegProp = presets.getProperty(presetName);

    if(null == presetRegProp) {
      LOG.warning(MessageFormat.format("{0}.linkInterToPreset() Property >>ModbusClientPresetRegister<< of {1} is not present", LOG_PREFIX, presets.getName()));
      return;
    }

    BModbusClientPresetRegister presetRegister = (BModbusClientPresetRegister)presets.get(presetRegProp);

    if (null == presetRegister) {
      LOG.warning(MessageFormat.format("{0}.linkInterToPreset() Property >>ModbusClientPresetRegister<< of {1} could not be found", LOG_PREFIX, presets.getName()));
      return;
    }

    presetRegister.loadSlots();

    Property destSlot = presetRegister.getProperty("value");
    if (null == destSlot) {
      LOG.warning(MessageFormat.format("{0}.linkInterToPreset() Property >>Value<< of {1} could not be found", LOG_PREFIX, presetRegister.getName()));
      return;
    }


    LOG.fine(MessageFormat.format("{0}.linkInterToPreset() create Link from {1}.{2} to {3}", LOG_PREFIX, comp.getName(), srcSlot.getName(), destSlot.getName()));
    presetRegister.add("presetLink", LinkUtils.createLink(comp, srcSlot, presetRegister, destSlot));
  }
}
