define( function(require) {
    'use strict';

    var Widget = require('bajaux/Widget');
    var subscriberMixIn = require('bajaux/mixin/subscriberMixIn');

    var ModuleTypeInfo = require("nmodule/oasModbusUtilitiesCore/rc/script/moduleTypeInfo");
    var ModuleAddress = require("nmodule/oasModbusUtilitiesCore/rc/script/ModuleAddress");

    var AiPoint = require("nmodule/oasModbusUtilitiesCore/rc/widgets/point/aiPointStatus");

    var IoModuleWidget = require('hbs!nmodule/oasModbusUtilitiesCore/rc/template/iomodule');

    require('css!nmodule/oasModbusUtilitiesCore/rc/style/module');

    var lexicons = require('lex!oasModbusUtilitiesCore');
    var lex = lexicons[0];


    var aiStatus = function() {

        Widget.apply(this, arguments);
        subscriberMixIn(this);



        this.$widget = null;
        this.$moduleAddress = null;
    };

    aiStatus.prototype = Object.create(Widget.prototype);
    aiStatus.prototype.constructor = aiStatus;


    aiStatus.prototype.doInitialize = function(jq) {


        jq.html(IoModuleWidget({
            view_type: lex.get("OasModule.header.status"),
            module_label: lex.get("OasModule.header.module"),
            module_name: "--",
            type_label: lex.get("OasModule.header.type"),
            module_type: "--",
            autobaud_label: lex.get("OasModule.header.autobaud"),
            autobaud_activ: "--",
            baud_label: lex.get("OasModule.header.baud"),
            module_baud: "--",
            firmware_label: lex.get("OasModule.header.firmware"),
            module_fw: "--",
            busAddress_label: lex.get("OasModule.header.busAddress"),
            bus_address: "--",
            show_buttons: false,
            slot_class: 'ai-slot',
            slots: [ 'eight', 'seven', 'six', 'five', 'four', 'three', 'two', 'one' ],

        }));

        this.$moduleAddress = new ModuleAddress(jq.find('.moduleaddress-container'), 0);
        this.$moduleAddress.render();


    };


    aiStatus.prototype.doLoad = function(obj) {

        var that = this;
        var jq = this.jq();
        this.$widget = obj;

        var module_name = this.$widget.$parent.getName();
        jq.find("div.module-name p.info").text(module_name);

        var type = this.$widget.getSetPointModuleType();
        jq.find("div.module-type p.info").text(ModuleTypeInfo.searchModuleType(type));

        var autobauding = this.$widget.getSetPointAutobauding();
        jq.find("div.module-autobaud p.info").text(autobauding.toString());

        var baudrate = this.$widget.getSetPointBaudrate();
        jq.find("div.module-baud p.info").text(ModuleTypeInfo.searchBaudrate(baudrate));

        var fw = this.$widget.getDisplayFirmware();
        jq.find("div.module-fw p.info").text(fw.getValue());

        var bus_address = this.$widget.getDisplayAddress().getValue();
        jq.find("div.module-busaddress").html(
            '<p>' + lex.get("OasModule.header.busAddress") + ": " + bus_address + '</p>'
        );
        console.log("module address: " + bus_address);
        this.$moduleAddress.update(bus_address);

        var points = this.getPoints();
        var position = [ 'eight', 'seven', 'six', 'five', 'four', 'three', 'two', 'one' ];


        points.forEach(function(point) {
            // console.log("load widget of " + point.getName());
            var pos = position.pop();
            var widget = new AiPoint(that);
            widget.initialize(jq.find('.ai-slot-' + pos)).then(function() {
                widget.load(point);
            });
        });

        this.getSubscriber().attach("changed", function(prop) {
            if (prop.getName() === 'displayAddress') {

                that.$moduleAddress.update(prop.$val.getValue());

                jq.find("div.module-busaddress").html(
                    '<p>' + lex.get("OasModule.header.busAddress") + ": " + prop.$val.getValue() + '</p>'
                );
            }
        });

    };

    /////////////////////////////////////////////////////////////
    //#                                                        #
    //#                                                        #
    //#                                                        #
    //#                 Convenient Functions                   #
    //#                                                        #
    //#                                                        #
    //#                                                        #
    /////////////////////////////////////////////////////////////

    aiStatus.prototype.getPoints = function() {
        if (this.$widget) {
            return this.$widget.getSlots().properties().is('oasModbusUtilitiesCore:AiPoint').toValueArray();
        }
        else {
            return [];
        }
    };

    return aiStatus;
});
