define( function(require) {
    'use strict';

    var Widget = require('bajaux/Widget');
    var subscriberMixIn = require('bajaux/mixin/subscriberMixIn');

    var ModuleTypeInfo = require("nmodule/oasModbusUtilitiesCore/rc/script/moduleTypeInfo");
    var ModuleAddress = require("nmodule/oasModbusUtilitiesCore/rc/script/ModuleAddress");

    var DiPoint = require("nmodule/oasModbusUtilitiesCore/rc/widgets/point/diPointStatus");
    var Ao3Point = require("nmodule/oasModbusUtilitiesCore/rc/widgets/point/ao3PointStatus");

    var IoModuleWidget = require('hbs!nmodule/oasModbusUtilitiesCore/rc/template/iomodule');


    require('css!nmodule/oasModbusUtilitiesCore/rc/style/module');


    var lexicons = require('lex!oasModbusUtilitiesCore');
    var lex = lexicons[0];


    var ao3Status = function() {
        var that = this;
        Widget.apply(this, arguments);

        subscriberMixIn(that);

        this.$widget = null;

        this.$moduleAddress = null;
    };

    ao3Status.prototype = Object.create(Widget.prototype);
    ao3Status.prototype.constructor = ao3Status;


    ao3Status.prototype.doInitialize = function(jq) {

        jq.html(IoModuleWidget({
            view_type: lex.get("OasModule.header.status"),
            module_label: lex.get("OasModule.header.module"),
            module_name: "--",
            type_label: lex.get("OasModule.header.type"),
            module_type: "--",
            autobaud_label: lex.get("OasModule.header.autobaud"),
            autobaud_activ: "--",
            baud_label: lex.get("OasModule.header.baud"),
            module_baud: "--",
            firmware_label: lex.get("OasModule.header.firmware"),
            module_fw: "--",
            busAddress_label: lex.get("OasModule.header.busAddress"),
            bus_address: "--",
            show_buttons: false,
            slot_class: 'ao3-slot',
            slots: ['eight', 'seven', 'six', 'five', 'four', 'three', 'two', 'one'],
        }));


        this.$moduleAddress = new ModuleAddress(jq.find('.moduleaddress-container'), 0);
        this.$moduleAddress.render();
    };



    ao3Status.prototype.doLoad = function(comp) {

        var that = this;
        var jq = this.jq();
        this.$widget = comp;

        var module_name = this.$widget.$parent.getName();
        jq.find("div.module-name p.info").text(module_name);

        var type = this.$widget.getSetPointModuleType();
        jq.find("div.module-type p.info").text(ModuleTypeInfo.searchModuleType(type));

        var autobauding = this.$widget.getSetPointAutobauding();
        jq.find("div.module-autobaud p.info").text(autobauding.toString());

        var baudrate = this.$widget.getSetPointBaudrate();
        jq.find("div.module-baud p.info").text(ModuleTypeInfo.searchBaudrate(baudrate));

        var fw = this.$widget.getDisplayFirmware();
        jq.find("div.module-fw p.info").text(fw.getValue());

        var bus_address = this.$widget.getDisplayAddress().getValue();
        jq.find("div.module-busaddress").html(
            '<p>' + lex.get("OasModule.header.busAddress") + ": " + bus_address + '</p>'
        );
        console.log("module address: " + bus_address);
        this.$moduleAddress.update(bus_address);

        var ao3Points = this.getAo3Points();
        var diPoints = this.getDiPoints();
        var position = ['eight', 'seven', 'six', 'five', 'four', 'three', 'two', 'one' ];


        ao3Points.forEach(function(point) {
            var pos = position.pop();
            var widget = new Ao3Point(that);
            widget.initialize(jq.find('.ao3-slot-' + pos)).then(function() {
                widget.load(point);
            });
        });

        diPoints.forEach(function(point) {
            var pos = position.pop();
            var widget = new DiPoint(that);
            widget.initialize(jq.find('.ao3-slot-' + pos)).then(function() {
                widget.load(point);
            });
        });


        this.getSubscriber().attach("changed", function(prop) {
            if (prop.getName() === 'displayAddress') {

                that.$moduleAddress.update(prop.$val.getValue());

                jq.find("div.module-busaddress").html(
                    '<p>' + lex.get("OasModule.header.busAddress") + ": " + prop.$val.getValue() + '</p>'
                );
            }
        });
    };



    /////////////////////////////////////////////////////////////
    //#                                                        #
    //#                                                        #
    //#                                                        #
    //#                 Convenient Functions                   #
    //#                                                        #
    //#                                                        #
    //#                                                        #
    /////////////////////////////////////////////////////////////


    ao3Status.prototype.getDiPoints = function() {
        if (this.$widget) {
            return this.$widget.getSlots().properties().is('oasModbusUtilitiesCore:DiPoint').toValueArray();
        }
        else {
            return [];
        }
    };

    ao3Status.prototype.getAo3Points = function() {
        if (this.$widget) {
            return this.$widget.getSlots().properties().is('oasModbusUtilitiesCore:Ao3Point').toValueArray();
        }
        else {
            return [];
        }
    };

    return ao3Status;
});
