define(function(require) {
    'use strict';

    var baja = require("baja!");
    var Promise = require("Promise");
    var $ = require('jquery');
    var _ = require('underscore');
    var fe = require('nmodule/webEditors/rc/fe/fe');
    var BaseEditor = require('nmodule/webEditors/rc/fe/baja/BaseEditor');

    var KEYS = [
        'ok',
        'modified',
        'wrongModuleType',
        'autobaudingModified',
        'baudrateModified',
        'timeoutModified',
        'offline',
        'configFailure'
    ];


    /**
     * @class
     * @extends module:nmodule/webEditors/rc/fe/baja/BaseEditor
     */
    var ConfigurationStatusEditor = function() {
        BaseEditor.apply(this, arguments);
    };

    ConfigurationStatusEditor.prototype = Object.create(BaseEditor.prototype);
    ConfigurationStatusEditor.prototype.constructor = ConfigurationStatusEditor;

    ConfigurationStatusEditor.prototype.doInitialize = function(dom) {

        return Promise.all(_.map(KEYS, function(key) {
            var subDom = $('<div/>').addClass(key).appendTo(dom);

            return fe.buildFor({
                dom: subDom,
                properties: {trueText: key, falseText: key},
                value: false,
                formFactor: 'mini'
            });
        }));
    };


    ConfigurationStatusEditor.prototype.doLoad = function(component) {
        var that = this;

        var kids = this.getChildEditors();
        var bits = component.$val;

        return Promise.all(_.map(KEYS, function(showKey, i) {
            return kids[i].load(!!(bits & (1 << i)));
        }));
    };

    ConfigurationStatusEditor.prototype.doRead = function () {

        return this.getChildEditors().readAll().then(function(boolArr) {
            var newBits = _.reduce(boolArr, function(memo, bool, i) {
                return bool ? memo + (1 << i) : memo;
            }, 0);
            var myStatus = baja.$('oasModbusUtilitiesCore:ConfigurationStatus');
            myStatus.$val = newBits;
            return myStatus;
        });
    };


    ConfigurationStatusEditor.prototype.doDestroy = function() {
        return this.getChildEditors().destroyAll();
    };

    ConfigurationStatusEditor.prototype.doEnabled = function (enabled) {
        return this.getChildEditors().setAllEnabled(enabled);
    };

    ConfigurationStatusEditor.prototype.doReadonly = function (readonly) {
        return this.getChildEditors().setAllReadonly(readonly);
    };

    return (ConfigurationStatusEditor);
});
