define(function(require) {
    'use strict';

    var baja = require("baja!");
    var Widget = require('bajaux/Widget');
    var subscriberMixIn = require('bajaux/mixin/subscriberMixIn');

    var Header = require("nmodule/oasModbusUtilitiesCore/rc/script/Header");

    var ModuleViewTempl = require('hbs!nmodule/oasModbusUtilitiesCore/rc/template/moduleView');
    require('css!nmodule/oasModbusUtilitiesCore/rc/style/moduleView');

    var ModuleStatus = function () {
        Widget.apply(this, arguments);
        subscriberMixIn(this);

        this.$header = new Header();
    };


    ModuleStatus.prototype = Object.create(Widget.prototype);
    ModuleStatus.prototype.constructor = ModuleStatus;



    ModuleStatus.prototype.doInitialize = function(dom) {
        dom.html(ModuleViewTempl({
            header: this.$header.toSafeString(),
        }));
    };

    ModuleStatus.prototype.doLoad = function(comp) {

        var that = this;

        var network = comp.getParent();

        network.loadSlots().then(function() {
            var homeOrd = new baja.Ord.make(network.getHomeOrd());
            that.$header.setHomeLink(homeOrd.toUri());


            var logoOrd = new baja.Ord.make(network.getLogoOrd());
            that.$header.setBrandLogo(logoOrd.toUri());

        });

        var moduleConfiguration = comp.getConfiguration();

        moduleConfiguration.loadSlots().then(function() {
            // console.log(moduleConfiguration);
            var configTyp = moduleConfiguration.getType();

            // console.log(configTyp);
            that.renderConfig(moduleConfiguration, configTyp);

        });

    };


    ModuleStatus.prototype.renderConfig = function(config, type) {

        var jq = this.jq();

        var select = jq.find('.module-content');

        if (type.is('oasModbusUtilitiesAsync:AsyncAiConfiguration')) {
            // console.log('found ai config');
            this.renderAiConfig(config, select);
        }
        else if (type.is('oasModbusUtilitiesAsync:AsyncAo3Configuration')) {
            // console.log('found ao3 config');
            this.renderAo3Config(config, select);
        }
        else if (type.is('oasModbusUtilitiesAsync:AsyncAoConfiguration')) {
            // console.log('found ao config');
            this.renderAoConfig(config, select);
        }
        else if (type.is('oasModbusUtilitiesAsync:AsyncDiConfiguration')) {
            // console.log('found di config');
            this.renderDiConfig(config, select);
        }
        else if (type.is('oasModbusUtilitiesAsync:AsyncDoConfiguration')) {
            // console.log('found do config');
            this.renderDoConfig(config, select);
        }
    };

    ModuleStatus.prototype.renderAiConfig = function(config, select) {

        var ConfigWidget = require('nmodule/oasModbusUtilitiesCore/rc/widgets/config/aiStatus');
            var widget = new ConfigWidget();
            widget.initialize(select).then(function() {
                widget.load(config);
        });
    };

    ModuleStatus.prototype.renderAo3Config = function(config, select) {

        var ConfigWidget = require('nmodule/oasModbusUtilitiesCore/rc/widgets/config/ao3Status');
            var widget = new ConfigWidget();
            widget.initialize(select).then(function() {
                widget.load(config);
        });
    };

    ModuleStatus.prototype.renderAoConfig = function(config, select) {

        var ConfigWidget = require('nmodule/oasModbusUtilitiesCore/rc/widgets/config/aoStatus');
            var widget = new ConfigWidget();
            widget.initialize(select).then(function() {
                widget.load(config);
        });
    };

    ModuleStatus.prototype.renderDiConfig = function(config, select) {

        var ConfigWidget = require('nmodule/oasModbusUtilitiesCore/rc/widgets/config/diStatus');
            var widget = new ConfigWidget();
            widget.initialize(select).then(function() {
                widget.load(config);
        });
    };

    ModuleStatus.prototype.renderDoConfig = function(config, select) {

        var ConfigWidget = require('nmodule/oasModbusUtilitiesCore/rc/widgets/config/doStatus');
            var widget = new ConfigWidget();
            widget.initialize(select).then(function() {
                widget.load(config);
        });
    };





    return ModuleStatus;
});
