define(function(require) {
    'use strict';


    var Widget = require('bajaux/Widget');
    var subscriberMixIn = require('bajaux/mixin/subscriberMixIn');

    var FeDialog = require('nmodule/webEditors/rc/fe/feDialogs');

    var SlotTemplate = require('hbs!nmodule/oasModbusUtilitiesCore/rc/template/aiSlot');
    var RatioTemplate = require('hbs!nmodule/oasModbusUtilitiesCore/rc/template/ratio');
    var OffsetTemplate = require('hbs!nmodule/oasModbusUtilitiesCore/rc/template/offset');


    require('css!nmodule/oasModbusUtilitiesCore/rc/style/aislot');


    var characteristics = Object.freeze({
        "0...10 V" : 0,
        "0...5 Ohm" : 2,
        "0...15 Ohm" : 3,
        "Pt100" : 5,
        "Pt1000" : 6,
        "Ni1000" : 7,
        "Ni1000 LnG" : 8,
        "KTY 81-110" : 9,
        "KTY 81-210" : 10,
        "NTC 20k" : 11,
        "NTC 10k" : 12
    });

    var AiPointConfig = function(configuration) {

        Widget.apply(this, arguments);
        subscriberMixIn(this);

        this.$configuration = configuration || null;
        this.$component = null;
    };


    AiPointConfig.prototype = Object.create(Widget.prototype);
    AiPointConfig.prototype.constructor = AiPointConfig;


    AiPointConfig.prototype.doInitialize = function(dom) {
        dom.html(SlotTemplate({
            slot_name: "--",
            slot_characteristic: "--",
            slot_show: false,
        }));
    };

    AiPointConfig.prototype.doLoad = function(comp) {

        var that = this;
        var jq = this.jq();

        this.$component = comp;

        jq.find(".ai-slot-header p").text(comp.getName().toUpperCase());

        this.updateChracteristic();

        this.getSubscriber().attach('changed', function(prop) {

            if (prop.getName() === 'activeCharacteristicValue') {
                that.updateChracteristic();
            }
            else if (prop.getName() === 'minIn' ||
                     prop.getName() === 'maxIn' ||
                     prop.getName() === 'minOut' ||
                     prop.getName() === 'maxOut') {
                that.updateRatio();
            }
            else if (prop.getName() === 'offset') {
                that.updateOffset();
            }
        });

        jq.on('click', '.char-handle', function() {

            FeDialog.action({component: comp, slot: 'setCharacteristic'}).then(function(decision) {
                that.setModified(true);
            });
        });

        jq.on('click', '.minIn', function() {

            FeDialog.action({component: comp, slot: 'setMinInValue'}).then(function(decision) {
                that.setModified(true);
            });
        });

        jq.on('click', '.maxIn', function() {

            FeDialog.action({component: comp, slot: 'setMaxInValue'}).then(function(decision) {
                that.setModified(true);
            });
        });

        jq.on('click', '.minOut', function() {
            FeDialog.action({component: comp, slot: 'setMinOutValue'}).then(function(decision) {
                that.setModified(true);
            });
        });

        jq.on('click', '.maxOut', function() {
            FeDialog.action({component: comp, slot: 'setMaxOutValue'}).then(function(decision) {
                that.setModified(true);
            });
        });

        jq.on('click', '.offset', function() {

            FeDialog.action({component: comp, slot: 'setOffsetValue'}).then(function(decision) {
                that.setModified(true);
            });
        });
    };

    AiPointConfig.prototype.updateChracteristic = function() {
        var jq = this.jq();

        var characteristic = this.$component.getActiveCharacteristicValue();

        jq.find(".characteristic p").text(this.searchCharacteristic(characteristic));

        var ai_prop = jq.find(".properties");

        if (characteristic.$ordinal === 0) {
            ai_prop.html(RatioTemplate({
                slot_minIn: this.$component.getMinIn(),
                slot_maxIn: this.$component.getMaxIn(),
                slot_minOut: this.$component.getMinOut(),
                slot_maxOut: this.$component.getMaxOut(),
            }));
        }
        else {
            ai_prop.html(OffsetTemplate({
                slot_offset: this.$component.getOffset()
            }));
        }
    };

    AiPointConfig.prototype.updateRatio = function() {

        var jq = this.jq();

        var characteristic = this.$component.getActiveCharacteristicValue();
        if (characteristic.$ordinal === 0) {
            jq.find('.minIn').text(this.$component.getMinIn());
            jq.find('.maxIn').text(this.$component.getMaxIn());
            jq.find('.minOut').text(this.$component.getMinOut());
            jq.find('.maxOut').text(this.$component.getMaxOut());
        }
    };

    AiPointConfig.prototype.updateOffset = function() {

        var jq = this.jq();

        var characteristic = this.$component.getActiveCharacteristicValue();
        if (characteristic.$ordinal !== 0) {
            jq.find('.offset').text(this.$component.getOffset());
        }
    };


    AiPointConfig.prototype.searchCharacteristic = function(key) {
        for (var value in characteristics) {
            if (characteristics[value] == key) {
                return value.toUpperCase();
            }
        }
    };


    AiPointConfig.prototype.setModified = function(modified) {
        try {
            this.$configuration.setModified(modified);
        }
        catch(error) {
            // nothing to do
        }
    };

    return AiPointConfig;
});
