define(function(require) {
    'use strict';


    var Widget = require('bajaux/Widget');
    var subscriberMixIn = require('bajaux/mixin/subscriberMixIn');

    var SlotTemplate = require('hbs!nmodule/oasModbusUtilitiesCore/rc/template/ao3pSlot');

    require('css!nmodule/oasModbusUtilitiesCore/rc/style/ao3pslot');

    var lexicons = require('lex!oasModbusUtilitiesCore');
    var lex = lexicons[0];

    var Ao3PointStatus = function(configuration) {

        Widget.apply(this, arguments);
        subscriberMixIn(this);

        this.$configuration = configuration || null;
        this.$component = null;
    };


    Ao3PointStatus.prototype = Object.create(Widget.prototype);
    Ao3PointStatus.prototype.constructor = Ao3PointStatus;


    Ao3PointStatus.prototype.doInitialize = function(dom) {
        dom.html(SlotTemplate({
            slot_name: "--",
            show_shutter: true,
            runtimes_label: lex.get("OasModule.slot.runtimes"),
            runtimesUp_label: lex.get("OasModule.slot.up"),
            slot_upTime: "--",
            runtimesDown_label: lex.get("OasModule.slot.down"),
            slot_downTime: "--",
            switchDelay_label: lex.get("OasModule.slot.switchingdelay"),
            onOffDelay_label: lex.get("OasModule.on") + "/" + lex.get("OasModule.off"),
            slot_onOffDelay: "--",
            changeOverDelay_label: lex.get("OasModule.slot.changeOver"),
            overrunDelay_label: lex.get('OasModule.slot.overrunDelay'),
            slot_changeOverDelay: "--",
            slot_overrunUp: "--",
            slot_overrunDown: "--",
        }));
    };

    Ao3PointStatus.prototype.doLoad = function(comp) {

        var that = this;
        var jq = this.jq();
        this.$component = comp;

        jq.find(".ao3-slot-header p").text(comp.getName().toUpperCase());

        this.updateRuntimeUp();
        this.updateRuntimeDown();
        this.updateOverrunUp();
        this.updateOverrunDown();
        this.updateOnOffDelay();
        this.updateChangeOverDelay();


        this.getSubscriber().attach('changed', function(prop) {

            // console.log(prop.getName() + " has changes: " + prop.$val);
            if (prop.getName() === "activeOnSideSwitchValue" ) {
                that.updateOnSideSwitch();
            }
            else if (prop.getName() === "activeRuntimeUp") {
                that.updateRuntimeUp();
            }
            else if (prop.getName() === "activeRuntimeDown") {
                that.updateRuntimeDown();
            }
            else if (prop.getName() === "activeOverrunUpTime") {
                that.updateOverrunUp();
            }
            else if (prop.getName() === "activeOverrunDownTime") {
                that.updateOverrunDown();
            }
            else if (prop.getName() === "activeOnOffDelay") {
                that.updateOnOffDelay();
            }
            else if (prop.getName() === "activeChangeOverDelay") {
                that.updateChangeOverDelay();
            }

        });

    };


    Ao3PointStatus.prototype.setModified = function(modified) {
        try {
            this.$configuration.setModified(modified);
        }
        catch(error) {
            // nothing to do
        }
    };

    Ao3PointStatus.prototype.updateOnSideSwitch = function() {
        var jq = this.jq();
        var onSideSwitch = this.$component.getActiveOnSideSwitchValue().getValue();
        // console.log("change onSideSwitch to " + onSideSwitch);
        if (onSideSwitch) {
            jq.find('.shutter-switch').removeClass('hidden');
        }
        else {
            jq.find('.shutter-switch').addClass('hidden');
        }
    };


    Ao3PointStatus.prototype.updateRuntimeUp = function() {
        var runtimeUp = this.$component.getActiveRuntimeUp().getValueDisplay();
        this.jq().find('.upTime').text(runtimeUp);
    };

    Ao3PointStatus.prototype.updateRuntimeDown = function() {
        var runtimeDown = this.$component.getActiveRuntimeDown().getValueDisplay();
        this.jq().find('.downTime').text(runtimeDown);
    };

    Ao3PointStatus.prototype.updateOverrunUp = function() {
        var overrunUpDelay = this.$component.getActiveOverrunUpTime().getValueDisplay();
        this.jq().find('.overrunUpTime').text(overrunUpDelay);
    };

    Ao3PointStatus.prototype.updateOverrunDown = function() {
        var overrunDownDelay = this.$component.getActiveOverrunDownTime().getValueDisplay();
        this.jq().find('.overrunDownTime').text(overrunDownDelay);
    };

    Ao3PointStatus.prototype.updateOnOffDelay = function() {
        var onOffDelay = this.$component.getActiveOnOffDelay().getValueDisplay();
        this.jq().find('.onOffDelay').text(onOffDelay);
    };

    Ao3PointStatus.prototype.updateChangeOverDelay = function() {
        var changeOverDelay = this.$component.getActiveChangeOverDelay().getValueDisplay();
        this.jq().find('.changeOverDelay').text(changeOverDelay);
    };


    return Ao3PointStatus;
});
