define(function(require) {
    'use strict';

    var baja = require('baja!');

    var Widget = require('bajaux/Widget');
    var subscriberMixIn = require('bajaux/mixin/subscriberMixIn');

    var FeDialog = require('nmodule/webEditors/rc/fe/feDialogs');

    var SlotTemplate = require('hbs!nmodule/oasModbusUtilitiesCore/rc/template/aoSlot');


    require('css!nmodule/oasModbusUtilitiesCore/rc/style/aoslot');

    var lexicons = require('lex!oasModbusUtilitiesCore');
    var lex = lexicons[0];

    var AiPointConfig = function(configuration) {

        Widget.apply(this, arguments);
        subscriberMixIn(this);

        this.$configuration = configuration || null;
        this.$component = null;
    };


    AiPointConfig.prototype = Object.create(Widget.prototype);
    AiPointConfig.prototype.constructor = AiPointConfig;


    AiPointConfig.prototype.doInitialize = function(dom) {
        dom.html(SlotTemplate({
            slot_name: "--",
            switch_status: false,
            checked: "checked" ,
            safePos_label: lex.get("OasModule.slot.safePos"),
            slot_savePosStatus: lex.get("OasModule.off"),
            position_label: lex.get("OasModule.slot.position"),
            slot_savePosActive: "deactivated",
            slot_savePosValue: "--"
        }));
    };

    AiPointConfig.prototype.doLoad = function(comp) {

        var that = this;
        var jq = this.jq();

        this.$component = comp;

        jq.find(".ao-slot-header p").text(comp.getName().toUpperCase());


        this.updateSwitchStatus();
        this.updateSwitchValue();
        this.updateSavePosition();


        this.getSubscriber().attach('changed', function(prop) {
            if (prop.getName() === 'activeOnSideSwitchStatus') {
                that.updateSwitchStatus();
            }
            else if (prop.getName() === 'activeOnSideSwitchValue') {
                that.updateSwitchValue();
            }
            else if (prop.getName() === 'activeSavePositionStatus' ||
                     prop.getName() === 'activeSavePositionValue') {
                that.updateSavePosition();
            }


        });

        jq.on('click', '.ao-save-pos', function(event) {
            var value = baja.$('baja:StatusBoolean');


            var savePos = jq.find(".ao-save-pos");
            var savePosValue = jq.find(".ao-sp-value");

            if (savePos.text() === lex.get("OasModule.on")) {
                value.setValue(false);
                savePos.text(lex.get("OasModule.off"));
                savePosValue.text(lex.get("OasModule.slot.lastValue"));

                if (savePosValue.hasClass('ao-sp-value-active')){
                    savePosValue.removeClass('ao-sp-value-active');
                    savePosValue.addClass('ao-sp-value-deactivated');
                }
            }
            else {
                savePos.text(lex.get("OasModule.on"));
                value.setValue(true);
                savePosValue.text(that.$component.getSetPointSavePositionValue().getValue());

                if (savePosValue.hasClass('ao-sp-value-deactivated')){
                    savePosValue.removeClass('ao-sp-value-deactivated');
                    savePosValue.addClass('ao-sp-value-active');
                }
            }

            that.$component.setSetPointSavePositionStatus(value);
            that.setModified(true);
        });

        jq.on('click', '.ao-sp-value', function(event) {
            if (jq.find(this).text() !== lex.get("OasModule.slot.lastValue")) {
                FeDialog.action({component: that.$component, slot: 'setSavePositionValue'}).then(function(decision){
                    that.setModified(true);
                });

            }
        });
    };

    AiPointConfig.prototype.updateSwitchStatus = function() {
        var jq = this.jq();
        var switch_status = this.$component.getActiveOnSideSwitchStatus().getValue();
        var ao_switch = jq.find('.ao-switch');
        if (switch_status) {
            ao_switch.removeClass('switch-disabled');
        }
        else {
            ao_switch.addClass('switch-disabled');
        }
    };

    AiPointConfig.prototype.updateSwitchValue = function() {
        var jq = this.jq();
        var switch_value = this.$component.getActiveOnSideSwitchValue().getValue();
        var switch_input = jq.find('div.ao-switch input');

        if (switch_value) {
            switch_input.prop('checked', true);
        }
        else {
            switch_input.prop('checked', false);
        }
    };

    AiPointConfig.prototype.updateSavePosition = function() {
        var jq = this.jq();
        var savepos_status = this.$component.getSetPointSavePositionStatus().getValue();
        var savepos_value = this.$component.getSetPointSavePositionValue().getValue();

        var sp_status = jq.find('.ao-save-pos');
        var sp_value = jq.find('.ao-sp-value');

        if (savepos_status) {
            sp_status.text(lex.get("OasModule.on"));
            sp_value.removeClass('ao-sp-value-deactivated');
            sp_value.addClass('ao-sp-value-active');
            sp_value.text(savepos_value);
        }
        else {
            sp_status.text(lex.get("OasModule.off"));
            sp_value.removeClass('ao-sp-value-active');
            sp_value.addClass('ao-sp-value-deactivated');
            sp_value.text(lex.get("OasModule.slot.lastValue"));
        }
    };


    AiPointConfig.prototype.setModified = function(modified) {
        try {
            this.$configuration.setModified(modified);
        }
        catch(error) {
            // nothing to do
        }
    };

    return AiPointConfig;
});
