define(function(require) {
    'use strict';


    var Widget = require('bajaux/Widget');
    var subscriberMixIn = require('bajaux/mixin/subscriberMixIn');

    var SlotTemplate = require('hbs!nmodule/oasModbusUtilitiesCore/rc/template/aoSlot');

    require('css!nmodule/oasModbusUtilitiesCore/rc/style/aoslot');

    var lexicons = require('lex!oasModbusUtilitiesCore');
    var lex = lexicons[0];

    var AiPointStatus = function(configuration) {

        Widget.apply(this, arguments);
        subscriberMixIn(this);

        this.$configuration = configuration || null;
        this.$component = null;
    };


    AiPointStatus.prototype = Object.create(Widget.prototype);
    AiPointStatus.prototype.constructor = AiPointStatus;


    AiPointStatus.prototype.doInitialize = function(dom) {
        dom.html(SlotTemplate({
            slot_name: "--",
            switch_status: false,
            checked: "checked" ,
            safePos_label: lex.get("OasModule.slot.safePos"),
            slot_savePosStatus: lex.get("OasModule.off"),
            position_label: lex.get("OasModule.slot.position"),
            slot_savePosActive: "deactivated",
            slot_savePosValue: "--"
        }));
    };

    AiPointStatus.prototype.doLoad = function(comp) {

        var that = this;
        var jq = this.jq();

        this.$component = comp;

        jq.find(".ao-slot-header p").text(comp.getName().toUpperCase());


        this.updateSwitchStatus();
        this.updateSwitchValue();
        this.updateSavePosition();


        this.getSubscriber().attach('changed', function(prop) {
            console.log(prop.getName() + " has changed");
            if (prop.getName() === 'activeOnSideSwitchStatus') {
                that.updateSwitchStatus();
            }
            else if (prop.getName() === 'activeOnSideSwitchValue') {
                that.updateSwitchValue();
            }
            else if (prop.getName() === 'activeSavePositionStatus' ||
                     prop.getName() === 'activeSavePositionValue') {
                that.updateSavePosition();
            }
        });
    };

    AiPointStatus.prototype.updateSwitchStatus = function() {
        var jq = this.jq();
        var switch_status = this.$component.getActiveOnSideSwitchStatus().getValue();
        var ao_switch = jq.find('.ao-switch');
        if (switch_status) {
            ao_switch.removeClass('switch-disabled');
        }
        else {
            ao_switch.addClass('switch-disabled');
        }
    };

    AiPointStatus.prototype.updateSwitchValue = function() {
        var jq = this.jq();
        var switch_value = this.$component.getActiveOnSideSwitchValue().getValue();
        var switch_input = jq.find('div.ao-switch input');

        if (switch_value) {
            switch_input.prop('checked', true);
        }
        else {
            switch_input.prop('checked', false);
        }
    };

    AiPointStatus.prototype.updateSavePosition = function() {
        var jq = this.jq();
        var savepos_status = this.$component.getSetPointSavePositionStatus().getValue();
        var savepos_value = this.$component.getSetPointSavePositionValue().getValue();

        var sp_status = jq.find('.ao-save-pos');
        var sp_value = jq.find('.ao-sp-value');

        if (savepos_status) {
            sp_status.text(lex.get("OasModule.on"));
            sp_value.removeClass('ao-sp-value-deactivated');
            sp_value.addClass('ao-sp-value-active');
            sp_value.text(savepos_value);
        }
        else {
            sp_status.text(lex.get("OasModule.off"));
            sp_value.removeClass('ao-sp-value-active');
            sp_value.addClass('ao-sp-value-deactivated');
            sp_value.text(lex.get("OasModule.slot.lastValue"));
        }
    };

    return AiPointStatus;
});
