define(function(require) {
    'use strict';

    var baja = require('baja!');

    var Widget = require('bajaux/Widget');
    var subscriberMixIn = require('bajaux/mixin/subscriberMixIn');

    var SlotTemplate = require('hbs!nmodule/oasModbusUtilitiesCore/rc/template/diSlot');


    require('css!nmodule/oasModbusUtilitiesCore/rc/style/dislot');

    var lexicons = require('lex!oasModbusUtilitiesCore');
    var lex = lexicons[0];

    var DiPointConfig = function(configuration) {

        Widget.apply(this, arguments);
        subscriberMixIn(this);

        this.$all_led_colors = 'led-green led-red led-yellow led-off';

        this.$configuration = configuration || null;
        this.$component = null;
    };


    DiPointConfig.prototype = Object.create(Widget.prototype);
    DiPointConfig.prototype.constructor = DiPointConfig;


    DiPointConfig.prototype.doInitialize = function(dom) {
        dom.html(SlotTemplate({
            slot_name: "--",
            slot_ledColor: 'off',
            checked: "checked" ,
            slot_switchLogic: "NO",
            debounce_label: lex.get("OasModule.slot.debounce"),
        }));
    };

    DiPointConfig.prototype.doLoad = function(comp) {

        var that = this;
        var jq = this.jq();

        this.$component = comp;

        jq.find(".di-slot-header p").text(comp.getName().toUpperCase());

        var led_color = this.$component.getLedColor();
        this.updateLedColor(led_color.toString());

        var switch_logic = this.$component.getSwitchLogic().getValue();
        that.updateSwitchLogic(switch_logic);

        var bounceProtection = this.$component.getBounceProtection().getValue();
        this.updateBounceProtection(bounceProtection);

        this.getSubscriber().attach('changed', function(prop) {

            if (prop.getName() === 'ledColor') {
                that.updateLedColor(prop.$val.toString());
            }
            else if (prop.getName() === 'switchLogic') {
                that.updateSwitchLogic(prop.$val.getValue());
            }
            else if (prop.getName() === 'bounceProtection') {
                that.updateBounceProtection(prop.$val.getValue());
            }
        });

        jq.on('click', '.di-led', function(event) {

            var OasStatusLed = baja.$('oasModbusUtilitiesCore:OasLedColor');

            var value = null;

            var led = jq.find(this);
            if (led.hasClass('led-green')) {
                led.removeClass('led-green');
                led.addClass('led-red');
                value = OasStatusLed.make(0);
            }
            else if (led.hasClass('led-red')) {
                led.removeClass('led-red');
                led.addClass('led-yellow');
                value = OasStatusLed.make(3);
            }
            else if (led.hasClass('led-yellow')) {
                led.removeClass('led-yellow');
                led.addClass('led-off');
                value = OasStatusLed.make(-1);
            }
            else if (led.hasClass('led-off')){
                led.removeClass('led-off');
                led.addClass('led-green');
                value = OasStatusLed.make(1);
            }

            that.$component.setLedColor(value);
            that.setModified(true);
        });

        jq.on('click', 'div.switch-logic p', function(event) {
            var value = baja.$('baja:StatusBoolean');

            var elem = jq.find(this);
            if (elem.text() === 'NC') {
                elem.text('NO');
                value.setValue(false);
            }
            else {
                elem.text('NC');
                value.setValue(true);
            }
            that.$component.setSwitchLogic(value);
            that.setModified(true);
        });

        jq.on('click', '.di-switch', function(event) {
            var value = baja.$('baja:StatusBoolean');
            var elem = jq.find(this);
            var input = elem.children()[0];

            value.setValue(input.checked);

            that.$component.setBounceProtection(value);
            that.setModified(true);
        });


    };

    DiPointConfig.prototype.updateLedColor = function(color) {
        var jq = this.jq();
        var led = jq.find('.di-led');

        led.removeClass(this.$all_led_colors);
        led.addClass("led-" + color.toLowerCase());
    };

    DiPointConfig.prototype.updateSwitchLogic = function(logic) {
        var jq = this.jq();
        var elem = jq.find('div.switch-logic p');

        if (logic) {
            elem.text('NC');
        }
        else {
            elem.text('NO');
        }
    };

    DiPointConfig.prototype.updateBounceProtection = function(bounceProtection) {
        var jq = this.jq();
        var switch_input = jq.find('label.di-switch input');

        if (bounceProtection) {
            console.log('switch checked');
            switch_input.prop('checked', true);
        }
        else {
            console.log('switch not checked');
            switch_input.prop('checked', false);
        }
    };



    DiPointConfig.prototype.setModified = function(modified) {
        try {
            this.$configuration.setModified(modified);
        }
        catch(error) {
            // nothing to do
        }
    };

    return DiPointConfig;
});
