define(function(require) {
    'use strict';

    var Widget = require('bajaux/Widget');
    var subscriberMixIn = require('bajaux/mixin/subscriberMixIn');

    var SlotTemplate = require('hbs!nmodule/oasModbusUtilitiesCore/rc/template/diSlot');


    require('css!nmodule/oasModbusUtilitiesCore/rc/style/dislot');

    var lexicons = require('lex!oasModbusUtilitiesCore');
    var lex = lexicons[0];

    var DiPointStatus = function(configuration) {

        Widget.apply(this, arguments);
        subscriberMixIn(this);

        this.$all_led_colors = 'led-green led-red led-yellow led-off';

        this.$configuration = configuration || null;
        this.$component = null;
    };


    DiPointStatus.prototype = Object.create(Widget.prototype);
    DiPointStatus.prototype.constructor = DiPointStatus;


    DiPointStatus.prototype.doInitialize = function(dom) {
        dom.html(SlotTemplate({
            slot_name: "--",
            slot_ledColor: 'off',
            checked: "checked" ,
            slot_switchLogic: "NO",
            debounce_label: lex.get("OasModule.slot.debounce"),
        }));
    };

    DiPointStatus.prototype.doLoad = function(comp) {

        var that = this;
        var jq = this.jq();

        this.$component = comp;

        jq.find(".di-slot-header p").text(comp.getName().toUpperCase());

        var led_color = this.$component.getLedColor();
        this.updateLedColor(led_color.toString());

        var switch_logic = this.$component.getSwitchLogic().getValue();
        that.updateSwitchLogic(switch_logic);

        var bounceProtection = this.$component.getBounceProtection().getValue();
        this.updateBounceProtection(bounceProtection);

        this.getSubscriber().attach('changed', function(prop) {

            if (prop.getName() === 'ledColor') {
                that.updateLedColor(prop.$val.toString());
            }
            else if (prop.getName() === 'switchLogic') {
                that.updateSwitchLogic(prop.$val.getValue());
            }
            else if (prop.getName() === 'bounceProtection') {
                that.updateBounceProtection(prop.$val.getValue());
            }
        });
    };

    DiPointStatus.prototype.updateLedColor = function(color) {
        var jq = this.jq();
        var led = jq.find('.di-led');

        led.removeClass(this.$all_led_colors);
        led.addClass("led-" + color.toLowerCase());
    };

    DiPointStatus.prototype.updateSwitchLogic = function(logic) {
        var jq = this.jq();
        var elem = jq.find('div.switch-logic p');

        if (logic) {
            elem.text('NC');
        }
        else {
            elem.text('NO');
        }
    };

    DiPointStatus.prototype.updateBounceProtection = function(bounceProtection) {
        var jq = this.jq();
        var switch_input = jq.find('label.di-switch input');

        if (bounceProtection) {
            console.log('switch checked');
            switch_input.prop('checked', true);
        }
        else {
            console.log('switch not checked');
            switch_input.prop('checked', false);
        }
    };

    return DiPointStatus;
});
