define(function(require) {
    'use strict';

    var baja = require('baja!');

    var Widget = require('bajaux/Widget');
    var subscriberMixIn = require('bajaux/mixin/subscriberMixIn');

    var SlotTemplate = require('hbs!nmodule/oasModbusUtilitiesCore/rc/template/doSlot');


    require('css!nmodule/oasModbusUtilitiesCore/rc/style/doslot');

    var lexicons = require('lex!oasModbusUtilitiesCore');
    var lex = lexicons[0];

    var DoPointConfig = function(configuration) {

        Widget.apply(this, arguments);
        subscriberMixIn(this);

        this.$all_led_colors = 'led-green led-red led-yellow led-off';

        this.$configuration = configuration || null;
        this.$component = null;
    };


    DoPointConfig.prototype = Object.create(Widget.prototype);
    DoPointConfig.prototype.constructor = DoPointConfig;


    DoPointConfig.prototype.doInitialize = function(dom) {
        dom.html(SlotTemplate({
            slot_name: "--",
            slot_ledColor: 'off',
            switch_status: false,
            checked: "checked" ,
            safePos_label: lex.get("OasModule.slot.safePos"),
            savePositionStatus: lex.get("OasModule.off"),
            position_label: lex.get("OasModule.slot.position"),
            savePosActive: "deactivated",
            savePosValue: lex.get("OasModule.slot.lastValue")
        }));
    };

    DoPointConfig.prototype.doLoad = function(comp) {

        var that = this;
        var jq = this.jq();

        this.$component = comp;

        jq.find(".do-slot-header p").text(comp.getName().toUpperCase());


        var led_color = this.$component.getLedColor();
        console.log(led_color.toString());
        this.updateLedColor(led_color.toString());

        this.updateSwitchStatus();
        this.updateSwitchValue();
        this.updateSavePosition();


        this.getSubscriber().attach('changed', function(prop) {

            if (prop.getName() === 'ledColor') {
                that.updateLedColor(prop.$val.toString());
            }
            if (prop.getName() === 'activeOnSideSwitchStatus') {
                that.updateSwitchStatus();
            }
            else if (prop.getName() === 'activeOnSideSwitchValue') {
                that.updateSwitchValue();
            }
            else if (prop.getName() === 'savePositionStatus' ||
                     prop.getName() === 'savePositionValue') {
                that.updateSavePosition();
            }

        });

        jq.on('click', '.do-led', function(event) {

            var OasStatusLed = baja.$('oasModbusUtilitiesCore:OasLedColor');

            var value = null;

            var led = jq.find(this);
            if (led.hasClass('led-green')) {
                led.removeClass('led-green');
                led.addClass('led-red');
                value = OasStatusLed.make(0);
            }
            else if (led.hasClass('led-red')) {
                led.removeClass('led-red');
                led.addClass('led-yellow');
                value = OasStatusLed.make(3);
            }
            else if (led.hasClass('led-yellow')) {
                led.removeClass('led-yellow');
                led.addClass('led-off');
                value = OasStatusLed.make(-1);
            }
            else if (led.hasClass('led-off')){
                led.removeClass('led-off');
                led.addClass('led-green');
                value = OasStatusLed.make(1);
            }

            that.$component.setLedColor(value);
            that.setModified(true);
        });

        jq.on('click', '.do-save-pos', function(event) {
            var value = baja.$('baja:StatusBoolean');
            var savePos_status = jq.find(this);
            var savePos_value = jq.find('.do-sp-value');


            if (savePos_status.text() === (lex.get("OasModule.on"))) {
                savePos_status.text(lex.get("OasModule.off"));
                savePos_value.text(lex.get("OasModule.slot.lastValue"));
                if (savePos_value.hasClass('do-sp-value-active')){
                    savePos_value.removeClass('do-sp-value-active');
                    savePos_value.addClass('do-sp-value-deactivated');
                }
                value.setValue(false);
            }
            else {
                savePos_status.text(lex.get("OasModule.on"));
                if (savePos_value.hasClass('do-sp-value-deactivated')){
                    savePos_value.removeClass('do-sp-value-deactivated');
                    savePos_value.addClass('do-sp-value-active');
                }

                if (that.$component.getSavePositionValue().getValue()) {
                    savePos_value.text(lex.get("OasModule.on"));
                }
                else {
                    savePos_value.text(lex.get("OasModule.off"));
                }


                value.setValue(true);
            }
            that.$component.setSavePositionStatus(value);
            that.setModified(true);
        });

        jq.on('click', '.do-sp-value', function(event) {
            var savePosValue = jq.find(this);

            if (savePosValue.hasClass('do-sp-value-active')) {

                var value = baja.$('baja:StatusBoolean');
                if (savePosValue.text() === (lex.get("OasModule.on"))) {
                    savePosValue.text(lex.get("OasModule.off"));
                    value.setValue(false);
                }
                else {
                    savePosValue.text(lex.get("OasModule.on"));
                    value.setValue(true);
                }

                that.$component.setSavePositionValue(value);
                that.setModified(true);
            }
        });

    };

    DoPointConfig.prototype.updateLedColor = function(color) {
        var jq = this.jq();
        var led = jq.find('.do-led');

        led.removeClass(this.$all_led_colors);
        led.addClass("led-" + color.toLowerCase());
    };


    DoPointConfig.prototype.updateSwitchStatus = function() {
        var jq = this.jq();
        var switch_status = this.$component.getActiveOnSideSwitchStatus().getValue();
        var ao_switch = jq.find('.do-switch');
        if (switch_status) {
            ao_switch.removeClass('switch-disabled');
        }
        else {
            ao_switch.addClass('switch-disabled');
        }
    };

    DoPointConfig.prototype.updateSwitchValue = function() {
        var jq = this.jq();
        var switch_value = this.$component.getActiveOnSideSwitchValue().getValue();
        var switch_input = jq.find('label.do-switch input');

        if (switch_value) {
            switch_input.prop('checked', true);
        }
        else {
            switch_input.prop('checked', false);
        }
    };

    DoPointConfig.prototype.updateSavePosition = function() {
        var jq = this.jq();
        var savepos_status = this.$component.getSavePositionStatus().getValue();
        var savepos_value = this.$component.getSavePositionValue().getValue();

        var sp_status = jq.find('.do-save-pos');
        var sp_value = jq.find('.do-sp-value');

        if (savepos_status) {
            sp_status.text(lex.get("OasModule.on"));
            sp_value.removeClass('do-sp-value-deactivated');
            sp_value.addClass('do-sp-value-active');
            if (savepos_value) {
                sp_value.text(lex.get("OasModule.on"));
            }
            else {
                sp_value.text(lex.get("OasModule.off"));
            }
        }
        else {
            sp_status.text(lex.get("OasModule.off"));
            sp_value.removeClass('do-sp-value-active');
            sp_value.addClass('do-sp-value-deactivated');
            sp_value.text(lex.get("OasModule.slot.lastValue"));
        }
    };


    DoPointConfig.prototype.setModified = function(modified) {
        try {
            this.$configuration.setModified(modified);
        }
        catch(error) {
            // nothing to do
        }
    };

    return DoPointConfig;
});
