define(function(require) {
    'use strict';

    var baja = require('baja!');

    var Widget = require('bajaux/Widget');
    var subscriberMixIn = require('bajaux/mixin/subscriberMixIn');

    var SlotTemplate = require('hbs!nmodule/oasModbusUtilitiesCore/rc/template/doSlot');


    require('css!nmodule/oasModbusUtilitiesCore/rc/style/doslot');

    var lexicons = require('lex!oasModbusUtilitiesCore');
    var lex = lexicons[0];

    var DoPointStatus = function(configuration) {

        Widget.apply(this, arguments);
        subscriberMixIn(this);

        this.$all_led_colors = 'led-green led-red led-yellow led-off';

        this.$configuration = configuration || null;
        this.$component = null;
    };


    DoPointStatus.prototype = Object.create(Widget.prototype);
    DoPointStatus.prototype.constructor = DoPointStatus;


    DoPointStatus.prototype.doInitialize = function(dom) {
        dom.html(SlotTemplate({
            slot_name: "--",
            slot_ledColor: 'off',
            switch_status: false,
            checked: "checked" ,
            safePos_label: lex.get("OasModule.slot.safePos"),
            savePositionStatus: lex.get("OasModule.off"),
            position_label: lex.get("OasModule.slot.position"),
            savePosActive: "deactivated",
            savePosValue: lex.get("OasModule.slot.lastValue")
        }));
    };

    DoPointStatus.prototype.doLoad = function(comp) {

        var that = this;
        var jq = this.jq();

        this.$component = comp;

        jq.find(".do-slot-header p").text(comp.getName().toUpperCase());


        var led_color = this.$component.getLedColor();
        console.log(led_color.toString());
        this.updateLedColor(led_color.toString());

        this.updateSwitchStatus();
        this.updateSwitchValue();
        this.updateSavePosition();


        this.getSubscriber().attach('changed', function(prop) {

            if (prop.getName() === 'ledColor') {
                that.updateLedColor(prop.$val.toString());
            }
            if (prop.getName() === 'activeOnSideSwitchStatus') {
                that.updateSwitchStatus();
            }
            else if (prop.getName() === 'activeOnSideSwitchValue') {
                that.updateSwitchValue();
            }
            else if (prop.getName() === 'savePositionStatus' ||
                     prop.getName() === 'savePositionValue') {
                that.updateSavePosition();
            }
        });
    };

    DoPointStatus.prototype.updateLedColor = function(color) {
        var jq = this.jq();
        var led = jq.find('.do-led');

        led.removeClass(this.$all_led_colors);
        led.addClass("led-" + color.toLowerCase());
    };


    DoPointStatus.prototype.updateSwitchStatus = function() {
        var jq = this.jq();
        var switch_status = this.$component.getActiveOnSideSwitchStatus().getValue();
        var ao_switch = jq.find('.do-switch');
        if (switch_status) {
            ao_switch.removeClass('switch-disabled');
        }
        else {
            ao_switch.addClass('switch-disabled');
        }
    };

    DoPointStatus.prototype.updateSwitchValue = function() {
        var jq = this.jq();
        var switch_value = this.$component.getActiveOnSideSwitchValue().getValue();
        var switch_input = jq.find('label.do-switch input');

        if (switch_value) {
            switch_input.prop('checked', true);
        }
        else {
            switch_input.prop('checked', false);
        }
    };

    DoPointStatus.prototype.updateSavePosition = function() {
        var jq = this.jq();
        var savepos_status = this.$component.getSavePositionStatus().getValue();
        var savepos_value = this.$component.getSavePositionValue().getValue();

        var sp_status = jq.find('.do-save-pos');
        var sp_value = jq.find('.do-sp-value');

        if (savepos_status) {
            sp_status.text(lex.get("OasModule.on"));
            sp_value.removeClass('do-sp-value-deactivated');
            sp_value.addClass('do-sp-value-active');
            if (savepos_value) {
                sp_value.text(lex.get("OasModule.on"));
            }
            else {
                sp_value.text(lex.get("OasModule.off"));
            }
        }
        else {
            sp_status.text(lex.get("OasModule.off"));
            sp_value.removeClass('do-sp-value-active');
            sp_value.addClass('do-sp-value-deactivated');
            sp_value.text(lex.get("OasModule.slot.lastValue"));
        }
    };


    DoPointStatus.prototype.setModified = function(modified) {
        try {
            this.$configuration.setModified(modified);
        }
        catch(error) {
            // nothing to do
        }
    };

    return DoPointStatus;
});
